//------------------------------------------------------------------------------ 
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------ 

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Security.Cryptography;
using System.IO;
using System.Text.RegularExpressions;
using System.Diagnostics;

using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using XShan.DotShoppingCart.Common;

namespace InstDSC {
    class InstDSC {
        private const string hostname = "dotshoppingcart";
        private string server, dbname, path, keypath;
        private string keyfile, configfile;
        private bool demo;

        private string emailServer;
        private string EmailServer 
        {
            set { emailServer = value; }
        }

        private string systemEmail;
        private string SystemEmail {
            set { systemEmail = value; }
        }

        private InstDSC(bool demo, string server, string dbname, string path, string keypath) {
            this.demo = demo;
            this.server = server;
            this.dbname = dbname;
            this.path = path;
            this.keypath = keypath;

            if (!Directory.Exists(keypath))
                Directory.CreateDirectory(keypath);
            this.keyfile = keypath + "\\DotshoppingcartMaster.key";
            this.configfile = Utils.GetInstallPath() + "\\..\\web\\web.config";
        }

        static int Main(string[] args) {
            if ((1 == args.Length) && args[0].Equals("/eventlogsource", StringComparison.OrdinalIgnoreCase)) {
                CreateEventLogSource();
                return 0;
            }

            if (args.Length < 4) {
                PrintHelp();
                return 1;
            }
            bool demo = true;
            int index = 0;
            if (args[index].Equals("/nodemo", StringComparison.OrdinalIgnoreCase)) {
                demo = false;
                ++index;
            }

            InstDSC dsc = new InstDSC(demo, args[index], args[++index], args[++index], args[++index]);
            if (args.Length > index + 1)
                dsc.EmailServer = args[++index];

            if (args.Length > index + 1)
                dsc.SystemEmail = args[++index];
            dsc.Install();
            return 0;
        }

        private void Install() {
            CreateDB();
            CreateKey();
            UpdateConfig();
            CreateEventLogSource();
        }

        private void CreateDB() {
            Console.Write("Creating database {0}.{1} at {2} ...", server, dbname, path);
            DBDeployment.CreateDSCDatabase(server, dbname, path);
            Console.WriteLine("done");

            if (demo) {
                string args = string.Format("-E -S {0} -d \"{1}\" -i \"" + Utils.GetInstallPath() + "\\CreateDemoStore.sql\" -b", server, dbname);
                Console.Write("Creating demo data ...");
                RunCmd cmd = new RunCmd();
                cmd.RunProcess("sqlcmd", args);

                File.Copy(Utils.GetInstallPath() + "\\..\\web\\DSC.config.demo", Utils.GetInstallPath() + "\\..\\web\\DSC.config", true);

                Console.WriteLine("done");
            }
        }

        private void CreateKey() {
            Console.Write("Creating encryption key file {0} ...", keyfile);

            RijndaelManaged symmetricAlgo = (RijndaelManaged)RijndaelManaged.Create();
            symmetricAlgo.GenerateKey();
            byte[] encryptedKey = ProtectedData.Protect(symmetricAlgo.Key, null, DataProtectionScope.LocalMachine);
            ProtectedKey key = ProtectedKey.CreateFromEncryptedKey(encryptedKey, DataProtectionScope.LocalMachine);
            KeyReaderWriter writer = new KeyReaderWriter();
            using (FileStream fs = new FileStream(keyfile, FileMode.CreateNew)) {
                writer.Write(fs, key);
            }

            Console.WriteLine("done");
        }

        private void UpdateConfig() {
            Console.Write("Updating {0} ...", configfile);

            string content = File.ReadAllText(configfile);
            Dictionary<string, string> replacements = new Dictionary<string,string>();
            replacements.Add("protectedKeyFilename=\"[^\"]+\"", string.Format("protectedKeyFilename=\"{0}\"", keyfile));
            replacements.Add("\"localhost\"", string.Format("\"{0}\"", emailServer));
            replacements.Add("\"system@nobody.com\"", string.Format("\"{0}\"", systemEmail));
            replacements.Add("connectionString=\"Data Source=localhost;Initial Catalog=dotshoppingcart.public;",
                string.Format("connectionString=\"Data Source={0};Initial Catalog={1};",
                server, dbname));
            replacements.Add("%InstallPath%", Path.GetFullPath(Path.GetDirectoryName(configfile)));

            foreach (string key in replacements.Keys) {
                content = Regex.Replace(content, key, replacements[key], RegexOptions.Singleline | RegexOptions.IgnoreCase);
            }
            File.WriteAllText(configfile, content);

            Console.WriteLine("done");
        }

        private static void CreateEventLogSource() {
            string eventLogSource = "DotShoppingCart";
            if (!EventLog.SourceExists(eventLogSource))
                EventLog.CreateEventSource(eventLogSource, "DotShoppingCart");
        }

        private static void PrintHelp() {
            Console.WriteLine(@"InstDSC.exe [/nodemo] <database server name> <database name> <database data file path> <encryption key path> [<smtp email server>] [<system email address>]");
            Console.WriteLine(@"[/nodemo]                   Don't install demo data. Optinal parameter and default to install demo data.");
            Console.WriteLine(@"<database server name>      The instance name of your database server. Use '.\SQLExpress' for SQL Express");
            Console.WriteLine(@"<database name>             The database name for DotShoppingCart.");
            Console.WriteLine(@"<database data file path>   Where you want database data files to be placed. Use a directory outside web direcotry.");
            Console.WriteLine(@"<encryption key path>       Where you want the encryption key file to be placed. Use a directory outside web direcotry.");
            Console.WriteLine(@"[<smtp server>]             Specify smpt server name if you want DotShoppingCart to send emails. Optinal parameter");
            Console.WriteLine(@"[<system email address>]    The system email address that gets all the exception emails. Optinal parameter");
            Console.WriteLine("\n\nExample:");
            Console.WriteLine(@"InstDSC.exe .\SQLExpress dotshoppingcart c:\dbdata c:\key mail.yourserver.com system@yourdomain.com");
        }
    }
}
