//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Configuration.Provider;
using System.Collections.Specialized;
using System.Net;
using System.IO;
using System.Text;

using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Payment {
    public class AuthorizeNetPaymentProvider : CreditCardPaymentProvider {
        private const string version = "3.1";
        private const string delim_char = "|";

        private string login, transactionKey;
        private bool testTransaction = true;

        private string getPostUrl() {
            return testTransaction ? "https://test.authorize.net/gateway/transact.dll" : "https://secure.authorize.net/gateway/transact.dll";
        }

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "AuthorizeNetPaymentProvider";

            base.Initialize(name, config);

            login = config["login"];
            if (String.IsNullOrEmpty(login))
                throw new ProviderException("Login ID is empty");

            bool.TryParse(config["testTransaction"], out testTransaction);

            transactionKey = config["transactionKey"];
            if (String.IsNullOrEmpty(transactionKey))
                throw new ProviderException("Transaction Key is empty");
        }
        #endregion

        public override string Name {
            get { return "Authorize.Net"; }
        }

        public override PaymentTransaction Authorize(Order order) {
            return AuthorizeOrSale(order, true);
        }

        public override PaymentTransaction Capture(Order order) {
            PaymentTransaction pt = order.GetLastAuthorizationTransaction();
            if (null == pt)
                throw new ApplicationException("Order doesn't have a successful authorization transaction.");
            string[] codes = pt.AuthorizationCode.Split(',');

            WebClient webClient = new WebClient();
            //webClient.Headers.Add("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)");

            NameValueCollection form = GetCommonForm();
            form.Add("x_type", "PRIOR_AUTH_CAPTURE");
            form.Add("x_amount", Math.Round(order.OrderTotalAmount, 2).ToString());
            form.Add("x_trans_id", codes[0]);

            string error = null;
            string reply = null;
            try {
                Byte[] responseData = webClient.UploadValues(getPostUrl(), form);
                reply = Encoding.ASCII.GetString(responseData);
            } catch (Exception e) {
                error = e.Message;
            }

            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.AuthorizeNet;
            trans.PaymentTransactionType = XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Capture;
            trans.Amount = order.OrderTotalAmount;

            if (null != reply) {
                string[] responseFields = reply.Split('|');
                switch (responseFields[0]) {
                    case "1":
                        trans.Response = string.Format("Approved ({0}: {1})", responseFields[2], responseFields[3]);
                        trans.AuthorizationCode = string.Format("{0},{1}", responseFields[6], responseFields[4]);
                        trans.Success = true;
                        break;
                    case "2":
                        trans.Response = string.Format("Declined ({0}: {1})", responseFields[2], responseFields[3]);
                        trans.Success = false;
                        break;
                    case "3":
                        trans.Response = "Error";
                        trans.Success = false;
                        error = reply;
                        break;
                }
                trans.TransactionDate = DateTime.Now;
            } else {
                trans.Success = false;
            }
            trans.Error = error;
            return trans;
        }

        public override PaymentTransaction Sale(Order order) {
            return AuthorizeOrSale(order, false);
        }

        public override PaymentTransaction Refund(Order order, decimal amount) {
            throw new ApplicationException("The method or operation is not implemented.");
        }

        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("testTransaction", "Test Transaction", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("login", "API Login ID", ConfigurationValueType.String));
            configSettings.Add(new ProviderAttribute("transactionKey", "Transaction Key", ConfigurationValueType.Password));
            return configSettings;
        }

        private string ConvertANBool(bool b) {
            return b ? "TRUE" : "FALSE";
        }

        private NameValueCollection GetCommonForm() {
            NameValueCollection form = new NameValueCollection();
            form.Add("x_login", login);
            form.Add("x_tran_key", transactionKey);
            form.Add("x_test_request", ConvertANBool(testTransaction));
            form.Add("x_delim_data", ConvertANBool(true));
            form.Add("x_delim_char", delim_char);
            form.Add("x_encap_char", "");
            form.Add("x_version", version);
            form.Add("x_relay_response", ConvertANBool(false));
            form.Add("x_method", "CC");
            form.Add("x_currency_code", "USD");
            return form;
        }

        private PaymentTransaction AuthorizeOrSale(Order order, bool authorizeOnly) {
            WebClient webClient = new WebClient();
            //webClient.Headers.Add("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)");

            NameValueCollection form = GetCommonForm();
            form.Add("x_type", authorizeOnly ? "AUTH_ONLY" : "AUTH_CAPTURE");
            form.Add("x_amount", Math.Round(order.OrderTotalAmount, 2).ToString());
            form.Add("x_card_num", order.CreditCard.CardNumber);
            form.Add("x_exp_date", order.CreditCard.ExpireMonth + order.CreditCard.ExpireYear);
            form.Add("x_card_code", order.CreditCard.CVV2);

            form.Add("x_first_name", order.BillingAddress.FirstName);
            form.Add("x_last_name", order.BillingAddress.LastName);
            if (string.IsNullOrEmpty(order.BillingAddress.CompanyName))
                form.Add("x_company", order.BillingAddress.CompanyName);
            form.Add("x_address", string.IsNullOrEmpty(order.BillingAddress.Address2) ? 
                order.BillingAddress.Address1 : order.BillingAddress.Address1 + " " + order.BillingAddress.Address2);
            form.Add("x_city", order.BillingAddress.City);
            form.Add("x_state", order.BillingAddress.State);
            form.Add("x_zip", order.BillingAddress.Zipcode);
            form.Add("x_country", order.BillingAddress.Country.CodeISO3166);
            form.Add("x_invoice_num", order.OrderNumber);
            form.Add("x_customer_ip", order.UserIp);

            string error = null;
            string reply = null;
            try {
                Byte[] responseData = webClient.UploadValues(getPostUrl(), form);
                reply = Encoding.ASCII.GetString(responseData);
            } catch (Exception e) {
                error = e.Message;
            }

            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.AuthorizeNet;
            trans.PaymentTransactionType = authorizeOnly ? XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Authorize : XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Sale;
            trans.Amount = order.OrderTotalAmount;

            if (null != reply) {
                string[] responseFields = reply.Split('|');
                switch (responseFields[0]) {
                    case "1":
                        trans.Response = string.Format("Approved ({0}: {1})", responseFields[2], responseFields[3]);
                        trans.AuthorizationCode = string.Format("{0},{1}", responseFields[6], responseFields[4]);
                        trans.AVS = responseFields[5];
                        trans.CVV = responseFields[38];
                        trans.Success = true;
                        break;
                    case "2":
                        trans.Response = string.Format("Declined ({0}: {1})", responseFields[2], responseFields[3]);
                        trans.Success = false;
                        break;
                    case "3":
                        trans.Response = "Error";
                        trans.Success = false;
                        error = reply;
                        break;
                }
                trans.TransactionDate = DateTime.Now;
            } else {
                trans.Success = false;
            }
            trans.Error = error;
            return trans;
        }
    }
}
