//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.IO;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Net;
using System.Web.Services.Protocols;
using CyberSource.Clients;
using CyberSource.Clients.SoapWebReference;

using XShan.DotShoppingCart.Common;
using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Payment {
    public class CybersourcePaymentProvider : CreditCardPaymentProvider {
        private string merchantId, keyDirectory;
        private bool sendToProduction = false;

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "CyberSourcePaymentProvider";

            base.Initialize(name, config);

            merchantId = config["merchantId"];
            if (String.IsNullOrEmpty(merchantId))
                throw new ProviderException("MerchantId is empty");

            bool.TryParse(config["sendToProduction"], out sendToProduction);

            string host = config["host"];
            if (String.IsNullOrEmpty(host))
                throw new ProviderException("host is empty");

            string keyFile = config["keyFile"];
            if (String.IsNullOrEmpty(keyFile))
                throw new ProviderException("CyberSource key file is empty");

            int binaryId;
            if (int.TryParse(keyFile, out binaryId)) {
                string keyFileSpec = new Binary().GetFile(host, binaryId);
                keyDirectory = Path.GetDirectoryName(keyFileSpec);
            } else {
                throw new ProviderException("Cannot get CyberSource key file from DB");
            }
        }
        #endregion

        public override string Name {
            get { return "CyberSource"; }
        }

        public override PaymentTransaction Authorize(Order order) {
            return AuthorizeOrSale(order, true);
        }

        public override PaymentTransaction Capture(Order order) {
            PaymentTransaction pt = order.GetLastAuthorizationTransaction();
            if (null == pt)
                throw new ApplicationException("Order doesn't have a successful authorization transaction.");

            RequestMessage request = new RequestMessage();
            request.ccCaptureService = new CCCaptureService();
            request.ccCaptureService.run = "true";
            string[] codes = pt.AuthorizationCode.Split(',');
            request.ccCaptureService.authRequestID = codes[0];

            // add required fields
            request.merchantID = merchantId;
            request.merchantReferenceCode = order.OrderNumber;
            request.purchaseTotals = GetPurchaseTotal(order.OrderTotalAmount);

            string error = null;
            ReplyMessage reply = null;
            try {
                reply = SoapClient.RunTransaction(GetCyberSourceConfig(), request);
            } catch (SignException se) {
                error = HandleSignException(se);
            } catch (SoapHeaderException she) {
                error = HandleSoapHeaderException(she);
            } catch (SoapBodyException sbe) {
                error = HandleSoapBodyException(sbe);
            } catch (WebException we) {
                error = HandleWebException(we);
            }

            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.CyberSource;
            trans.PaymentTransactionType = XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Capture;

            if (null != reply) {
                trans.Response = string.Format("{0} ({1})", reply.decision, GetDetailReason(reply));
                decimal amount = 0;
                DateTime time = DateTime.Now;
                if (null != reply.ccCaptureReply) {
                    trans.AuthorizationCode = string.Format("{0},{1}", reply.requestID, reply.ccCaptureReply.reconciliationID);
                    decimal.TryParse(reply.ccCaptureReply.amount, out amount);
                    DateTime.TryParse(reply.ccCaptureReply.requestDateTime, out time);
                } else {
                    trans.AuthorizationCode = reply.requestID;
                }
                trans.Amount = amount;
                if (!Utils.IsValidateDBTime(time)) {
                    time = DateTime.Now;
                }
                trans.TransactionDate = time;
                trans.Success = CheckSuccess(reply.decision);
            } else {
                trans.Success = false;
            }
            trans.Error = error;
            return trans;
        }

        public override PaymentTransaction Sale(Order order) {
            return AuthorizeOrSale(order, false);
        }

        public override PaymentTransaction Refund(Order order, decimal amount) {
            throw new ApplicationException("The method or operation is not implemented.");
        }

        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("sendToProduction", "Send To Production Site", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("merchantId", "MerchantID", ConfigurationValueType.String));
            configSettings.Add(new ProviderAttribute("keyFile", "Key File", ConfigurationValueType.File));
            return configSettings;
        }

        private PaymentTransaction AuthorizeOrSale(Order order, bool authorizeOnly) {
            RequestMessage request = new RequestMessage();
            // Credit Card Authorization
            request.ccAuthService = new CCAuthService();
            request.ccAuthService.run = "true";

            // Credit Card Capture
            if (!authorizeOnly) {
                request.ccCaptureService = new CCCaptureService();
                request.ccCaptureService.run = "true";
            }

            // add required fields
            request.merchantID = merchantId;
            request.merchantReferenceCode = order.OrderNumber;

            BillTo billTo = new BillTo();
            request.billTo = billTo;
            billTo.firstName = order.BillingAddress.FirstName;
            billTo.lastName = order.BillingAddress.LastName;
            if (!string.IsNullOrEmpty(order.BillingAddress.CompanyName))
                billTo.company = order.BillingAddress.CompanyName;
            billTo.street1 = order.BillingAddress.Address1;
            if (!string.IsNullOrEmpty(order.BillingAddress.Address2))
                billTo.street2 = order.BillingAddress.Address2;
            billTo.city = order.BillingAddress.City;
            billTo.state = XShan.DotShoppingCart.Common.Object.Address.GetStandardStateName(order.BillingAddress);
            billTo.postalCode = order.BillingAddress.Zipcode;
            billTo.country = order.BillingAddress.Country.CodeISO3166;
            billTo.email = order.Email;
            billTo.ipAddress = order.UserIp;

            Card card = new Card();
            request.card = card;
            card.accountNumber = order.CreditCard.CardNumber;
            card.expirationMonth = order.CreditCard.ExpireMonth;
            card.expirationYear = order.CreditCard.ExpireYear;
            string cardType = ToCardType(order.CreditCard.CreditCardType.CreditCardTypeId);
            if (!string.IsNullOrEmpty(cardType))
                card.cardType = cardType;
            card.cvNumber = order.CreditCard.CVV2;

            request.purchaseTotals = GetPurchaseTotal(order.OrderTotalAmount);

            string error = null;
            ReplyMessage reply = null;
            try {
                reply = SoapClient.RunTransaction(GetCyberSourceConfig(), request);
            } catch (SignException se) {
                error = HandleSignException(se);
            } catch (SoapHeaderException she) {
                error = HandleSoapHeaderException(she);
            } catch (SoapBodyException sbe) {
                error = HandleSoapBodyException(sbe);
            } catch (WebException we) {
                error = HandleWebException(we);
            }

            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.CyberSource;
            trans.PaymentTransactionType = authorizeOnly ? XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Authorize : XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Sale;
            trans.Amount = order.OrderTotalAmount;

            if (null != reply) {
                trans.Response = reply.decision;
                DateTime time = DateTime.Now;
                if (null != reply.ccAuthReply) {
                    trans.AuthorizationCode = string.Format("{0},{1}", reply.requestID, reply.ccAuthReply.authorizationCode);
                    trans.AVS = reply.ccAuthReply.avsCode;
                    trans.CVV = reply.ccAuthReply.cvCode;
                    DateTime.TryParse(reply.ccAuthReply.authorizedDateTime, out time);
                } else {
                    trans.AuthorizationCode = reply.requestID;
                }
                if (!Utils.IsValidateDBTime(time)) {
                    time = DateTime.Now;
                }
                trans.TransactionDate = time;
                trans.Success = CheckSuccess(reply.decision);
                if (!trans.Success)
                    error = GetDetailReason(reply);
            } else {
                trans.Success = false;
            }
            trans.Error = error;
            return trans;
        }

        private bool CheckSuccess(string decision) {
            return (string.Compare("ACCEPT", decision, true) == 0) || (string.Compare("REVIEW", decision, true) == 0);
        }

        private CyberSource.Clients.Configuration GetCyberSourceConfig() {
            CyberSource.Clients.Configuration config = new Configuration();
            config.KeysDirectory = keyDirectory;
            config.MerchantID = merchantId;
            config.SendToProduction = sendToProduction;
            return config;
        }

        private PurchaseTotals GetPurchaseTotal(decimal amount) {
            PurchaseTotals purchaseTotals = new PurchaseTotals();
            purchaseTotals.currency = "USD";
            purchaseTotals.grandTotalAmount = Math.Round(amount, 2).ToString();
            return purchaseTotals;
        }

        private string ToCardType(int creditCardTypeId) {
            switch (creditCardTypeId) {
                case 1: //visa
                    return "001";
                case 2: //MC
                    return "002";
                case 3: //American Express
                    return "003";
                case 4: //Discover
                    return "004";
            }
            return null;
        }

        private static string GetDetailReason(ReplyMessage reply) {
            switch (reply.reasonCode) {
                // Success
                case "100":
                    return reply.reasonCode;

                // Missing field(s)
                case "101":
                    return ("The following required field(s) are missing: " + EnumerateValues(reply.missingField));

                // Invalid field(s)
                case "102":
                    return ("The following field(s) are invalid: " + EnumerateValues(reply.invalidField));

                case "202":
                    return ("Expired card. Reason code: 202");

                case "203":
                    return ("General decline of the card. Reason code: 203");

                case "204":
                    return ("Insufficient funds in the account. Reason code: 204");

                case "211":
                    return ("Invalid card verification number. Reason code: 211");

                case "231":
                    return ("Invalid account number. Reason code: 231");


                // add additional reason codes here that you need to handle
                // specifically.

                default:
                    // For all other reason codes, return an empty string,
                    // in which case, the template will be displayed with no
                    // specific content.
                    return (string.Format("Reason code: {0}", reply.reasonCode));
            }
        }

        private static string HandleSignException(SignException se) {
            return String.Format("Failed to sign the request with error code '{0}' and message '{1}'.", se.ErrorCode, se.Message);
        }

        private static string HandleSoapHeaderException(SoapHeaderException she) {
            return String.Format("A SOAP header exception was returned with fault code '{0}' and message '{1}'.", she.Code, she.Message);
        }

        private static string HandleSoapBodyException(SoapBodyException sbe) {
            string error = String.Format("A SOAP body exception was returned with fault code '{0}' and message '{1}'.", sbe.Code, sbe.Message);

            if (sbe.Code.Namespace.Equals(SoapClient.CYBS_NAMESPACE) &&
                sbe.Code.Name.Equals("CriticalServerError")) {
                /* The transaction may have been completed by CyberSource.
                 * If your request included a payment service, you should
                 * notify the appropriate department in your company (e.g. by
                 * sending an email) so that they can confirm if the request
                 * did in fact complete by searching the CyberSource Support
                 * Screens using the request id.
                 * 
                 * The line below demonstrates how to retrieve the request id.
                 */
                error += ". RequestID: " + sbe.RequestID.ToString();
            }
            return error;
        }

        private static string HandleWebException(WebException we) {
            string error = String.Format("Failed to get a response with status '{0}' and message '{1}'", we.Status, we.Message);
            if (IsCriticalError(we)) {
                /*Todo:
                 * The transaction may have been completed by CyberSource.
                 * If your request included a payment service, you should
                 * notify the appropriate department in your company (e.g. by
                 * sending an email) so that they can confirm if the request
                 * did in fact complete by searching the CyberSource Support
                 * Screens using the value of the merchantReferenceCode in
                 * your request.
                 */
            }
            return error;
        }

        private static string EnumerateValues(string[] array) {
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            foreach (string val in array) {
                sb.Append(val + "\n");
            }

            return (sb.ToString());
        }

        private static bool IsCriticalError(WebException we) {
            switch (we.Status) {
                case WebExceptionStatus.ProtocolError:
                    if (we.Response != null) {
                        HttpWebResponse response
                            = (HttpWebResponse)we.Response;

                        // GatewayTimeout may be returned if you are
                        // connecting through a proxy server.
                        return (response.StatusCode ==
                            HttpStatusCode.GatewayTimeout);
                    }

                    // In case of ProtocolError, the Response property
                    // should always be present.  In the unlikely case 
                    // that it is not, we assume something went wrong
                    // along the way and to be safe, treat it as a
                    // critical error.
                    return (true);

                case WebExceptionStatus.ConnectFailure:
                case WebExceptionStatus.NameResolutionFailure:
                case WebExceptionStatus.ProxyNameResolutionFailure:
                case WebExceptionStatus.SendFailure:
                    return (false);

                default:
                    return (true);
            }
        }
    }
}
