//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Configuration;
using System.Configuration.Provider;
using System.Web.Configuration;
using System.Xml;
using GCheckout.Checkout;
using GCheckout.Util;
using GCheckout.OrderProcessing;
using XShan.DotShoppingCart.Tax;
using XShan.DotShoppingCart.Shipping;
using XShan.DotShoppingCart.Common;
using DSC = XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Payment {
    public class GooglePaymentProvider : DSCProvider {
        private bool useSandBox = true;
        private bool enabled = false;
        private string merchantId, merchantKey;


        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "GoogleCheckoutPaymentProvider";

            base.Initialize(name, config);

            bool.TryParse(config["enabled"], out enabled);
            bool.TryParse(config["useSandBox"], out useSandBox);

            merchantId = config["merchantId"];
            if (enabled && String.IsNullOrEmpty(merchantId))
                throw new ProviderException("MerchantId is empty");

            merchantKey = config["merchantKey"];
            if (enabled && String.IsNullOrEmpty(merchantKey))
                throw new ProviderException("merchantKey is empty");
        }
        #endregion

        public override string Name {
            get { return "Google Check"; }
        }

        public bool Enabled {
            get { return enabled; }
        }

        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("enabled", "Enabled", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("useSandBox", "Use Sandbox", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("merchantId", "Merchant ID", ConfigurationValueType.String));
            configSettings.Add(new ProviderAttribute("merchantKey", "Merchant Key", ConfigurationValueType.Password));
            return configSettings;
        }

        public static GooglePaymentProvider GetProvider() {
            return GetProvider(SiteNavigation.GetHost());
        }

        public static GooglePaymentProvider GetProvider(string host) {
            GooglePaymentServiceConfigurationSection pcs = (GooglePaymentServiceConfigurationSection)SqlConfigurationController.GetSection(host,
                GooglePaymentServiceConfigurationSection.SectionName);
            if (null == pcs)
                return null;
            ProviderSettings ps = pcs.Providers[pcs.DefaultProvider];
            //Initalize host which is not defined in the provider settings. The reason of not putting it in provider settings is to avoid data duplication.
            ps.Parameters["host"] = host;
            GooglePaymentProvider provider = (GooglePaymentProvider)ProvidersHelper.InstantiateProvider(ps,
                typeof(GooglePaymentProvider));
            return provider;
        }

        public string PostCart(DSC.Cart cart, string userName) {
            CheckoutShoppingCartRequest req = new CheckoutShoppingCartRequest(merchantId, merchantKey,
                useSandBox ? GCheckout.EnvironmentType.Sandbox : GCheckout.EnvironmentType.Production, "USD", 10);

            XmlDocument tempDoc = new XmlDocument();
            XmlNode tempNode;
            foreach (DSC.CartItem item in cart.CartItems) {
                XmlNode skuNode = tempDoc.CreateElement("sku");
                    skuNode.InnerText = item.Product.SKU;
                    tempNode = tempDoc.CreateElement("attributeIds");
                    tempNode.InnerText = item.AttributeValueIds;
                    req.AddItem(item.Product.ProductName + " " + item.AttributeSelectionsText, item.Product.ShortDescription, item.Product.FinalPrice,
                        item.Quantity, skuNode, tempNode);
            }

            req.MerchantCalculatedTax = true;
            req.AddCountryTaxRule(GCheckout.AutoGen.USAreas.ALL, 0, true);
            ShippingRestrictions restrictions = new ShippingRestrictions();
            restrictions.AddAllowedWorldArea();
            foreach (DSC.ShippingMethod method in ShippingService.GetAllShippingMethods(cart)) {
                req.AddMerchantCalculatedShippingMethod(method.Name, method.Price, restrictions, restrictions);
            }
            req.MerchantCalculationsUrl = SiteNavigation.GoogleCheckoutMerchantCalculationHandler;

            //add the username
            tempNode = tempDoc.CreateElement("username");
            tempNode.InnerText = userName;
            req.AddMerchantPrivateDataNode(tempNode);

            //setup URLs
            req.ContinueShoppingUrl = SiteNavigation.HomePage;
            req.EditCartUrl = SiteNavigation.CartPage;

            GCheckoutResponse resp = req.Send();
            if (resp.IsGood) {
                return resp.RedirectUrl;
            } else {
                PaymentExecption ex = new PaymentExecption("Goolge Checkout exception");
                ex.Data.Add("ResponseXml", resp.ResponseXml);
                ex.Data.Add("RedirectUrl", resp.RedirectUrl);
                ex.Data.Add("IsGood", resp.IsGood);
                ex.Data.Add("ErrorMessage", resp.ErrorMessage);
                throw ex;
            }
        }

        public DSC.PaymentTransaction Capture(DSC.Order order) {
            DSC.PaymentTransaction pt = order.GetLastAuthorizationTransaction();
            if (null == pt)
                throw new ApplicationException("Order doesn't have a successful authorization transaction.");
            GCheckout.EnvironmentType envType = useSandBox ? GCheckout.EnvironmentType.Sandbox : GCheckout.EnvironmentType.Production;
            ChargeOrderRequest req = new ChargeOrderRequest(merchantId, merchantKey, envType.ToString(), order.CustomField);
            GCheckoutResponse resp = req.Send();
            DSC.PaymentTransaction trans = new DSC.PaymentTransaction();
            trans.PaymentGatewayType = DSC.PaymentGatewayType.Google_Checkout;
            trans.PaymentTransactionType = DSC.PaymentTransactionType.Capture;
            trans.Amount = order.OrderTotalAmount;
            trans.TransactionDate = DateTime.Now;
            trans.Response = resp.ResponseXml;
            trans.Success = resp.IsGood;
            trans.AuthorizationCode = resp.SerialNumber;
            trans.Error = resp.ErrorMessage;
            return trans;
        }

        public bool Ship(DSC.Order order) {
            GCheckout.EnvironmentType envType = useSandBox ? GCheckout.EnvironmentType.Sandbox : GCheckout.EnvironmentType.Production;
            if (null != order.ShipTrackings) {
                foreach (DSC.ShipTracking tracking in order.ShipTrackings) {
                    AddTrackingDataRequest trackingReq = new AddTrackingDataRequest(merchantId, merchantKey, envType.ToString(), 
                        order.CustomField, tracking.Carrier.Name, tracking.TrackingNumber);
                    trackingReq.Send();
                }
            }

            DeliverOrderRequest req = new DeliverOrderRequest(merchantId, merchantKey, envType.ToString(), order.CustomField, false);
            GCheckoutResponse resp = req.Send();
            return resp.IsGood;
        }

        public bool Archive(DSC.Order order) {
            GCheckout.EnvironmentType envType = useSandBox ? GCheckout.EnvironmentType.Sandbox : GCheckout.EnvironmentType.Production;
            ArchiveOrderRequest req = new ArchiveOrderRequest(merchantId, merchantKey, envType.ToString(), order.CustomField);
            GCheckoutResponse resp = req.Send();
            return resp.IsGood;
        }
    }
}
