//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Text;
using System.IO;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Security.Cryptography.X509Certificates;

using XShan.DotShoppingCart.Payment.PayPalSvc;
using XShan.DotShoppingCart.Common.Object;

namespace XShan.DotShoppingCart.Payment {
    public static class PaymentUtils {
        public const string APIVersion = "2.0";

        public static void InitializePaypalPaymentProvider(bool enabled, NameValueCollection config, out PayPalAPISoapBinding service1, out PayPalAPIAASoapBinding service2) {
            string userName = config["userName"];
            if (enabled && String.IsNullOrEmpty(userName))
                throw new ProviderException("Paypal Service Username is empty");

            string userPassword = config["userPassword"];
            if (enabled && String.IsNullOrEmpty(userPassword))
                throw new ProviderException("Paypal Service Password is empty");

            string host = config["host"];
            if (enabled && String.IsNullOrEmpty(host))
                throw new ProviderException("host is empty");

            string certFile = config["certFile"];
            if (enabled && String.IsNullOrEmpty(certFile))
                throw new ProviderException("Paypal Service certificate file is empty");

            int binaryId;
            if (int.TryParse(certFile, out binaryId)) {
                certFile = new Binary().GetFile(host, binaryId);
            }

            string certPassword = config["certPassword"];
            if (enabled && String.IsNullOrEmpty(certPassword))
                throw new ProviderException("Paypal Service certificate password is empty");

            bool useSandBox = true;
            bool.TryParse(config["useSandBox"], out useSandBox);

            service1 = new PayPalAPISoapBinding();
            service2 = new PayPalAPIAASoapBinding();

            if (!useSandBox) {
                service2.Url = service1.Url = "https://api.paypal.com/2.0/";
            }
            service1.RequesterCredentials = new CustomSecurityHeaderType();
            service1.RequesterCredentials.Credentials = new UserIdPasswordType();
            service1.RequesterCredentials.Credentials.Username = userName;
            service1.RequesterCredentials.Credentials.Password = userPassword;
            service1.RequesterCredentials.Credentials.Subject = "";

            service2.RequesterCredentials = new CustomSecurityHeaderType();
            service2.RequesterCredentials.Credentials = new UserIdPasswordType();
            service2.RequesterCredentials.Credentials.Username = userName;
            service2.RequesterCredentials.Credentials.Password = userPassword;
            service2.RequesterCredentials.Credentials.Subject = "";

            byte[] buffer = File.ReadAllBytes(certFile);
            X509Certificate2 cert = new X509Certificate2(buffer, certPassword, X509KeyStorageFlags.MachineKeySet);
            service1.ClientCertificates.Add(cert);
            service2.ClientCertificates.Add(cert);
        }

        public static CountryCodeType ToPaypalContryCodeType(string country) {
            //Todo: need to populate this
            return CountryCodeType.US;
        }

        public static bool CheckSuccess(AbstractResponseType abstractResponse, out string errorMsg) {
            bool success = false;
            StringBuilder sb = new StringBuilder();
            switch (abstractResponse.Ack) {
                case AckCodeType.Success:
                case AckCodeType.SuccessWithWarning:
                    success = true;
                    break;
                default:
                    break;
            }
            if (null != abstractResponse.Errors) {
                foreach (ErrorType errorType in abstractResponse.Errors) {
                    if (sb.Length <= 0) {
                        sb.Append(Environment.NewLine);
                    }
                    sb.Append("LongMessage: ").Append(errorType.LongMessage).Append(Environment.NewLine);
                    sb.Append("ShortMessage: ").Append(errorType.ShortMessage).Append(Environment.NewLine);
                    sb.Append("ErrorCode: ").Append(errorType.ErrorCode).Append(Environment.NewLine);
                }
            }
            errorMsg = sb.ToString();
            return success;
        }

        public static PaymentTransaction Capture(PayPalAPIAASoapBinding service2, Order order) {
            PaymentTransaction pt = order.GetLastAuthorizationTransaction();
            if (null == pt)
                throw new ApplicationException("Order doesn't have a successful authorization transaction.");

            DoCaptureReq req = new DoCaptureReq();
            req.DoCaptureRequest = new DoCaptureRequestType();
            req.DoCaptureRequest.Version = APIVersion;
            req.DoCaptureRequest.AuthorizationID = pt.AuthorizationCode;
            req.DoCaptureRequest.Amount = new BasicAmountType();
            req.DoCaptureRequest.Amount.Value = Math.Round(order.OrderTotalAmount, 2).ToString();
            req.DoCaptureRequest.Amount.currencyID = CurrencyCodeType.USD;
            req.DoCaptureRequest.CompleteType = CompleteCodeType.Complete;
            DoCaptureResponseType response = service2.DoCapture(req);
            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.Paypal_DirectPay;
            trans.PaymentTransactionType = XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Capture;
            trans.Amount = order.OrderTotalAmount;
            trans.AuthorizationCode = response.DoCaptureResponseDetails.PaymentInfo.TransactionID;
            trans.Response = response.Ack.ToString();
            trans.TransactionDate = response.Timestamp;
            string error;
            trans.Success = PaymentUtils.CheckSuccess(response, out error);
            trans.Error = error;
            return trans;
        }

    }
}
