//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration.Provider;

using XShan.DotShoppingCart.Payment.PayPalSvc;
using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Payment {
    public class PaypalDirectPaymentProvider : CreditCardPaymentProvider {
        private PayPalAPISoapBinding service1;
        private PayPalAPIAASoapBinding service2;

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "PaypalDirectPaymentProvider";

            base.Initialize(name, config);

            PaymentUtils.InitializePaypalPaymentProvider(true, config, out service1, out service2);
        }
        #endregion

        public override string Name {
            get { return "PayPal PaymentsPro DirectPay"; }
        }

        public override PaymentTransaction Authorize(Order order) {
            return AuthorizeOrSale(order, true);
        }

        public override PaymentTransaction Capture(Order order) {
            return PaymentUtils.Capture(service2, order);
        }

        public override PaymentTransaction Sale(Order order) {
            return AuthorizeOrSale(order, false);
        }

        public override PaymentTransaction Refund(Order order, decimal amount) {
            PaymentTransaction pt = order.GetLastCaptureOrSaleTransaction();
            if (null == pt)
                throw new ApplicationException("Order doesn't have a capture or sale transaction.");

            RefundTransactionReq req = new RefundTransactionReq();
            req.RefundTransactionRequest = new RefundTransactionRequestType();
            req.RefundTransactionRequest.Version = PaymentUtils.APIVersion;
            req.RefundTransactionRequest.TransactionID = pt.AuthorizationCode;
            if (0 == amount) {
                req.RefundTransactionRequest.RefundType = RefundType.Full;
                req.RefundTransactionRequest.RefundTypeSpecified = true;
            } else {
                req.RefundTransactionRequest.RefundType = RefundType.Partial;
                req.RefundTransactionRequest.RefundTypeSpecified = true;
                req.RefundTransactionRequest.Amount = new BasicAmountType();
                req.RefundTransactionRequest.Amount.Value = Math.Round(amount, 2).ToString();
                req.RefundTransactionRequest.Amount.currencyID = CurrencyCodeType.USD;
            }

            RefundTransactionResponseType response = service1.RefundTransaction(req);
            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.Paypal_DirectPay;
            trans.PaymentTransactionType = XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Refund;
            trans.Amount = 0;
            decimal refundAmount;
            if ((null != response.GrossRefundAmount) && 
                decimal.TryParse(response.GrossRefundAmount.Value, out refundAmount))
                trans.Amount = refundAmount;
            trans.AuthorizationCode = response.RefundTransactionID;
            trans.Response = response.Ack.ToString();
            trans.TransactionDate = response.Timestamp;
            string error;
            trans.Success = PaymentUtils.CheckSuccess(response, out error);
            trans.Error = error;
            return trans;
        }

        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("useSandBox", "Use Sandbox", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("userName", "API Account Name", ConfigurationValueType.String));
            configSettings.Add(new ProviderAttribute("userPassword", "API Account Password", ConfigurationValueType.Password));
            configSettings.Add(new ProviderAttribute("certFile", "Certificate", ConfigurationValueType.File));
            configSettings.Add(new ProviderAttribute("certPassword", "Certificate Password", ConfigurationValueType.Password));
            return configSettings;
        }

        #region Helper Functions
        private PaymentTransaction AuthorizeOrSale(Order order, bool authorize) {
            DoDirectPaymentReq req = new DoDirectPaymentReq();
            req.DoDirectPaymentRequest = new DoDirectPaymentRequestType();
            req.DoDirectPaymentRequest.Version = PaymentUtils.APIVersion;
            DoDirectPaymentRequestDetailsType details = new DoDirectPaymentRequestDetailsType();
            req.DoDirectPaymentRequest.DoDirectPaymentRequestDetails = details;

            details.IPAddress = order.UserIp;

            if (authorize) {
                details.PaymentAction = PaymentActionCodeType.Authorization;
            } else {
                details.PaymentAction = PaymentActionCodeType.Sale;
            }

            //credit card info
            details.CreditCard = new CreditCardDetailsType();
            details.CreditCard.CreditCardNumber = order.CreditCard.CardNumber;
            details.CreditCard.CreditCardType = (CreditCardTypeType)Enum.Parse(typeof(CreditCardTypeType), order.CreditCard.CreditCardType.ShortType);
            details.CreditCard.ExpMonth = int.Parse(order.CreditCard.ExpireMonth);
            details.CreditCard.ExpYear = int.Parse(order.CreditCard.ExpireYear);
            details.CreditCard.CVV2 = order.CreditCard.CVV2;

            //card owner
            details.CreditCard.CardOwner = new PayerInfoType();
            details.CreditCard.CardOwner.PayerCountry = PaymentUtils.ToPaypalContryCodeType(order.BillingAddress.Country.Name);

            //card owner address
            details.CreditCard.CardOwner.Address = new AddressType();
            details.CreditCard.CardOwner.Address.CountrySpecified = true;
            details.CreditCard.CardOwner.Address.Street1 = order.BillingAddress.Address1;
            details.CreditCard.CardOwner.Address.Street2 = order.BillingAddress.Address2;
            details.CreditCard.CardOwner.Address.CityName = order.BillingAddress.City;
            details.CreditCard.CardOwner.Address.StateOrProvince = order.BillingAddress.State;
            details.CreditCard.CardOwner.Address.Country = PaymentUtils.ToPaypalContryCodeType(order.BillingAddress.Country.Name);
            details.CreditCard.CardOwner.Address.PostalCode = order.BillingAddress.Zipcode;

            //card owner name
            details.CreditCard.CardOwner.PayerName = new PersonNameType();
            details.CreditCard.CardOwner.PayerName.FirstName = order.BillingAddress.FirstName;
            details.CreditCard.CardOwner.PayerName.LastName = order.BillingAddress.LastName;

            //payment detail
            details.PaymentDetails = new PaymentDetailsType();
            details.PaymentDetails.OrderTotal = new BasicAmountType();
            details.PaymentDetails.OrderTotal.Value = Math.Round(order.OrderTotalAmount, 2).ToString();
            // NOTE: The only currency supported by the Direct Payment API at this time is US dollars (USD).
            details.PaymentDetails.OrderTotal.currencyID = CurrencyCodeType.USD;
            details.PaymentDetails.Custom = order.OrderNumber;
            details.PaymentDetails.ButtonSource = "DotShoppingCart";

            //send the request
            DoDirectPaymentResponseType response = service2.DoDirectPayment(req);
            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.Paypal_DirectPay;
            trans.PaymentTransactionType = authorize ? XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Authorize : XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Sale;
            trans.Amount = order.OrderTotalAmount;
            trans.AuthorizationCode = response.TransactionID;
            trans.Response = response.Ack.ToString();
            trans.AVS = response.AVSCode;
            trans.CVV = response.CVV2Code;
            trans.TransactionDate = response.Timestamp;
            string error;
            trans.Success = PaymentUtils.CheckSuccess(response, out error);
            trans.Error = error;
            return trans;
        }

        #endregion
    }
}
