//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Configuration.Provider;
using System.Web.Configuration;

using XShan.DotShoppingCart.Common;
using XShan.DotShoppingCart.Payment.PayPalSvc;
using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Payment {
    public class PaypalExpressPaymentProvider : DSCProvider {
        private PayPalAPISoapBinding service1;
        private PayPalAPIAASoapBinding service2;
        private bool useSandBox = true;
        private bool enabled = false;

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "PaypalExpressCheckoutPaymentProvider";

            base.Initialize(name, config);

            bool.TryParse(config["enabled"], out enabled);
            bool.TryParse(config["useSandBox"], out useSandBox);
            PaymentUtils.InitializePaypalPaymentProvider(enabled, config, out service1, out service2);
        }
        #endregion

        public override string Name {
            get { return "PayPal Express Checkout"; }
        }

        public bool Enabled {
            get { return enabled; }
        }

        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("enabled", "Enabled", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("useSandBox", "Use Sandbox", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("userName", "API Account Name", ConfigurationValueType.String));
            configSettings.Add(new ProviderAttribute("userPassword", "API Account Password", ConfigurationValueType.Password));
            configSettings.Add(new ProviderAttribute("certFile", "Certificate", ConfigurationValueType.File));
            configSettings.Add(new ProviderAttribute("certPassword", "Certificate Password", ConfigurationValueType.Password));
            return configSettings;
        }

        public static PaypalExpressPaymentProvider GetProvider() {
            return GetProvider(SiteNavigation.GetHost());
        }

        public static PaypalExpressPaymentProvider GetProvider(string host) {
            PaypalExpressPaymentServiceConfigurationSection pcs = (PaypalExpressPaymentServiceConfigurationSection)SqlConfigurationController.GetSection(host, 
                PaypalExpressPaymentServiceConfigurationSection.SectionName);
            if (null == pcs)
                return null;
            ProviderSettings ps = pcs.Providers[pcs.DefaultProvider];
            //Initalize host which is not defined in the provider settings. The reason of not putting it in provider settings is to avoid data duplication.
            ps.Parameters["host"] = host;
            PaypalExpressPaymentProvider provider = (PaypalExpressPaymentProvider)ProvidersHelper.InstantiateProvider(ps, 
                typeof(PaypalExpressPaymentProvider));
            return provider;
        }

        public PaymentTransaction Capture(Order order) {
            return PaymentUtils.Capture(service2, order);
        }

        public string SetExpressCheckout(string email, decimal orderTotal, string returnUrl, string cancelUrl, bool AuthOnly) {
            SetExpressCheckoutReq req = new SetExpressCheckoutReq();
            req.SetExpressCheckoutRequest = new SetExpressCheckoutRequestType();
            req.SetExpressCheckoutRequest.Version = PaymentUtils.APIVersion;
            SetExpressCheckoutRequestDetailsType details = new SetExpressCheckoutRequestDetailsType();
            req.SetExpressCheckoutRequest.SetExpressCheckoutRequestDetails = details;

            details.BuyerEmail = email;
            details.PaymentAction = AuthOnly ? PaymentActionCodeType.Authorization : PaymentActionCodeType.Sale;
            details.PaymentActionSpecified = true;

            details.OrderTotal = new BasicAmountType();
            details.OrderTotal.Value = Math.Round(orderTotal, 2).ToString();  
            details.OrderTotal.currencyID = CurrencyCodeType.USD;

            details.ReturnURL = returnUrl;
            details.CancelURL = cancelUrl;

            SetExpressCheckoutResponseType response = service2.SetExpressCheckout(req);
            string error;
            if (PaymentUtils.CheckSuccess(response, out error)) {
                return GetPaypalUrl(response.Token);
            }
            throw new PaymentExecption(error);
        }

        private string GetPaypalUrl(string token) {
            return useSandBox ? "https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=" + token :
                "https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=" + token;
        }

        public PaypalPayer GetExpressCheckout(string token) {
            GetExpressCheckoutDetailsReq req = new GetExpressCheckoutDetailsReq();
            GetExpressCheckoutDetailsRequestType request = new GetExpressCheckoutDetailsRequestType();
            req.GetExpressCheckoutDetailsRequest = request;

            request.Token = token;
            request.Version = PaymentUtils.APIVersion;

            GetExpressCheckoutDetailsResponseType response = service2.GetExpressCheckoutDetails(req);

            string error;
            if (!PaymentUtils.CheckSuccess(response, out error)) {
                throw new PaymentExecption(error);
            }

            PaypalPayer payer = new PaypalPayer();
            payer.Address.FirstName = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.PayerName.FirstName;
            payer.Address.LastName = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.PayerName.LastName;
            payer.Address.CompanyName = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.PayerBusiness;
            payer.Address.Address1 = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.Street1;
            payer.Address.Address2 = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.Street2;
            payer.Address.City = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.CityName;
            payer.Address.State = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.StateOrProvince;
            payer.Address.Zipcode = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.PostalCode;
            payer.Address.Phone = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.ContactPhone;
            payer.PaypalCountryName = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.CountryName;
            payer.Address.Country = Country.GetFromCodeISO3166(response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.Address.Country.ToString());
            payer.PaypalPaymentInfo.PayerId = response.GetExpressCheckoutDetailsResponseDetails.PayerInfo.PayerID;
            payer.PaypalPaymentInfo.Token = response.GetExpressCheckoutDetailsResponseDetails.Token;
            return payer;
        }

        public PaymentTransaction DoExpressCheckout(string token, string payerId, decimal orderTotal, bool AuthOnly) {
            DoExpressCheckoutPaymentReq req = new DoExpressCheckoutPaymentReq();
            DoExpressCheckoutPaymentRequestType request = new DoExpressCheckoutPaymentRequestType();
            req.DoExpressCheckoutPaymentRequest = request;
            request.Version = PaymentUtils.APIVersion;

            DoExpressCheckoutPaymentRequestDetailsType details = new DoExpressCheckoutPaymentRequestDetailsType();
            request.DoExpressCheckoutPaymentRequestDetails = details;

            PaymentDetailsType paymentDetails = new PaymentDetailsType();
            details.PaymentDetails = paymentDetails;
            details.PaymentAction = AuthOnly ? PaymentActionCodeType.Authorization : PaymentActionCodeType.Sale;
            details.Token = token;
            details.PayerID = payerId;

            paymentDetails.OrderTotal = new BasicAmountType();
            paymentDetails.OrderTotal.Value = Math.Round(orderTotal, 2).ToString();
            paymentDetails.OrderTotal.currencyID = CurrencyCodeType.USD;
            paymentDetails.ButtonSource = "DotShoppingCart";

            DoExpressCheckoutPaymentResponseType response = service2.DoExpressCheckoutPayment(req);
            string error;
            if (!PaymentUtils.CheckSuccess(response, out error)) {
                throw new PaymentExecption(error);
            }

            PaymentTransaction trans = new PaymentTransaction();
            trans.PaymentGatewayType = PaymentGatewayType.Paypal_Express_Checkout;
            trans.PaymentTransactionType = AuthOnly ? XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Authorize :
                XShan.DotShoppingCart.Common.Object.PaymentTransactionType.Sale;
            trans.Amount = orderTotal;
            trans.AuthorizationCode = response.DoExpressCheckoutPaymentResponseDetails.PaymentInfo.TransactionID;
            trans.Response = response.Ack.ToString();
            trans.TransactionDate = response.Timestamp;
            trans.Success = true;
            return trans;
        }
    }
}
