//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Configuration.Provider;
using System.Web.Configuration;
using System.Web;
using System.Net;
using System.IO;

using XShan.DotShoppingCart.Common;
using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Payment {
    public class PaypalStandardPaymentProvider : DSCProvider {
        private bool useSandBox = true;
        private bool enabled = false;
        private string businessEmail, PDTId;

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "PaypalStandardPaymentProvider";

            base.Initialize(name, config);

            bool.TryParse(config["enabled"], out enabled);
            bool.TryParse(config["useSandBox"], out useSandBox);

            businessEmail = config["businessEmail"];
            if (enabled && String.IsNullOrEmpty(businessEmail))
                throw new ProviderException("Paypal Standard business Email is empty");

            PDTId = config["PDTId"];
            if (enabled && String.IsNullOrEmpty(PDTId))
                throw new ProviderException("Paypal Standard PTD Id is empty");
        }
        #endregion

        public override string Name {
            get { return "PayPal Standard Checkout"; }
        }

        public bool Enabled {
            get { return enabled; }
        }

        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("enabled", "Enabled", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("useSandBox", "Use Sandbox", ConfigurationValueType.Boolean));
            configSettings.Add(new ProviderAttribute("businessEmail", "Business Email", ConfigurationValueType.String));
            configSettings.Add(new ProviderAttribute("PDTId", "PDT Identity Token", ConfigurationValueType.String));
            return configSettings;
        }

        public static PaypalStandardPaymentProvider GetProvider() {
            return GetProvider(SiteNavigation.GetHost());
        }

        public static PaypalStandardPaymentProvider GetProvider(string host) {
            PaypalStandardPaymentServiceConfigurationSection pcs = (PaypalStandardPaymentServiceConfigurationSection)SqlConfigurationController.GetSection(host, 
                PaypalStandardPaymentServiceConfigurationSection.SectionName);
            if (null == pcs)
                return null;
            ProviderSettings ps = pcs.Providers[pcs.DefaultProvider];
            PaypalStandardPaymentProvider provider = (PaypalStandardPaymentProvider)ProvidersHelper.InstantiateProvider(ps, 
                typeof(PaypalStandardPaymentProvider));
            return provider;
        }

        public string GetUploadOrderUrl(string orderNumber) {
            Order order = Order.Get(orderNumber, Utils.GetUserName());

            decimal total = Math.Round(order.OrderTotalAmount, 2);

            StringBuilder url = new StringBuilder();
            url.Append(GetPaypalUrl()).Append("?cmd=_xclick&business=").Append(HttpUtility.UrlEncode(businessEmail));
            url.AppendFormat("&amount={0}", Math.Round(order.OrderTotalAmount, 2));
            url.AppendFormat("&item_name=Order Number {0}", orderNumber);
            url.AppendFormat("&custom={0}", orderNumber);
            url.Append("&no_shipping=1");
            url.AppendFormat("&first_name={0}", order.ShippingAddress.FirstName);
            url.AppendFormat("&last_name={0}", order.ShippingAddress.LastName);
            url.AppendFormat("&address1={0}", order.ShippingAddress.Address1);
            url.AppendFormat("&address2={0}", order.ShippingAddress.Address2);
            url.AppendFormat("&city={0}", order.ShippingAddress.City);
            url.AppendFormat("&state={0}", order.ShippingAddress.State);
            url.AppendFormat("&country={0}", order.ShippingAddress.Country.CodeISO3166);
            url.AppendFormat("&zip={0}", order.ShippingAddress.Zipcode);
            url.AppendFormat("&return={0}", HttpUtility.UrlEncode(SiteNavigation.PaypalPDTHandlerPage));
            url.AppendFormat("&cancel_return={0}", HttpUtility.UrlEncode(SiteNavigation.CheckoutPage));

            return url.ToString();
        }

        public bool GetPDTDetails(string tx, out Dictionary<string, string> values, out string response) {
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(GetPaypalUrl());
            req.Method = "POST";
            req.ContentType = "application/x-www-form-urlencoded";

            string formContent = string.Format("cmd=_notify-synch&at={0}&tx={1}", PDTId, tx);
            req.ContentLength = formContent.Length;

            using (StreamWriter sw = new StreamWriter(req.GetRequestStream(), Encoding.ASCII)) {
                sw.Write(formContent);
            }

            response = null;
            using (StreamReader sr = new StreamReader(req.GetResponse().GetResponseStream())) {
                response = HttpUtility.UrlDecode(sr.ReadToEnd());
            }

            values = new Dictionary<string,string>(StringComparer.OrdinalIgnoreCase);
            bool firstLine = true, success = false;
            foreach (string l in response.Split('\n')) {
                string line = l.Trim();
                if (firstLine) {
                    success = line.Equals("SUCCESS", StringComparison.OrdinalIgnoreCase);
                    firstLine = false;
                } else {
                    int equalPox = line.IndexOf('=');
                    if (equalPox >= 0)
                        values.Add(line.Substring(0, equalPox), line.Substring(equalPox + 1));
                }
            }

            return success;
        }

        public bool VerifyIPN(string formString, out Dictionary<string, string> values) {
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(GetPaypalUrl());
            req.Method = "POST";
            req.ContentType = "application/x-www-form-urlencoded";

            string formContent = string.Format("{0}&cmd=_notify-validate", formString);
            req.ContentLength = formContent.Length;

            using (StreamWriter sw = new StreamWriter(req.GetRequestStream(), Encoding.ASCII)) {
                sw.Write(formContent);
            }

            string response = null;
            using (StreamReader sr = new StreamReader(req.GetResponse().GetResponseStream())) {
                response = HttpUtility.UrlDecode(sr.ReadToEnd());
            }
            bool success = response.Trim().Equals("VERIFIED", StringComparison.OrdinalIgnoreCase);

            values = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase);
            foreach (string l in formString.Split('&')) {
                string line = l.Trim();
                int equalPox = line.IndexOf('=');
                if (equalPox >= 0)
                    values.Add(line.Substring(0, equalPox), line.Substring(equalPox + 1));
            }

            return success;
        }

        private string GetPaypalUrl() {
            return useSandBox ? "https://www.sandbox.paypal.com/us/cgi-bin/webscr" :
                "https://www.paypal.com/us/cgi-bin/webscr";
        }
    }
}
