//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Web;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text.RegularExpressions;
using System.Web.Caching;
using XShan.DotShoppingCart.Common;
using XShan.DotShoppingCart.Common.Object;

namespace Xshan.DotShoppingCart.Providers {
    public class CategorySiteMapProvider : SiteMapProvider {
        public const string RootUrl = "AllCategories.aspx";

        class NodeHolder {
            private SiteMapNode node;
            private NodeHolder parent;
            public List<NodeHolder> Children;

            public SiteMapNode Node {
                get { return node; }
            }
            public NodeHolder Parent {
                get { return parent; }
            }
            public NodeHolder(SiteMapNode node, NodeHolder parent) {
                this.node = node;
                this.parent = parent;
                Children = new List<NodeHolder>();
            }
        }

        private SiteMapProvider parentSiteMapProvider = null;
        private const int cacheTimeByHour = 1; // 1 hour

        private string siteMapCategories {
            get { return SiteNavigation.GetHost() + "_SiteMapCategories"; }
        }

        private string siteMapProduct {
            get { return SiteNavigation.GetHost() + "SiteMapProduct_"; }
        }

        // A default constructor. The Name property is initialized in the
        // Initialize method.
        public CategorySiteMapProvider() {
        }

        // Implement the CurrentNode property.
        public override SiteMapNode CurrentNode {
            get { return FindSiteMapNode(HttpContext.Current.Request.RawUrl); }
        }

        public override SiteMapProvider ParentProvider {
            get { return parentSiteMapProvider; }
            set { parentSiteMapProvider = value; }
        }

        public override SiteMapProvider RootProvider {
            get {
                if (this.ParentProvider != null) {
                    return ParentProvider.RootProvider;
                } else {
                    return this;
                }
            }
        }

        private Dictionary<string, NodeHolder> GetCategoryNodes() {
            Dictionary<string, NodeHolder> categoryNodes = (Dictionary<string, NodeHolder>)HttpContext.Current.Cache[siteMapCategories];
            if (null != categoryNodes) {
                return categoryNodes;
            }
            return LoadCategoryNodes();
        }

        private Dictionary<string, NodeHolder> LoadCategoryNodes() {
            Dictionary<string, NodeHolder> categoryNodes = new Dictionary<string, NodeHolder>(StringComparer.OrdinalIgnoreCase);
            // add root node
            NodeHolder rootNode = new NodeHolder(new SiteMapNode(this, RootUrl, SiteNavigation.ResolveUrl(RootUrl), "Categories"), null);
            categoryNodes.Add(RootUrl, rootNode);
            // load category nodes
            Queue<Category> categories = new Queue<Category>(Category.GetCategories());
            while (categories.Count > 0) {
                Category c = categories.Dequeue();
                string url = ToCategoryUrl(c.ParentFullPath, c.Name);
                NodeHolder parentNodeHolder = null;
                if (!categoryNodes.TryGetValue(ToCategoryUrl(c.ParentFullPath), out parentNodeHolder)) {
                    parentNodeHolder = rootNode;
                }
                NodeHolder nodeHolder = new NodeHolder(new SiteMapNode(this, url, SiteNavigation.ResolveUrl(url), c.Name), parentNodeHolder);
                parentNodeHolder.Children.Add(nodeHolder);
                categoryNodes.Add(url, nodeHolder);
                if (c.SubCategories != null) {
                    foreach (Category sub in c.SubCategories) {
                        categories.Enqueue(sub);
                    }
                }
            }
            // add it to cache
            HttpContext.Current.Cache.Insert(siteMapCategories, categoryNodes, null, DateTime.Now.AddHours(cacheTimeByHour), Cache.NoSlidingExpiration);
            return categoryNodes;
        }

        private NodeHolder FindNodeHolder(string rawUrl) {
            string url = FormulizeUrl(rawUrl);
            string sku = null;
            if (string.IsNullOrEmpty(url))
                return null;
            if (IsCategoryUrl(url)) {
                Dictionary<string, NodeHolder> categoryNodes = GetCategoryNodes();
                NodeHolder nodeHolder;
                if (categoryNodes.TryGetValue(url, out nodeHolder)) {
                    return nodeHolder;
                }
                // reload category nodes from DB to see if it can find it
                categoryNodes = LoadCategoryNodes();
                if (categoryNodes.TryGetValue(url, out nodeHolder)) {
                    return nodeHolder;
                }
                return null;
            } else if (IsProductUrl(url, out sku)) {
                NodeHolder productNodeHolder = (NodeHolder)HttpContext.Current.Cache[siteMapProduct + url];
                if (productNodeHolder == null) {
                    Category [] categories = Product.GetCategories(sku);
                    string categoryUrl = ToCategoryUrl(categories[0].ParentFullPath, categories[0].Name);
                    NodeHolder categoryNodeHolder = FindNodeHolder(categoryUrl);
                    productNodeHolder = new NodeHolder(new SiteMapNode(this, url, SiteNavigation.ResolveUrl(url), sku), categoryNodeHolder);
                    categoryNodeHolder.Children.Add(productNodeHolder);
                    // add it to cache
                    HttpContext.Current.Cache.Insert(siteMapProduct + url, productNodeHolder, null, DateTime.Now.AddHours(cacheTimeByHour), Cache.NoSlidingExpiration);
                }
                return productNodeHolder;
            } else
                return null;
        }

        private string FormulizeUrl(string rawUrl) {
            string relativeUrl = SiteNavigation.GetRelativePathToApplicatonRoot(rawUrl);
            Match m = Regex.Match(relativeUrl.ToLower(), @"^(?<header>.+\.aspx)");
            if (m.Success)
                return m.Groups["header"].Value;
            return null;
        }

        public override SiteMapNode FindSiteMapNode(string rawUrl) {
            NodeHolder nodeHolder = FindNodeHolder(rawUrl);
            if (null != nodeHolder)
                return nodeHolder.Node;
            return null;
        }

        private bool IsCategoryUrl(string rawUrl) {
            return Regex.IsMatch(rawUrl, @"Category\/", RegexOptions.IgnoreCase) ||
                (0 == string.Compare(rawUrl, RootUrl, true));
        }

        private bool IsProductUrl(string rawUrl, out string sku) {
            sku = null;
            Match m = Regex.Match(rawUrl, @"Product\/(?<sku>.*)\.aspx$", RegexOptions.IgnoreCase);
            if (m.Success) {
                sku = m.Groups["sku"].Value;
                return true;
            }
            return false;
        }

        private string ToCategoryUrl(params string[] names) {
            return string.Format(@"Category/{0}.aspx", SiteNavigation.ConcatenatePartsToUrl(names));
        }

        public override SiteMapNodeCollection GetChildNodes(SiteMapNode node) {
            NodeHolder nodeHolder = FindNodeHolder(node.Key);
            if (null == nodeHolder)
                return null;

            if (nodeHolder.Children.Count == 0)
                return null;

            SiteMapNodeCollection children = new SiteMapNodeCollection(nodeHolder.Children.Count);
            foreach (NodeHolder nh in nodeHolder.Children) {
                children.Add(nh.Node);
            }

            return children;
        }

        protected override SiteMapNode GetRootNodeCore() {
            Dictionary<string, NodeHolder> categoryNodes = GetCategoryNodes();
            return categoryNodes[RootUrl].Node;
        }

        public override SiteMapNode GetParentNode(SiteMapNode node) {
            NodeHolder nodeHolder = FindNodeHolder(node.Key);
            if (null != nodeHolder.Parent) {
                return nodeHolder.Parent.Node;
            }
            if (null != parentSiteMapProvider) {
                return parentSiteMapProvider.RootNode;
            }
            return null;
        }

        // Implement the ProviderBase.Initialize property.
        // Initialize is used to initialize the state that the Provider holds, but
        // not actually build the site map.
        public override void Initialize(string name, NameValueCollection attributes) {
            base.Initialize(name, attributes); 
        }
    }
}
