//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Configuration.Provider;
using System.Web.Security;
using System.Web.Configuration;
using System.Configuration;

using XShan.DotShoppingCart.Common;
using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Shipping {
    public class TableBasedShippingProvider : ShippingProvider {
        private List<TableBasedShippingMethod> methods;

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "TableBasedShippingProvider";

            base.Initialize(name, config);

            string shippingMethodsConfiguredData = config["shippingMethods"];
            if (String.IsNullOrEmpty(shippingMethodsConfiguredData))
                throw new ProviderException("Table Based Shipping configuration 'shippingMethods' is empty");

            methods = DeSerializeTaxShippingMethods(shippingMethodsConfiguredData);
        }
        #endregion

        public override string Name {
            get { return "Table Based Shipping"; }
        }

        public override List<ShippingMethod> GetAllShippingMethods() {
            if (null == methods)
                return null;

            Dictionary<string, ShippingMethod> shippingMethods = 
                new Dictionary<string,ShippingMethod>(StringComparer.OrdinalIgnoreCase);
            foreach (TableBasedShippingMethod method in methods) {
                //Todo: need to use the defaultShippingCharge from configuration setting
                if ((!shippingMethods.ContainsKey(method.Name)) && (method.ShippingRegions.Count > 0) && 
                    (method.ShippingRegions[0].Rates.Count > 0)) {
                    decimal defaultShippingCharge = method.ShippingRegions[0].Rates.Values[0];
                    shippingMethods.Add(method.Name, new ShippingMethod(method.Name, defaultShippingCharge));
                }
            }
            return new List<ShippingMethod>(shippingMethods.Values);
        }
        

        public override List<ShippingMethod> GetShippingMethods(ShippingPackage package) {
            if (null == methods)
                return null;

            List<ShippingMethod> shippingMethods = new List<ShippingMethod>();
            foreach (TableBasedShippingMethod method in methods) {
                if (null != method.ShippingRegions) {
                    foreach (TableBasedShippingRegion region in method.ShippingRegions) {
                        if (!CheckConditions(region.Conditions, package))
                            continue;

                        decimal target = 0;
                        switch (method.Target) {
                            case RateTarget.TotalOrderAmount:
                                target = package.SubTotal;
                                break;
                            case RateTarget.TotalOrderWeight:
                                target = package.Weight;
                                break;
                        }

                        // look up rates
                        decimal rate = 0;
                        foreach (decimal boundryValue in region.Rates.Keys) {
                            if (target < boundryValue)
                                break; //found
                            else
                                rate = region.Rates[boundryValue];
                        }

                        decimal shippingCharge = 0;
                        switch (method.Unit) {
                            case RateUnit.Amount:
                                shippingCharge = rate;
                                break;
                            case RateUnit.Percentage:
                                shippingCharge = target * rate;
                                break;
                        }
                        shippingMethods.Add(new ShippingMethod(method.Name, shippingCharge));
                    }
                }
            }
            return shippingMethods;
        }

        private bool CheckConditions(List<Condition> conditions, ShippingPackage package) {
            if (null == conditions)
                return true;

            foreach (Condition condition in conditions) {
                string property = null;
                switch (condition.Property) {
                    case "Country":
                        property = package.ToCountryCode;
                        break;
                    case "State":
                        property = package.ToState;
                        break;
                    case "Zipcode":
                        property = package.ToZipcode;
                        break;
                    case "Role":
                        property = string.Join(Constant.DB.ListSeparator.ToString(), Roles.GetRolesForUser());
                        break;
                }
                switch (condition.ConditionOperator) {
                    case ConditionOperator.Equal:
                        if (0 != string.Compare(property, condition.Value, true))
                            return false;
                        break;
                    case ConditionOperator.NotEqual:
                        if (0 == string.Compare(property, condition.Value, true))
                            return false;
                        break;
                    default:
                        throw new ApplicationException(string.Format("Unknown condition operator {0}", condition.ConditionOperator.ToString()));
                }
            }
            return true;
        }

        //Admin UI related APIs
        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = base.GetConfigurationSettings();
            configSettings.Add(new ProviderAttribute("shippingMethods", ConfigurationValueType.CustomType, "TableBasedShippingConfiguration.ascx"));
            return configSettings;
        }

        //Helper Methods
        public static List<TableBasedShippingMethod> DeSerializeTaxShippingMethods(string shippingMethodsConfiguredData) {
            byte[] configuredData = Convert.FromBase64String(shippingMethodsConfiguredData);
            MemoryStream stream = new MemoryStream(configuredData);
            BinaryFormatter formatter = new BinaryFormatter();
            return (List<TableBasedShippingMethod>)formatter.Deserialize(stream);
        }

        public static string SerializeShippingMethods(List<TableBasedShippingMethod> shippingMethods) {
            if (null == shippingMethods)
                return null;
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, shippingMethods);
            byte[] data = stream.GetBuffer();
            return Convert.ToBase64String(data);
        }

        public static List<TableBasedShippingMethod> GetShippingMethods() {
            ShippingServiceConfigurationSection scs = (ShippingServiceConfigurationSection)SqlConfigurationController.GetSection(ShippingServiceConfigurationSection.SectionName);
            if (null == scs)
                scs = new ShippingServiceConfigurationSection();

            TableBasedShippingProvider provider = new TableBasedShippingProvider();
            ProviderSettings ps = scs.Providers[provider.Name];
            if (null == ps) {
                ps = new ProviderSettings(provider.Name, provider.GetType().AssemblyQualifiedName);
                scs.Providers.Add(ps);
            }

            List<TableBasedShippingMethod> shippingMethods = null;
            string shippingMethodsValue = scs.Providers[provider.Name].Parameters["shippingMethods"];
            if (string.IsNullOrEmpty(shippingMethodsValue)) {
                shippingMethods = new List<TableBasedShippingMethod>();
            } else {
                shippingMethods = DeSerializeTaxShippingMethods(shippingMethodsValue);
            }
            return shippingMethods;
        }

        public static void SaveShippingMethods(List<TableBasedShippingMethod> shippingMethods) {
            ShippingServiceConfigurationSection scs = (ShippingServiceConfigurationSection)SqlConfigurationController.GetSection(ShippingServiceConfigurationSection.SectionName);
            if (null == scs)
                scs = new ShippingServiceConfigurationSection();

            TableBasedShippingProvider provider = new TableBasedShippingProvider();
            ProviderSettings ps = scs.Providers[provider.Name];
            if (null == ps) {
                ps = new ProviderSettings(provider.Name, provider.GetType().AssemblyQualifiedName);
                scs.Providers.Add(ps);
                scs.Providers[provider.Name].Parameters["enabled"] = false.ToString();
                scs.Providers[provider.Name].Parameters["sortOrder"] = "0";
            }

            scs.Providers[provider.Name].Parameters["shippingMethods"] = SerializeShippingMethods(shippingMethods);
            SqlConfigurationController.SaveSection(ShippingServiceConfigurationSection.SectionName, scs);
        }
    }
}
