//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Web.Configuration;
using System.Configuration;

using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Tax {
    public class TableBasedTaxProvider : TaxProvider {
        private List<TableBasedTaxRate> rates;

        #region Provider Specific Behaviors
        public override void Initialize(string name, NameValueCollection config) {
            // Assign the provider a default name if it doesn't have one
            if (String.IsNullOrEmpty(name))
                name = "TableBasedTaxProvider";

            base.Initialize(name, config);

            string taxRatesConfiguredData = config["taxRates"];
            if (String.IsNullOrEmpty(taxRatesConfiguredData))
                throw new ProviderException("Table Based Tax configuration 'taxRates' is empty");
            rates = DeSerializeTaxRates(taxRatesConfiguredData);
        }
        #endregion

        public override string Name {
            get { return "Table Based Tax"; }
        }

        public override decimal GetTaxRate(TaxLocation taxLocation) {
            foreach (TableBasedTaxRate rate in rates) {
                if (!CheckConditions(rate.Conditions, taxLocation))
                    continue;
                return rate.Rate;
            }
            return 0; //? might throw exception if no match is found?
        }

        private bool CheckConditions(List<Condition> conditions, TaxLocation taxLocation) {
            if (null == conditions)
                return true;

            foreach (Condition condition in conditions) {
                string property = null;
                switch (condition.Property) {
                    case "Country":
                        property = taxLocation.CountryCode;
                        break;
                    case "State":
                        property = taxLocation.State;
                        break;
                    case "Zipcode":
                        property = taxLocation.Zipcode;
                        break;
                }
                switch (condition.ConditionOperator) {
                    case ConditionOperator.Equal:
                        if (0 != string.Compare(property, condition.Value, true))
                            return false;
                        break;
                    case ConditionOperator.NotEqual:
                        if (0 == string.Compare(property, condition.Value, true))
                            return false;
                        break;
                    default:
                        throw new ApplicationException(string.Format("Unknown condition operator {0}", condition.ConditionOperator.ToString()));
                }
            }
            return true;
        }

        //Admin UI related APIs
        public override List<ProviderAttribute> GetConfigurationSettings() {
            List<ProviderAttribute> configSettings = new List<ProviderAttribute>();
            configSettings.Add(new ProviderAttribute("taxRates", ConfigurationValueType.CustomType, "TableBasedTaxConfiguration.ascx"));
            return configSettings;
        }

        //Helper Methods
        public static List<TableBasedTaxRate> DeSerializeTaxRates(string taxRatesConfiguredData) {
            byte[] configuredData = Convert.FromBase64String(taxRatesConfiguredData);
            MemoryStream stream = new MemoryStream(configuredData);
            BinaryFormatter formatter = new BinaryFormatter();
            return (List<TableBasedTaxRate>)formatter.Deserialize(stream);
        }

        public static string SerializeTaxRates(List<TableBasedTaxRate> rates) {
            if (null == rates)
                return null;
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, rates);
            byte[] data = stream.GetBuffer();
            return Convert.ToBase64String(data);
        }

        public static List<TableBasedTaxRate> GetTaxRates() {
            TaxServiceConfigurationSection pcs = (TaxServiceConfigurationSection)SqlConfigurationController.GetSection(TaxServiceConfigurationSection.SectionName);
            if (null == pcs)
                pcs = new TaxServiceConfigurationSection();

            TableBasedTaxProvider provider = new TableBasedTaxProvider();
            if (null == pcs.Providers[provider.Name]) {
                ProviderSettings ps = new ProviderSettings(provider.Name, provider.GetType().AssemblyQualifiedName);
                pcs.Providers.Add(ps);
            }

            List<TableBasedTaxRate> rates = null;
            string taxRatesValue = pcs.Providers[provider.Name].Parameters["taxRates"];
            if (string.IsNullOrEmpty(taxRatesValue)) {
                rates = new List<TableBasedTaxRate>();
            } else {
                rates = DeSerializeTaxRates(taxRatesValue);
            }
            return rates;
        }

        public static void SaveTaxRates(List<TableBasedTaxRate> rates) {
            TaxServiceConfigurationSection pcs = (TaxServiceConfigurationSection)SqlConfigurationController.GetSection(TaxServiceConfigurationSection.SectionName);
            if (null == pcs)
                pcs = new TaxServiceConfigurationSection();

            TableBasedTaxProvider provider = new TableBasedTaxProvider();
            ProviderSettings ps = new ProviderSettings(provider.Name, provider.GetType().AssemblyQualifiedName);
            if (null == pcs.Providers[provider.Name])
                pcs.Providers.Add(ps);

            pcs.Providers[provider.Name].Parameters["taxRates"] = SerializeTaxRates(rates);
            SqlConfigurationController.SaveSection(TaxServiceConfigurationSection.SectionName, pcs);
        }
    }
}
