//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Text;  

namespace XShan.DotShoppingCart.Common {
    public class DynamicImage {
        #region Settings
        public enum CornerPosition {
            TopLeft = 1,
            TopRight = 2,
            BottomLeft = 4,
            BottomRight = 8
        }
        public enum GradientType {
            Linear,
            Button,
            Triangular
        }
        public struct Settings {
            public int Width;
            public int Height;
            public int CornerSize;
            public int CornerPositionMap; //1 top left, 2 top right, 4 bottom left, 8 bottom right
            public Color BackGroundColor;
            public Color StartColor;
            public Color BackColor;
            public Color TextColor;
            public Color BorderColor;
            public Font TextFont;
            public string Text;
            public bool ApplyInternalFill;
            public bool ApplyBorder;
            public bool ApplyGradient;
            public GradientType GType;
        }
        #endregion


        public static Bitmap GetRoundedRectangle(Settings settings) {
            // Set positions for the arcs and center positions for the rectangle
            int w = settings.Width;
            int h = settings.Height; 
            float sweepAngle = 90.0f;// Standard angle for each arc.
            Bitmap bm = null;

            bm = new Bitmap(w, h, PixelFormat.Format32bppArgb);
            using (Graphics g = Graphics.FromImage(bm)) {
                g.SmoothingMode = SmoothingMode.AntiAlias; // Give us fairly high line quality.
                Rectangle bgImg = new Rectangle(-1, -1, w + 1, h + 1); // Create a rectangle for the background of the image
                using (SolidBrush bgBrush = new SolidBrush(settings.BackGroundColor)) { // Fill the entire image with our image background color
                    g.FillRectangle(bgBrush, bgImg);
                }

                Brush fillBrush;
                if (settings.ApplyGradient) {
                    LinearGradientBrush gbrush = new LinearGradientBrush(new Point(0, 0), new Point(0, h), settings.StartColor, settings.BackColor);
                    Blend bl = new Blend();
                    switch (settings.GType) {
                        case GradientType.Button:
                            bl.Positions = new float[] 
                            {
                                0, 0.35f, 0.35f, 0.7f, 1.0f
                            };
                            bl.Factors = new float[]
                            {
                                0, 0.1f, 0.7f, 0.7f, 1.0f
                            };
                            gbrush.Blend = bl;
                            break;
                        case GradientType.Triangular:
                            bl.Positions = new float[] 
                            {
                                0, 0.6f, 0.7f, 1.0f
                            };
                            bl.Factors = new float[]
                            {
                                0, 1f, 1f, 0f
                            };
                            gbrush.Blend = bl;
                            break;
                    }
                    fillBrush = gbrush;
                } else
                    fillBrush = new SolidBrush(settings.ApplyInternalFill ? settings.BackColor : settings.BackGroundColor);

                if ((0 == settings.CornerPositionMap) || (settings.CornerSize <= 0)) { // no corner needed
                    g.FillRectangle(fillBrush, bgImg);
                } else {
                    // Create 4 10 x 10 rectangles for our arcs to fit in. tl=topleft, br=bottomright
                    Rectangle tl = new Rectangle(0, 0, settings.CornerSize, settings.CornerSize);
                    Rectangle tr = new Rectangle(w - settings.CornerSize - 1, 0, settings.CornerSize, settings.CornerSize);
                    Rectangle bl = new Rectangle(0, h - settings.CornerSize - 1, settings.CornerSize, settings.CornerSize);
                    Rectangle br = new Rectangle(w - settings.CornerSize - 1, h - settings.CornerSize - 1, settings.CornerSize, settings.CornerSize);

                    // Here's how it is all bound together.  We need to add the arcs and the
                    // inner rectangle to a single GraphicsPath object.  This allows us to call
                    // the .FillPath method to only fill in the section inside the arcs and our rectangle.
                    GraphicsPath path = new System.Drawing.Drawing2D.GraphicsPath();
                    if ((settings.CornerPositionMap & (int)CornerPosition.TopLeft) > 0)
                        path.AddArc(tl, 180, sweepAngle);
                    else
                        path.AddLine(0, 0, 0, 0);

                    if ((settings.CornerPositionMap & (int)CornerPosition.TopRight) > 0)
                        path.AddArc(tr, 270, sweepAngle);
                    else
                        path.AddLine(w - 1, 0, w - 1, 0);

                    if ((settings.CornerPositionMap & (int)CornerPosition.BottomRight) > 0)
                        path.AddArc(br, 360, sweepAngle);
                    else
                        path.AddLine(w - 1, h - 1, w - 1, h - 1);

                    if ((settings.CornerPositionMap & (int)CornerPosition.BottomLeft) > 0)
                        path.AddArc(bl, 90, sweepAngle);
                    else
                        path.AddLine(0, h - 1, 0, h - 1);

                    path.CloseAllFigures();

                    g.FillPath(fillBrush, path);

                    if (settings.ApplyBorder) {
                        // If we want a border, we'll need draw the path first and then
                        // draw a horizontal line at the top of our inner rectangle and again
                        // at the bottom with the same color as our rectangle fill color to hide
                        // extra lines drawn above in the path.CloseAllFigures() method.

                        using (Pen borderPen = new Pen(settings.BorderColor, 1)) {
                            g.DrawPath(borderPen, path);
                        }
                    }
                }
                fillBrush.Dispose();

                if (!string.IsNullOrEmpty(settings.Text)) {
                    // If our image has text, draw it using the Font passed in and center the
                    // text vertically and horizontally against our innerRect rectangle.

                    using (SolidBrush TextBrush = new SolidBrush(settings.TextColor)) {
                        using (StringFormat sf = new StringFormat()) {
                            sf.Alignment = StringAlignment.Center;
                            sf.Trimming = StringTrimming.EllipsisCharacter;
                            sf.LineAlignment = StringAlignment.Center;
                            g.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;
                            g.DrawString(settings.Text, settings.TextFont, TextBrush, new Rectangle(0, 0, w, h), sf);
                        }
                    }
                }
            }
            return bm;
        }
    }
}
