//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using System.Web;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace XShan.DotShoppingCart.Common.Object {
    public class Cart : ILoadFromDataRow {
        private int cartId;
        private string promoCodes;
        private List<CartItem> cartItems;

        public int CartId {
            get { return cartId; }
        }
        public string PromoCodes {
            get { return promoCodes; }
        }
        public List<CartItem> CartItems {
            get { return cartItems; }
        }

        public int ItemsCount {
            get {
                int count = 0;
                foreach (CartItem item in cartItems) {
                    count += item.Quantity;
                }
                return count;
            }
        }
        public int InstockItemsCount {
            get {
                int count = 0;
                foreach (CartItem item in cartItems) {
                    count += item.InstockQuantity;
                }
                return count;
            }
        }
        public decimal SubTotal {
            get {
                decimal subTotal = 0;
                foreach (CartItem item in cartItems) {
                    subTotal += item.Quantity * item.Product.FinalPrice;
                }
                return subTotal;
            }
        }
        public decimal InstockSubTotal {
            get {
                decimal subTotal = 0;
                foreach (CartItem item in cartItems) {
                    subTotal += item.InstockQuantity * item.Product.FinalPrice;
                }
                return subTotal;
            }
        }
        public decimal TaxableSubTotal {
            get {
                decimal subTotal = 0;
                foreach (CartItem item in cartItems) {
                    if (!item.Product.TaxType.IsExempt)
                        subTotal += item.Quantity * item.Product.FinalPrice;
                }
                return subTotal;
            }
        }

        public bool IsDownloadable {
            get {
                foreach (CartItem item in cartItems) {
                    if (!item.Product.ShipType.IsDownloadable) {
                        return false;
                    }
                }
                return true;
            }
        }

        #region Constructor
        public Cart() {
        }

        public bool LoadFromDataRow(DataRow row) {
            cartId = (int)row["cartId"];
            promoCodes = row["promoCodes"] as string;

            // CartItem
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.CART_CARTITEM)) {
                cartItems = new RecordList<CartItem>(row.GetChildRows(Constant.DBRelation.CART_CARTITEM));
            } else {
                throw new ApplicationException("CartItem data is expected when loading Cart.");
            }
            return true;
        }
        #endregion
 
        #region Public Methods
        public bool IsEmpty() {
            return ItemsCount == 0;
        }

        /// <summary>
        /// Check the quantity for each item in the cart and make sure they are not out of stock.
        /// </summary>
        /// <returns>false if out of stock</returns>
        public bool ValidateStock() {
            foreach (CartItem item in this.cartItems) {
                if (item.InstockQuantity < item.Quantity)
                    return false;
            }
            return true;
        }
        #endregion

        #region Public Static Methods
        public static Cart Get() {
            return Get(Utils.GetUserName());
        }

        public static Cart Get(string userName) {
            Cart cart = (Cart)HttpContext.Current.Items["Cart"];
            if (null == cart) {
                Database db = DBController.CreateDatabase();
                DbCommand cmd = db.GetStoredProcCommand("DSC_Cart_Get", userName);
                DataSet ds = db.ExecuteDataSet(cmd);

                SetCartTableRelations(ds, 0);

                RecordList<Cart> carts = new RecordList<Cart>(ds.Tables["Cart"]);
                cart = 0 == carts.Count ? null : carts[0];
                HttpContext.Current.Items["Cart"] = cart;
            }
            return cart;
        }

        public static void SetCartTableRelations(DataSet ds, int startTableIndex) {
            int i = startTableIndex;
            ds.Tables[i++].TableName = "Cart";
            ds.Tables[i++].TableName = "CartItem";
            Product.SetProductTableRelations(ds, i);

            ds.Relations.Add(Constant.DBRelation.CART_CARTITEM, ds.Tables["Cart"].Columns["cartId"], ds.Tables["CartItem"].Columns["cartId"]);
            ds.Relations.Add(Constant.DBRelation.CARTITEM_PRODUCT, ds.Tables["Product"].Columns["productId"], ds.Tables["CartItem"].Columns["productId"]);
        }

        public static int AddItem(string userName, int productId, string attributeValueIds, int quantity) {
            return AddItem(userName, productId, attributeValueIds, quantity, null);
        }

        public static int AddItem(string userName, int productId, string attributeValueIds, int quantity, string customField) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Cart_AddItem", userName, productId, attributeValueIds, quantity, customField);
            object output = db.ExecuteScalar(cmd);
            return int.Parse(output.ToString());
        }

        public static void DeleteItem(int cartItemId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Cart_DeleteItem", cartItemId);
            db.ExecuteNonQuery(cmd);
        }

        public static void UpdateItemQuantities(Dictionary<int, int> cartItemIdQuantity) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Cart_UpdateItems", ConvertToCartItemQuantityValues(cartItemIdQuantity));
            db.ExecuteNonQuery(cmd);
        }

        public static void Migrate(string fromUserName, string toUserName) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Cart_Migrate", fromUserName, toUserName);
            db.ExecuteNonQuery(cmd);
        }

        public static void Clear(string userName) {
            Cart cart = Get(userName);
            if ((null != cart) && (null != cart.CartItems))
                foreach (CartItem item in cart.CartItems) {
                    DeleteItem(item.CartItemId);
                }
        }
        #endregion

        #region private methods
        private static string ConvertToCartItemQuantityValues(Dictionary<int, int> cartItemIdQuantity) {
            StringBuilder sb = new StringBuilder();
            bool first = true;
            foreach (int cartItemId in cartItemIdQuantity.Keys) {
                if (first) {
                    first = false;
                } else {
                    sb.Append(Constant.DB.ListSeparator);
                }
                sb.Append(cartItemId).Append(Constant.DB.FieldSeparator).Append(cartItemIdQuantity[cartItemId]);
            }
            return sb.ToString();
        }

        #endregion
    }
}
