//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace XShan.DotShoppingCart.Common.Object {
    public class ProductAttributeSelection {
        private DSCAttribute attribute;
        private ProductAttributeValue value;

        public DSCAttribute Attribute {
            get { return attribute; }
        }
        public ProductAttributeValue Value {
            get { return value; }
        }
        public ProductAttributeSelection(DSCAttribute attribute, ProductAttributeValue value) {
            this.attribute = attribute;
            this.value = value;
        }
    }

    public class CartItem : ILoadFromDataRow {
        private int cartItemId, quantity, instockQuantity;
        private string attributeValueIds, customField;
        private Product product;
        private List<ProductAttributeSelection> attributeSelections;
        private int? imageId;
        private string sku = null;

        public int CartItemId {
            get { return cartItemId; }
        }
        public string AttributeValueIds {
            get { return attributeValueIds; }
        }
        public string CustomField {
            get { return customField; }
        }
        public int Quantity {
            get { return quantity; }
        }
        public int InstockQuantity {
            get { return instockQuantity; }
        }
        public decimal TotalPrice {
            get { return (decimal)quantity * product.FinalPrice; }
        }
        public Product Product {
            get { return product; }
            set { product = value; }
        }
        public List<ProductAttributeSelection> AttributeSelections {
            get { return attributeSelections; }
        }
        public string AttributeSelectionsText {
            get {
                if (null != attributeSelections) {
                    StringBuilder sb = new StringBuilder();
                    foreach (ProductAttributeSelection selection in attributeSelections) {
                        if (sb.Length > 0)
                            sb.Append(", ");
                        sb.AppendFormat("{0}: {1}", selection.Attribute.Name, selection.Value.Value);
                    }
                    return sb.ToString();
                }
                return null;
            }
        }
        public int? ImageId {
            get { return imageId; }
        }
        public string SKU {
            get { return sku; }
        }

        public CartItem() {
        }

        /// <summary>
        /// This is only used by adding order not using shoppingcart
        /// </summary>
        /// <param name="sku"></param>
        /// <param name="attributeValueIds"></param>
        /// <param name="quantity"></param>
        public CartItem(string sku, string attributeValueIds, int quantity) : 
            this(sku, attributeValueIds, quantity, null) {
        }

        /// <summary>
        /// This is only used by adding order not using shoppingcart
        /// </summary>
        /// <param name="sku"></param>
        /// <param name="attributeValueIds"></param>
        /// <param name="quantity"></param>
        /// <param name="customField"></param>
        public CartItem(string sku, string attributeValueIds, int quantity, string customField) {
            this.sku = sku;
            this.attributeValueIds = attributeValueIds;
            this.quantity = quantity;
            this.customField = customField;
        }

        public bool LoadFromDataRow(DataRow row) {
            cartItemId = (int)row["cartItemId"];
            attributeValueIds = (string)row["attributeValueIds"];
            quantity = (int)row["quantity"];
            customField = row["customField"] as string;

            // Product
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.CARTITEM_PRODUCT)) {
                product = new RecordList<Product>(row.GetParentRows(Constant.DBRelation.CARTITEM_PRODUCT))[0];
            } else {
                // product has been removed since it was added to the cart
                Cart.DeleteItem(cartItemId);
                return false;
            }
            // populate AttributeSelections
            attributeSelections = null;
            if (!string.IsNullOrEmpty(attributeValueIds)) {
                Dictionary<int, ProductAttributeSelection> attributeValueIdToAttribute = new Dictionary<int, ProductAttributeSelection>();
                foreach (ProductAttribute pa in product.ProductAttributes) {
                    foreach (ProductAttributeValue pav in pa.Values) {
                        attributeValueIdToAttribute.Add(pav.ProductAttributeValueId, new ProductAttributeSelection(pa.Attribute, pav));
                    }
                }

                attributeSelections = new List<ProductAttributeSelection>();
                string[] ids = attributeValueIds.Split(Constant.DB.ListSeparator);
                foreach (string id in ids) {
                    ProductAttributeSelection pas = null;
                    if (attributeValueIdToAttribute.TryGetValue(int.Parse(id), out pas)) {
                        attributeSelections.Add(pas);
                    } else {
                        // product attribute value has been changed since it was added to the cart
                        Cart.DeleteItem(cartItemId);
                        return false;
                    }
                }
            }
            // instockQuantity
            List<int> attributeValueIdList = new List<int>();
            if (!string.IsNullOrEmpty(attributeValueIds)) {
                foreach (string idString in attributeValueIds.Split(Constant.DB.ListSeparator))
                    attributeValueIdList.Add(int.Parse(idString));
            }
            instockQuantity = product.GetStockQuantity(quantity, attributeValueIdList);

            // set imageId
            if (product.UseAttributeImageInCart && (null != attributeSelections)) {
                foreach (ProductAttributeSelection attribute in attributeSelections) {
                    if (null != attribute.Value.ImageId) {
                        imageId = attribute.Value.ImageId; // found the first value that has imageId
                        break;
                    }
                }
            } else {
                imageId = product.DefaultImageId;
            }
            return true;
        }

        public static CartItem Load(int cartItemId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_CartItem_Get", cartItemId);
            DataSet ds = db.ExecuteDataSet(cmd);

            int i = 0;
            ds.Tables[i++].TableName = "CartItem";
            Product.SetProductTableRelations(ds, i);

            ds.Relations.Add(Constant.DBRelation.CARTITEM_PRODUCT, ds.Tables["Product"].Columns["productId"], ds.Tables["CartItem"].Columns["productId"]);

            RecordList<CartItem> cartItems = new RecordList<CartItem>(ds.Tables["CartItem"]);
            if (cartItems.Count > 0)
                return cartItems[0];
            return null;
        }
    }
}
