//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Data.Common;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;

using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Common.Object {
    public class Category : ILoadFromDataRow {
        #region Private Variables
        private int categoryId, sortOrder, showLevels, featuredSortOrder;
        private int? parentId, imageId, count;
        private string name, description, parentFullPath;
        private bool showUsefulLinks;
        private bool hasChildren, isFeatured = false;
        private Category[] subCategories = null;
        private PriceRange[] priceRanges = null;
        private ManufacturerCount[] manufacturerCounts = null;
        private const string separater = " -> ";
        #endregion

        #region Properties
        public int CategoryId {
            get { return categoryId; }
        }
        public int? ParentId {
            get { return parentId; }
        }
        public string Name {
            get { return name; }
        }
        public int? ImageId {
            get { return imageId; }
        }
        public string Description {
            get { return description; }
        }
        public int SortOrder {
            get { return sortOrder; }
        }
        public int FeaturedSortOrder {
            get { return featuredSortOrder; }
        }
        public int Count {
            get { return count?? 0; }
        }
        public string ParentFullPath {
            get { return parentFullPath; }
        }
        public string FullPath {
            get { return string.IsNullOrEmpty(parentFullPath) ? name : parentFullPath.Replace(Constant.DB.CategoryPathSeparator, separater) + separater + name; }
        }
        public int ShowLevels {
            get { return showLevels; }
        }
        public bool ShowUsefulLinks {
            get { return showUsefulLinks; }
        }
        public bool HasChildren {
            get { return hasChildren; }
        }
        public bool IsFeatured {
            get { return isFeatured; }
        }
        public Category[] SubCategories {
            get { return subCategories; }
        }
        public PriceRange[] PriceRanges {
            get { return priceRanges; }
        }
        public string PriceRangeList {
            get {
                if (null == priceRanges)
                    return null;
                List<string> list = new List<string>();
                foreach (PriceRange r in priceRanges) {
                    list.Add(string.Format("{0:F2}", r.PriceFrom));
                }
                return string.Join(",", list.ToArray(), 1, list.Count - 1);
            }
        }
        public ManufacturerCount[] ManufacturerCounts {
            get { return manufacturerCounts; }
        }
        #endregion

        #region constructor
        public Category() {
        }

        public bool LoadFromDataRow(DataRow row) {
            categoryId = (int)row["categoryId"];
            parentId = DBHelper.ToNullableInt(row["parentId"]);
            name = (string)row["name"];
            imageId = DBHelper.ToNullableInt(row["imageId"]);
            description = row["description"] as string;
            sortOrder = (int)row["sortOrder"];
            showLevels = (int)row["showLevels"];
            showUsefulLinks = (bool)row["showUsefulLinks"];
            count = DBHelper.ToNullableInt(row["count"]);
            parentFullPath = (string)row["parentFullPath"];
            hasChildren = (bool)row["hasChildren"];
            if (row.Table.Columns.Contains("isFeatured")) {
                isFeatured = (bool)row["isFeatured"];
            }
            if (row.Table.Columns.Contains("featuredSortOrder")) {
                featuredSortOrder = (int)row["featuredSortOrder"];
            }

            // Populate subCategories
            string relation = CategoryParentChildRelation(GetLevelFromTableName(row.Table.TableName));
            if (row.Table.DataSet.Relations.Contains(relation)) {
                subCategories = new RecordList<Category>(row.GetChildRows(relation)).ToArray();
            } else {
                subCategories = null;
            }

            // Optional
            // PriceRange
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.CATEGORY_PRICERANGE_COUNT)) {
                DataRow[] priceRangeRows = row.GetChildRows(Constant.DBRelation.CATEGORY_PRICERANGE_COUNT);
                if (priceRangeRows.Length > 0) {
                    priceRanges = new RecordList<PriceRange>(priceRangeRows).ToArray();
                    PriceRange.SetPriceFrom(priceRanges);
                }
            }

            // ManufacturerCount
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.CATEGORY_MANUFACTURER_COUNT)) {
                DataRow[] manufacturerCountRows = row.GetChildRows(Constant.DBRelation.CATEGORY_MANUFACTURER_COUNT);
                if (manufacturerCountRows.Length > 0) {
                    manufacturerCounts = new RecordList<ManufacturerCount>(manufacturerCountRows).ToArray();
                }
            }
            return true;
        }
        #endregion

        #region Public Static Methods

        /// <summary>
        /// Return flat level all categories. This could be useful when binding to user control
        /// </summary>
        /// <returns></returns>
        public static Category[] GetAll() {
            return GetAll(null);
        }
        /// <summary>
        /// Exclude itself and its children in the return array
        /// </summary>
        /// <returns></returns>
        public static Category[] GetAll(int? categoryIdExcludeChildren) {
            return GetAll(categoryIdExcludeChildren, false);
        }

        /// <summary>
        /// Exclude itself and its children in the return array
        /// </summary>
        /// <returns></returns>
        public static Category[] GetAll(int? categoryIdExcludeChildren, bool excludeNonLeaf) {
            List<Category> all = new List<Category>();
            Stack<Category> stack = new Stack<Category>();
            Category[] categories = GetCategories();
            for (int j = categories.Length - 1; j >= 0; j--) {
                stack.Push(categories[j]);
            }

            while (stack.Count > 0) {
                Category current = stack.Pop();
                if ((null == categoryIdExcludeChildren) || (current.categoryId != categoryIdExcludeChildren)) {
                    if ((!excludeNonLeaf) || (!current.HasChildren))
                        all.Add(current);
                    if (null != current.subCategories) {
                        for (int i = current.subCategories.Length - 1; i >= 0; i--) {
                            stack.Push(current.subCategories[i]);
                        }
                    }
                }
            }
            return all.ToArray();
        }

        /// <summary>
        /// Return flat level all categories. This could be useful when binding to user control
        /// </summary>
        /// <returns></returns>
        public static Category[] GetAllLeafCategories() {
            return GetAll(null, true);
        }

        public static Category[] GetCategories() {
            return GetCategories(0, 0);
        }

        public static Category[] GetCategories(int categoryId) {
            return GetCategories(categoryId, 0);
        }

        public static Category[] GetCategories(int categoryId, int levelCount) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_Get", categoryId, levelCount);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetCategoryRelations(ds, ds.Tables.Count);

            RecordList<Category> categories = new RecordList<Category>(ds.Tables[0]);
            return categories.ToArray();
        }

        public static Category GetCategory(int categoryId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_GetDetail", categoryId);
            DataSet ds = db.ExecuteDataSet(cmd);
            return GetCategoryFromDataSet(ds);
        }

        public static List<Category> GetCategoryForDataBind(int categoryId) {
            if (-1 == categoryId)
                return null;
            List<Category> categories = new List<Category>();
            categories.Add(GetCategory(categoryId));
            return categories;
        }

        public static Category GetCategoryByCategoryPath(string categoryPath) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_GetDetailByCategoryPath", categoryPath);
            DataSet ds = db.ExecuteDataSet(cmd);
            return GetCategoryFromDataSet(ds);
        }

        private static Category GetCategoryFromDataSet(DataSet ds) {
            SetCategoryRelations(ds, ds.Tables.Count - 3);
            ds.Tables[ds.Tables.Count - 3].TableName = "PriceRangeCount";
            ds.Tables[ds.Tables.Count - 2].TableName = "ManufacturerCount";
            ds.Tables[ds.Tables.Count - 1].TableName = "Manufacturer";
            ds.Relations.Add(Constant.DBRelation.CATEGORY_PRICERANGE_COUNT, ds.Tables[0].Columns["categoryId"], ds.Tables["PriceRangeCount"].Columns["categoryId"]);
            ds.Relations.Add(Constant.DBRelation.CATEGORY_MANUFACTURER_COUNT, ds.Tables[0].Columns["categoryId"], ds.Tables["ManufacturerCount"].Columns["categoryId"]);
            ds.Relations.Add(Constant.DBRelation.MANUFACTURER_MANUFACTURER_COUNT, ds.Tables["Manufacturer"].Columns["manufacturerId"], ds.Tables["ManufacturerCount"].Columns["manufacturerId"]);

            RecordList<Category> categories = new RecordList<Category>(ds.Tables[0]);
            if ((null == categories) || (categories.Count == 0))
                return null;
            return categories[0];
        }

        public static void SetCategoryRelations(DataSet ds, int maxCount) {
            for (int i = 0; i < maxCount; i++) {
                if (ds.Tables[i].Rows.Count == 0) //empty table means it reaches the end
                    break;
                ds.Tables[i].TableName = string.Format("CategoryLevel_{0}", i);
                if (i > 0) {
                    ds.Relations.Add(CategoryParentChildRelation(i - 1), ds.Tables[i - 1].Columns["categoryId"], ds.Tables[i].Columns["parentId"]);
                }
            }
        }

        public static int Add(int? parentId, string name, int? imageId, string description, int sortOrder,
            int showLevels, bool showUsefulLinks, string priceRangeList) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_Add", parentId, name, imageId, description,
                sortOrder, showLevels, showUsefulLinks, ConvertPriceRangeListValues(priceRangeList));
            object output = db.ExecuteScalar(cmd);
            return int.Parse(output.ToString());
        }

        public static void Update(int categoryId, int? parentId, string name, int? imageId, string description, int sortOrder, 
            int showLevels, bool showUsefulLinks, string priceRangeList) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_Update", categoryId, parentId, name, imageId, description,
                sortOrder, showLevels, showUsefulLinks, ConvertPriceRangeListValues(priceRangeList));
            db.ExecuteNonQuery(cmd);
        }

        public static void Delete(int categoryId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_Delete", categoryId);
            db.ExecuteNonQuery(cmd);
        }

        public static bool CanDelete(int categoryId) {
            Database db = DBController.CreateDatabase();
            object output = db.ExecuteScalar("DSC_Category_Deletable", categoryId);
            return (bool)output;
        }

        public static void UpdateProductCount(string host) {
            string[] categoryProductCountRefreshSprocs = new string[] 
            {
                "DSC_Category_Populate_Manufacture_Count",
                "DSC_Category_Populate_PriceRange_Count",
            };
            StoreConfigurationSection storeSection = (StoreConfigurationSection)SqlConfigurationController.GetSection(host, StoreConfigurationSection.SectionName);
            foreach (string sproc in categoryProductCountRefreshSprocs) {
                Database db = DBController.CreateDatabase(host);
                DbCommand cmd = db.GetStoredProcCommand(sproc, 0, storeSection.ShowStock);
                db.ExecuteNonQuery(cmd);
            }
        }

        #endregion

        #region Public Method
        public bool CanDelete() {
            return CanDelete(categoryId);
        }

        #endregion
        #region internal helper methods
        private static string CategoryParentChildRelation(int i) {
            return string.Format("Level_{0}_To_Level_{1}", i, i + 1);
        }

        private static int GetLevelFromTableName(string tableName) {
            Match m = Regex.Match(tableName, @"CategoryLevel_(?<level>\d+)");
            if (! m.Success)
                throw new ApplicationException("Wrong tableName " + tableName);
            return int.Parse(m.Groups["level"].Value);
        }

        private static string ConvertPriceRangeListValues(string priceRangeList) {
            if (string.IsNullOrEmpty(priceRangeList))
                return null;

            bool firstRecord = true;
            StringBuilder values = new StringBuilder();
            foreach (string price in priceRangeList.Split(',', ' ', ';')) {
                string p = price.Trim();
                if (!string.IsNullOrEmpty(p)) {
                    if (!firstRecord) {
                        values.Append(Constant.DB.ListSeparator);
                    } else {
                        firstRecord = false;
                    }
                    values.Append(Utils.EscapeSeparators(p));
                }
            }
            if (values.Length == 0)
                return null;
            values.Append(Constant.DB.ListSeparator).Append(Constant.DB.MaxPriceRangeValue.ToString());
            return values.ToString();
        }

        #endregion
    }
}
