//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Data.Common;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;

namespace XShan.DotShoppingCart.Common.Object {
    [Serializable]
    public class CreditCard : ILoadFromDataRow {
        #region Member Variables
        private int creditCardId = 0;
        private string cardNumber, encryptedCardNumber, cardholderName, cvv2, phone, expireYear, expireMonth;
        private CreditCardType creditCardType;
        #endregion

        #region Properties
        public int CreditCardId {
            get { return creditCardId; }
        }
        public string CardNumber {
            get { return cardNumber; }
        }
        public string EncryptedCardNumber {
            get { return encryptedCardNumber; }
        }
        public string CardholderName {
            get { return cardholderName; }
        }
        public string CVV2 {
            get { return cvv2; }
        }
        public string Phone {
            get { return phone; }
        }
        public string ExpireYear {
            get { return expireYear; }
        }
        public string ExpireMonth {
            get { return expireMonth; }
        }
        public string MaskedCardNumber {
            get { return GetMaskedCardNumber(cardNumber); }
        }

        public CreditCardType CreditCardType {
            get { return creditCardType; }
        }
        #endregion

        #region Constructors
        public CreditCard() {
        }

        public CreditCard(int creditCardTypeId, string cardNumber, string cardholderName, string expireYear, string expireMonth, string cvv2, string phone) {
            this.cardNumber = cardNumber;
            this.encryptedCardNumber = Cryptographer.EncryptSymmetric(Constant.Security.SymmetricInstance, cardNumber);
            this.cardholderName = cardholderName;
            this.expireYear = expireYear;
            this.expireMonth = expireMonth;
            this.cvv2 = cvv2;
            this.phone = phone;
            this.creditCardType = CreditCardType.Get(creditCardTypeId);
        }

        public bool LoadFromDataRow(DataRow row) {
            creditCardId = (int)row["creditCardId"];
            encryptedCardNumber = (string)row["cardNumber"];
            cardNumber = Cryptographer.DecryptSymmetric(Constant.Security.SymmetricInstance, encryptedCardNumber);
            cardholderName = (string)row["cardholderName"];
            expireYear = (string)row["expireYear"];
            expireMonth = (string)row["expireMonth"];
            cvv2 = (string)row["cvv2"];
            phone = (string)row["phone"];

            // CreditCardType
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.CREDITCARD_CREDITCARDTYPE)) {
                // expect only one record
                creditCardType = new RecordList<CreditCardType>(row.GetParentRows(Constant.DBRelation.CREDITCARD_CREDITCARDTYPE))[0];
            } else {
                throw new ApplicationException("CreditCardType data is expected when loading CreditCard.");
            }
            return true;
        }
        #endregion

        #region Public Static Methods
        public static string GetMaskedCardNumber(string cardNumber) {
            return string.Format("****{0}", cardNumber.Substring(cardNumber.Length - 4));
        }

        public static List<CreditCard> Get() {
            return Get(Utils.GetUserName());
        }

        public static List<CreditCard> Get(string userName) {
            return Get(SiteNavigation.GetHost(), userName);
        }

        public static List<CreditCard> Get(string host, string userName) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_CreditCard_Get", userName);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetCreditCardQueryDBRelations(ds);

            return new RecordList<CreditCard>(ds.Tables[0]);
        }

        public static CreditCard Get(int creditCardId) {
            return Get(SiteNavigation.GetHost(), creditCardId);
        }

        public static CreditCard Get(string host, int creditCardId) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_CreditCard_GetById", creditCardId);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetCreditCardQueryDBRelations(ds);

            RecordList<CreditCard> cards = new RecordList<CreditCard>(ds.Tables[0]);
            return cards[0]; // expect only one credit card returned
        }

        public static int SetCreditCardQueryDBRelations(DataSet ds, int startTableIndex) {
            int index = startTableIndex;
            ds.Tables[index++].TableName = "CreditCard";
            ds.Tables[index++].TableName = "CreditCardType";

            ds.Relations.Add(Constant.DBRelation.CREDITCARD_CREDITCARDTYPE, ds.Tables["CreditCardType"].Columns["creditCardTypeId"],
                ds.Tables["CreditCard"].Columns["creditCardTypeId"]);

            return index;
        }

        private static void SetCreditCardQueryDBRelations(DataSet ds) {
            SetCreditCardQueryDBRelations(ds, 0);
        }

        public static int Add(string host, string userName, int creditCardTypeId, string cardNumber, string cardholderName, string expireYear,
                string expireMonth, string cvv2, string phone) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_CreditCard_Add", userName, creditCardTypeId,
                Cryptographer.EncryptSymmetric(Constant.Security.SymmetricInstance, cardNumber), cardholderName, expireYear, 
                expireMonth, cvv2, phone);
            object output = db.ExecuteScalar(cmd);
            return int.Parse(output.ToString());
        }
        public static int Add(string userName, int creditCardTypeId, string cardNumber, string cardholderName, string expireYear,
                string expireMonth, string cvv2, string phone) {
            return Add(SiteNavigation.GetHost(), userName, creditCardTypeId, cardNumber, cardholderName, expireYear, expireMonth, cvv2, phone);
        }
        public static int Add(int creditCardTypeId, string cardNumber, string cardholderName, string expireYear,
                string expireMonth, string cvv2, string phone) {
            return Add(Utils.GetUserName(), creditCardTypeId, cardNumber, cardholderName, expireYear, expireMonth, cvv2, phone);
        }

        public static void Update(int creditCardId, int creditCardTypeId, string cardNumber, string cardholderName, string expireYear,
                string expireMonth, string cvv2, string phone) {
            Update(creditCardId, creditCardTypeId, cardNumber, cardholderName, expireYear, expireMonth, cvv2, phone, false);
        }

        public static void Update(int creditCardId, int creditCardTypeId, string cardNumber, string cardholderName, string expireYear,
                string expireMonth, string cvv2, string phone, bool isDefault) {
            Update(SiteNavigation.GetHost(), creditCardId, creditCardTypeId, cardNumber, cardholderName, expireYear,
                expireMonth, cvv2, phone, isDefault, false);
        }

        public static void Update(string host, int creditCardId, int creditCardTypeId, string cardNumber, string cardholderName, string expireYear,
                string expireMonth, string cvv2, string phone, bool isDefault, bool isCleanedUp) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_CreditCard_Update", creditCardId, creditCardTypeId,
                Cryptographer.EncryptSymmetric(Constant.Security.SymmetricInstance, cardNumber), cardholderName, expireYear,
                expireMonth, cvv2, phone, isDefault, isCleanedUp);
            db.ExecuteNonQuery(cmd);
        }

        /// <summary>
        /// Mask credit card# and just leave last 4 digits
        /// </summary>
        /// <param name="host"></param>
        public static void CleanUp(string host) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_CreditCard_GetForCleanUp");
            DataSet ds = db.ExecuteDataSet(cmd);
            foreach (DataRow row in ds.Tables[0].Rows) {
                int creditCardId = (int)row["creditCardId"];
                CleanUp(host, creditCardId);
            }
        }

        private static void CleanUp(string host, int creditCardId) {
            CreditCard cc = CreditCard.Get(host, creditCardId);
            Update(host, creditCardId, cc.CreditCardType.CreditCardTypeId, cc.MaskedCardNumber, cc.CardholderName, cc.ExpireYear, 
                cc.ExpireMonth, "000", cc.Phone, false, true);
        }

        public static void Delete(int creditCardId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_CreditCard_Delete", creditCardId);
            db.ExecuteNonQuery(cmd);
        }
        #endregion
    }
}
