//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.Common;
using System.Web;
using System.Web.Caching;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace XShan.DotShoppingCart.Common.Object {
    public class DbImage {
        public const string ImageDirectory = "Images";
        private int imageId;
        private string mime;
        private DateTime timeUpdated;
        private byte[] bits;

        public int ImageId {
            get { return imageId; }
        }
        public string MIME {
            get { return mime; }
        }
        public DateTime TimeUpdated {
            get { return timeUpdated; }
        }
        public byte[] Bits {
            get { return bits; }
        }

        public DbImage(int imageId, byte[] bits, string mime, DateTime timeUpdated) {
            this.imageId = imageId;
            this.bits = bits;
            this.mime = mime;
            this.timeUpdated = timeUpdated;
        }

        public static DbImage Get(int imageId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Image_Get", imageId);
            DbImage img = null;
            using (IDataReader dataReader = db.ExecuteReader(cmd)) {
                if (dataReader.Read()) {
                    img = new DbImage((int)dataReader["imageId"],
                        (byte[])dataReader["bits"],
                        (string)dataReader["mime"],
                        (DateTime)dataReader["timeUpdated"]);
                }
            }
            return img;
        }

        /// <summary>
        /// Add a new image into DB
        /// </summary>
        /// <param name="bits"></param>
        /// <param name="mime"></param>
        /// <returns>image ID</returns>
        public static int Add(byte[] bits, string mime)
        {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Image_Add", bits,  mime);
            return Convert.ToInt32(db.ExecuteScalar(cmd));
        }

        public static string GetFile(string host, int imageId, int width, int height, bool noTimeStampCheck) {
            string localImagePath = GetImagePath();

            string[] files = Directory.GetFiles(localImagePath,
                (0 == width) || (0 == height) ? string.Format("{0}.*", imageId) : string.Format("{0}_{1}_{2}.*", imageId, width, height));
            if (files.Length > 0) {
                //Expect only 1 file with that filename
                string localFileSpec = files[0];

                if (noTimeStampCheck)
                    return Path.GetFileName(localFileSpec);

                DateTime localFileTimeStamp = File.GetLastWriteTime(localFileSpec);
                // Look up in the cache
                DateTime fileTimeStampInDB;
                string cacheKey = string.Format("{0}_ImageId_{1}", host, imageId);
                object o = Utils.Cache.Get(cacheKey);
                if (null == o) {
                    // get timestamp from DB
                    SqlCacheDependency dep;
                    fileTimeStampInDB = DbImage.GetTimeStamp(host, imageId, out dep);
                    Utils.Cache.Insert(cacheKey, fileTimeStampInDB, dep, Cache.NoAbsoluteExpiration, 
                        Cache.NoSlidingExpiration, CacheItemPriority.Normal,
                        new CacheItemRemovedCallback(WebUtils.CacheItemRemovedCallback));
                } else {
                    fileTimeStampInDB = (DateTime)o;
                }
                if (localFileTimeStamp >= fileTimeStampInDB)
                    return Path.GetFileName(localFileSpec);
                else
                    File.Delete(localFileSpec); // delete outdated file
            }
            return CreateFile(imageId, localImagePath, width, height);
        }

        public static string GetImagePath() {
            string localImagePath = HttpContext.Current.Request.PhysicalApplicationPath + Path.Combine(ImageDirectory, SiteNavigation.GetHost());

            if (!Directory.Exists(localImagePath))
                Directory.CreateDirectory(localImagePath);
            return localImagePath;
        }

        #region Private Methods
        private static DateTime GetTimeStamp(string host, int imageId, out SqlCacheDependency dep) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_Image_GetTimeStamp", imageId);
            dep = DBController.GetSqlCacheDependency(host, (SqlCommand)cmd);
            return Convert.ToDateTime(db.ExecuteScalar(cmd));
        }

        private static string CreateFile(int imageId, string localImagePath, int width, int height) {
            DbImage img = DbImage.Get(imageId);
            if (null == img)
                return null;
            string[] parts = img.MIME.Split('/');
            string lastPart = parts[parts.Length - 1];
            switch (lastPart) {
                case "pjpeg":
                    lastPart = "jpg";
                    break;
                case "x-png":
                    lastPart = "png";
                    break;
                case "x-icon":
                    lastPart = "ico";
                    break;
            }

            string localFilename;
            if ((0 == width) || (0 == height)) {
                localFilename = string.Format("{0}.{1}", imageId, lastPart);
                File.WriteAllBytes(Path.Combine(localImagePath, localFilename), img.Bits);
            } else {
                localFilename = string.Format("{0}_{1}_{2}.jpg", imageId, width, height);
                using (MemoryStream stream = new MemoryStream(img.Bits)) {
                    Bitmap b = new Bitmap(stream);
                    int newWidth, newHeight;
                    if ((double)height / width > (double)b.Height / b.Width) {
                        newWidth = width;
                        newHeight = width * b.Height / b.Width;
                    } else {
                        newWidth = height * b.Width / b.Height;
                        newHeight = height;
                    }
                    Bitmap newBitMap = new Bitmap(newWidth, newHeight);
                    Graphics g = Graphics.FromImage(newBitMap);
                    g.DrawImage(b, 0, 0, newWidth, newHeight);
                    newBitMap.Save(Path.Combine(localImagePath, localFilename), ImageFormat.Jpeg);
                    newBitMap.Dispose();
                    b.Dispose();
                }
            }
            return localFilename;
        }
        #endregion
    }
}
