//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Collections.Generic;
using System.Data.Common;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.Security;
using System.IO;
using System.Collections.Specialized;
using Microsoft.Practices.EnterpriseLibrary.Data;

using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Common.Object {
    public class Order : ILoadFromDataRow {
        #region Member Variables
        private int orderId, userId;
        private int? parentOrderId;
        private string orderNumber, shippingMethod, email, userIp, specialInstructions, customField, orderPipeline;
        private DateTime created;
        private OrderStatus orderStatus;
        private List<OrderTotal> orderTotals;
        private Address billingAddress, shippingAddress;
        private List<OrderItem> orderItems;
        private List<OrderNote> orderNotes;
        private List<PaymentTransaction> paymentTransactions;
        private CheckoutPaymentType checkoutPaymentType;
        private CreditCard creditCard;
        private List<ShipTracking> shipTrackings;
        #endregion

        #region Properties
        public int OrderId {
            get { return orderId; }
        }
        public int UserId {
            get { return userId; }
        }
        public string OrderNumber {
            get { return orderNumber; }
        }
        public string ShippingMethod {
            get { return shippingMethod; }
        }
        public string Email {
            get { return email; }
        }
        public string UserIp {
            get { return userIp; }
        }
        public string SpecialInstructions {
            get { return specialInstructions; }
        }
        public string CustomField {
            get { return customField; }
        }
        public string OrderPipeline {
            get { return orderPipeline; }
        }
        public DateTime Created {
            get { return created; }
        }
        public int? ParentOrderId {
            get { return parentOrderId; }
        }
        public List<OrderTotal> OrderTotals {
            get { return orderTotals; }
        }
        public OrderStatus OrderStatus {
            get { return orderStatus; }
        }
        public Address BillingAddress {
            get { return billingAddress; }
        }
        public Address ShippingAddress {
            get { return shippingAddress; }
        }
        public List<OrderItem> OrderItems {
            get { return orderItems; }
        }
        public List<OrderNote> OrderNotes {
            get { return orderNotes; }
        }
        public List<PaymentTransaction> PaymentTransactions {
            get { return paymentTransactions; }
            set { paymentTransactions = value; }
        }
        public CheckoutPaymentType CheckoutPaymentType {
            get { return checkoutPaymentType; }
            set { checkoutPaymentType = value; }
        }
        public CreditCard CreditCard {
            get { return creditCard; }
        }
        public List<ShipTracking> ShipTrackings {
            get { return shipTrackings; }
        }
        public string ShipTrackingsHtml {
            get { 
                StringBuilder sb = new StringBuilder();
                if (null == shipTrackings)
                    return string.Empty;
                foreach (ShipTracking tracking in shipTrackings) {
                    if (sb.Length > 0)
                        sb.Append(", &nbsp;");
                    sb.AppendFormat("{0}#: <a href=\"{1}\" target=\"_blank\">{2}</a>", tracking.Carrier.Name, tracking.TrackingUrl, tracking.TrackingNumber);
                }
                return sb.ToString(); 
            }
        }

        public decimal OrderTotalAmount {
            get { return orderTotals.Count > 0 ? OrderTotals[orderTotals.Count - 1].Amount : 0; }
        }

        public int OrderItemCount {
            get {
                int count = 0;
                if (null != orderItems) {
                    foreach (OrderItem item in orderItems) {
                        count += (int)item.Quantity;
                    }
                }
                return count;
            }
        }
        #endregion
        
        #region Constructors
        public Order() {
        }

        public bool LoadFromDataRow(DataRow row) {
            orderId = (int)row["orderId"];
            userId = (int)row["userId"];
            created = (DateTime)row["created"];
            orderNumber = (string)row["orderNumber"];
            shippingMethod = (string)row["shippingMethod"];
            email = (string)row["email"];
            userIp = (string)row["userIp"];
            specialInstructions = row["specialInstructions"] as string;
            customField = row["customField"] as string;
            orderPipeline = row["orderPipeline"] as string;
            parentOrderId = DBHelper.ToNullableInt(row["parentOrderId"]);

            // OrderStatus
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_ORDERSTATUS)) {
                // expect only one record
                orderStatus = new RecordList<OrderStatus>(row.GetParentRows(Constant.DBRelation.ORDER_ORDERSTATUS))[0];
            } else {
                throw new ApplicationException("OrderStatus data is expected when loading Order.");
            }

            // OrderTotal
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_ORDERTOTAL)) {
                orderTotals = new RecordList<OrderTotal>(row.GetChildRows(Constant.DBRelation.ORDER_ORDERTOTAL));
            } else {
                throw new ApplicationException("ORDER_ORDERTOTAL data is expected when loading Order.");
            }

            // ShippingAddress
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_SHIPPINGADDRESS)) {
                // expect only one record
                shippingAddress = new RecordList<Address>(row.GetParentRows(Constant.DBRelation.ORDER_SHIPPINGADDRESS))[0];
            } else {
                throw new ApplicationException("ShippingAddress data is expected when loading Order.");
            }

            // BillingAddress
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_BILLINGADDRESS)) {
                // expect only one record
                billingAddress = new RecordList<Address>(row.GetParentRows(Constant.DBRelation.ORDER_BILLINGADDRESS))[0];
            } else {
                throw new ApplicationException("ShippingAddress data is expected when loading Order.");
            }

            // CheckoutPaymentType
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_CHECKOUTPAYMENTTYPE)) {
                // expect only one record
                checkoutPaymentType = new RecordList<CheckoutPaymentType>(row.GetParentRows(Constant.DBRelation.ORDER_CHECKOUTPAYMENTTYPE))[0];
            } else {
                throw new ApplicationException("CheckoutPaymentType data is expected when loading Order.");
            }

            // Optional
            // OrderItem
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_ORDERITEM)) {
                // expect only one record
                orderItems = new RecordList<OrderItem>(row.GetChildRows(Constant.DBRelation.ORDER_ORDERITEM));
            }

            // OrderNote
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_ORDERNOTE)) {
                // expect only one record
                orderNotes = new RecordList<OrderNote>(row.GetChildRows(Constant.DBRelation.ORDER_ORDERNOTE));
            }

            // PaymentTransactions
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_PAYMENTTRANSACTION)) {
                // expect only one record
                paymentTransactions = new RecordList<PaymentTransaction>(row.GetChildRows(Constant.DBRelation.ORDER_PAYMENTTRANSACTION));
            }

            // CreditCard
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.ORDER_CREDITCARD)) {
                // expect one record or null
                List<CreditCard> creditCards = new RecordList<CreditCard>(row.GetChildRows(Constant.DBRelation.ORDER_CREDITCARD));
                if ((creditCards != null) && (creditCards.Count > 0))
                    creditCard = creditCards[0];
            }

            // populate ShipTracking
            if (null != orderItems) {
                shipTrackings = new List<ShipTracking>();
                Dictionary<int, bool> trackingIds = new Dictionary<int, bool>();
                foreach (OrderItem item in orderItems) {
                    if ((null != item.ShipTracking) && (!trackingIds.ContainsKey(item.ShipTracking.ShipTrackingId))) {
                        shipTrackings.Add(item.ShipTracking);
                        trackingIds.Add(item.ShipTracking.ShipTrackingId, true);
                    }
                }
                if (0 == shipTrackings.Count)
                    shipTrackings = null;
            }

            return true;
        }
        #endregion

        #region Pulic Methods
        public List<OrderPipelineProcessor> GetOrderPipelineActions() {
            List<OrderPipelineProcessor> processors = new List<OrderPipelineProcessor>();
            foreach (OrderPipelineProcessor processor in OrderPipelineProcessor.GetConfiguredProcessors(orderPipeline)) {
                if ((orderStatus.StatusId == processor.OrderStatusIdSource) && (processor.IsWebDriven))
                    processors.Add(processor);
            }

            return processors;
        }

        public void SetToNextStatusInPipeline(string note) {
            SetToNextStatusInPipeline(SiteNavigation.GetHost(), note);
        }

        public void SetToNextStatusInPipeline(string host, string note) {
            int orderStatusIdTarget = Constant.DB.OrderStatusCompleteId;
            bool start = false;
            foreach (OrderPipelineProcessor processor in OrderPipelineProcessor.GetConfiguredProcessors(host, orderPipeline)) {
                if (start) {
                    if ((!BypassShipping(processor)) && (!BypassAwaitingPaypalPaymentCleared(processor))) {
                        orderStatusIdTarget = processor.OrderStatusIdSource;
                        break;
                    }
                } else if (orderStatus.StatusId == processor.OrderStatusIdSource)
                    start = true;
            }
            SetStatus(host, orderId, orderStatusIdTarget, note);
        }

        private bool BypassShipping(OrderPipelineProcessor processor) {
            return processor.IsDownloadBypass && (shippingMethod == Constant.Shipping.DownloadMethodName);
        }

        private bool BypassAwaitingPaypalPaymentCleared(OrderPipelineProcessor processor) {
            if (processor.OrderPipelineProcessorId == 25) {// PaypalStandardAwaitingPaymentCleared
                PaymentTransaction t = GetLastAuthorizationTransaction();
                if (null != t)
                    return t.CVV.Equals("Completed");
            }
            return false;
        }

        public PaymentTransaction GetLastAuthorizationTransaction() {
            if (null == paymentTransactions)
                return null;
            // paymentTransactions is sorted by date
            for (int i = paymentTransactions.Count - 1; i >= 0; i--) {
                PaymentTransaction pt = paymentTransactions[i];
                if ((PaymentTransactionType.Authorize == pt.PaymentTransactionType) && pt.Success)
                    return pt;
            }
            return null;
        }

        public PaymentTransaction GetLastCaptureOrSaleTransaction() {
            if (null == paymentTransactions)
                return null;
            // paymentTransactions is sorted by date
            for (int i = paymentTransactions.Count - 1; i >= 0; i--) {
                PaymentTransaction pt = paymentTransactions[i];
                if (((PaymentTransactionType.Capture == pt.PaymentTransactionType) || (PaymentTransactionType.Sale == pt.PaymentTransactionType))
                    && pt.Success)
                    return pt;
            }
            return null;
        }

        public void AddTokens(NameValueCollection tokens) {
            //Todo: use reflection to auto populate tokens
            tokens.Add("Order.OrderNumber", OrderNumber);
            tokens.Add("Order.Email", Email);
            tokens.Add("Order.PaymentMethod", CheckoutPaymentType.Type);
            tokens.Add("Order.ShippingMethod", ShippingMethod);
            tokens.Add("Order.ShipTrackings", ShipTrackingsHtml);
            tokens.Add("Order.ShippingAddress.FirstName", ShippingAddress.FirstName);
            tokens.Add("Order.ShippingAddress.LastName", ShippingAddress.LastName);
            tokens.Add("Order.ShippingAddress.Address",
                (!string.IsNullOrEmpty(ShippingAddress.CompanyName) ? ShippingAddress.CompanyName + "<br />" : "") +
                ShippingAddress.Address1 +
                (!string.IsNullOrEmpty(ShippingAddress.Address2) ? "<br />" + ShippingAddress.Address2 : ""));
            tokens.Add("Order.ShippingAddress.City", ShippingAddress.City);
            tokens.Add("Order.ShippingAddress.State", ShippingAddress.State);
            tokens.Add("Order.ShippingAddress.Zipcode", ShippingAddress.Zipcode);
            tokens.Add("Order.ShippingAddress.Country", ShippingAddress.Country.Name);
            tokens.Add("Order.BillingAddress.FirstName", BillingAddress.FirstName);
            tokens.Add("Order.BillingAddress.LastName", BillingAddress.LastName);
            tokens.Add("Order.BillingAddress.Address",
                (!string.IsNullOrEmpty(BillingAddress.CompanyName) ? BillingAddress.CompanyName + "<br />" : "") +
                BillingAddress.Address1 +
                (!string.IsNullOrEmpty(BillingAddress.Address2) ? "<br />" + BillingAddress.Address2 : ""));
            tokens.Add("Order.BillingAddress.City", BillingAddress.City);
            tokens.Add("Order.BillingAddress.State", BillingAddress.State);
            tokens.Add("Order.BillingAddress.Zipcode", BillingAddress.Zipcode);
            tokens.Add("Order.BillingAddress.Country", BillingAddress.Country.Name);
            tokens.Add("Order.DetailHtml", GetDetailHtml());
        }

        private string GetDetailHtml() {
            StringWriter sw = new StringWriter();
            HtmlTextWriter writer = new HtmlTextWriter(sw);
            writer.WriteBeginTag("table");
            writer.WriteAttribute("style", "width:100%;");
            writer.Write(HtmlTextWriter.TagRightChar);
            writer.Write("<tr><th style='text-align:left;'>Product Name</th><th style='text-align:left; width:100px;'>SKU</th><th style='text-align:left; width:100px;'>Unit Price</th>" +
                "<th style='text-align:left; width:50px;'>Qty.</th><th style='text-align:left; width:100px;'>Total Price</th></tr>");
            foreach (OrderItem item in orderItems) {
                writer.WriteFullBeginTag("tr");
                //ProductName
                writer.WriteFullBeginTag("td");
                writer.WriteEncodedText(item.ProductName);
                writer.WriteBreak();
                if (null != item.Attributes) {
                    bool first = true;
                    foreach (KeyValuePair<string, string> kv in item.Attributes) {
                        if (first) {
                            writer.WriteEncodedText("   ");
                            first = false;
                        } else {
                            writer.WriteEncodedText(", ");
                        }
                        writer.WriteFullBeginTag("b");
                        writer.WriteEncodedText(kv.Key + ": ");
                        writer.WriteEndTag("b");
                        writer.WriteEncodedText(kv.Value);
                    }
                }
                if (!string.IsNullOrEmpty(item.CustomField)) {
                    if (ProductTypeEnum.RegistrationCodeAutoGen == item.ProductType)
                        writer.WriteEncodedText(string.Format(" Registration Code: {0}", item.CustomField));
                    else if (parentOrderId != null) // is the recurring order
                        writer.WriteEncodedText(item.CustomField);
                }
                writer.WriteEndTag("td");
                //SKU
                writer.WriteFullBeginTag("td");
                writer.WriteEncodedText(item.SKU);
                writer.WriteEndTag("td");
                //Unit Price
                writer.WriteFullBeginTag("td");
                writer.WriteEncodedText(item.Price.ToString("C"));
                writer.WriteEndTag("td");
                //Qty
                writer.WriteFullBeginTag("td");
                writer.WriteEncodedText(item.Quantity.ToString());
                writer.WriteEndTag("td");
                //SubTotal
                writer.WriteFullBeginTag("td");
                writer.WriteEncodedText(item.TotalPrice.ToString("C"));
                writer.WriteEndTag("td");
                writer.WriteEndTag("tr");
            }

            foreach (OrderTotal total in orderTotals) {
                //Total Name
                writer.WriteBeginTag("td");
                writer.WriteAttribute("colspan", "4");
                writer.WriteAttribute("style", "text-align:right;font-weight:bold;");
                writer.Write(HtmlTextWriter.TagRightChar);
                writer.WriteEncodedText(total.Name + ": ");
                writer.WriteEndTag("td");
                //SubTotal
                writer.WriteFullBeginTag("td");
                writer.WriteEncodedText(total.Amount.ToString("C"));
                writer.WriteEndTag("td");
                writer.WriteEndTag("tr");
            }
            writer.WriteEndTag("table");
            return sw.ToString();
        }


        #endregion

        #region Static Public Methods
        public static Order Get(int orderId) {
            Database db = DBController.CreateDatabase();
            //Avoid non-administrator check somebody else order
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_Get", orderId, 
                Roles.IsUserInRole("Administrator") ?  null : Utils.GetUserName());
            DataSet ds = db.ExecuteDataSet(cmd);

            SetOrderDBRelations(ds);

            List<Order> orders = new RecordList<Order>(ds.Tables[0]);
            if ((null != orders) && (orders.Count > 0))
                return orders[0];
            return null;
        }

        public static Order Get(string orderNumber) {
            return Get(orderNumber, null);
        }

        public static Order Get(string orderNumber, string userName) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_GetByOrderNumber", orderNumber, userName);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetOrderDBRelations(ds);

            List<Order> orders = new RecordList<Order>(ds.Tables[0]);
            if ((null != orders) && (orders.Count > 0))
                return orders[0];
            return null;
        }

        public static Order GetByCustomField(string customField) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_GetByCustomField", customField);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetOrderDBRelations(ds);

            List<Order> orders = new RecordList<Order>(ds.Tables[0]);
            if ((null != orders) && (orders.Count > 0))
                return orders[0];
            return null;
        }

        public static List<Order> Search(string keywords, DateTime from, DateTime to, int? orderStatusId, int rowsPerPage, int rowIndex, string sortBy) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_Search");
            db.AddInParameter(cmd, "@keywords", DbType.String, Utils.ConvertToFullTextSearchString(keywords));
            db.AddInParameter(cmd, "@from", DbType.DateTime, from);
            db.AddInParameter(cmd, "@to", DbType.DateTime, to);
            db.AddInParameter(cmd, "@orderStatusId", DbType.Int32, orderStatusId);
            db.AddInParameter(cmd, "@rowsPerPage", DbType.Int32, rowsPerPage);
            db.AddInParameter(cmd, "@rowIndex", DbType.Int32, rowIndex);

            if (!string.IsNullOrEmpty(sortBy)) {
                bool asc = true;
                if (sortBy.EndsWith(" DESC")) {
                    sortBy = sortBy.Substring(0, sortBy.Length - 5);
                    asc = false;
                }
                db.AddInParameter(cmd, "@orderBy", DbType.String, sortBy);
                db.AddInParameter(cmd, "@asc", DbType.Boolean, asc);
            }
            db.AddOutParameter(cmd, "@totalOrders", DbType.Int32, sizeof(Int32));

            DataSet ds = db.ExecuteDataSet(cmd);
            int totalNumber = (int)db.GetParameterValue(cmd, "@totalOrders");

            SetOrderDBRelations(ds);

            RecordList<Order> orders = new RecordList<Order>(ds.Tables[0]);

            if (null != HttpContext.Current)
                HttpContext.Current.Items[GetOrderCountCacheKey(keywords, from, to, orderStatusId)] = totalNumber;

            return orders;
        }

        public static int GetOrderCount(string keywords, DateTime from, DateTime to, int? orderStatusId) {
            int count = 0;
            if ((null != HttpContext.Current) &&
                (null != HttpContext.Current.Items[GetOrderCountCacheKey(keywords, from, to, orderStatusId)]))
                count = (int)HttpContext.Current.Items[GetOrderCountCacheKey(keywords, from, to, orderStatusId)];
            return count;
        }

        private static string GetOrderCountCacheKey(string keywords, DateTime from, DateTime to, int? orderStatusId) {
            return string.Format("OrderCount_{0}_{1}_{2}_{3}_{4}", SiteNavigation.GetHost(), keywords, from, to, orderStatusId);
        }

        public static List<Order> GetSummary(string userName, int rowsPerPage, int rowIndex, string sortBy) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_GetSummary");
            db.AddInParameter(cmd, "@userName", DbType.String, userName);
            db.AddInParameter(cmd, "@rowsPerPage", DbType.Int32, rowsPerPage);
            db.AddInParameter(cmd, "@rowIndex", DbType.Int32, rowIndex);

            if (!string.IsNullOrEmpty(sortBy)) {
                bool asc = true;
                if (sortBy.EndsWith(" DESC")) {
                    sortBy = sortBy.Substring(0, sortBy.Length - 5);
                    asc = false;
                }
                db.AddInParameter(cmd, "@orderBy", DbType.String, sortBy);
                db.AddInParameter(cmd, "@asc", DbType.Boolean, asc);
            }
            db.AddOutParameter(cmd, "@totalOrders", DbType.Int32, sizeof(Int32));

            DataSet ds = db.ExecuteDataSet(cmd);
            int totalNumber = (int)db.GetParameterValue(cmd, "@totalOrders");

            SetOrderDBRelations(ds);

            RecordList<Order> orders = new RecordList<Order>(ds.Tables[0]);

            if (null != HttpContext.Current)
                HttpContext.Current.Items[GetSummaryOrderCountCacheKey(userName)] = totalNumber;

            return orders;
        }

        public static int GetSummaryOrderCount(string userName) {
            int count = 0;
            if ((null != HttpContext.Current) &&
                (null != HttpContext.Current.Items[GetSummaryOrderCountCacheKey(userName)]))
                count = (int)HttpContext.Current.Items[GetSummaryOrderCountCacheKey(userName)];
            return count;
        }

        private static string GetSummaryOrderCountCacheKey(string userName) {
            return string.Format("SummaryOrderCount_{0}_{1}", SiteNavigation.GetHost(), userName);
        }
        

        public static List<Order> GetActiveOrders(DateTime from, DateTime to, string orderStatusIds) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_GetByDateRangeAndStatusList", from, to, orderStatusIds);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetOrderDBRelations(ds);

            return new RecordList<Order>(ds.Tables[0]);
        }

        public static List<Order> GetByStatus(string host, int orderStatusId) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_SearchByStatus", orderStatusId);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetOrderDBRelations(ds);

            return new RecordList<Order>(ds.Tables[0]);
        }

        public static void SetStatus(int orderId, int orderStatusId, string note) {
            Order.SetStatus(SiteNavigation.GetHost(), orderId, orderStatusId, note);
        }

        public static void SetStatus(string host, int orderId, int orderStatusId, string note) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_SetStatus", orderId, orderStatusId, note);
            db.ExecuteNonQuery(cmd);
        }

        public static void AddNote(int orderId, string note) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_AddNote", orderId, note, null);
            db.ExecuteNonQuery(cmd);
        }

        public static void AddNote(int orderId, string note, DateTime created) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_AddNote", orderId, note, created);
            db.ExecuteNonQuery(cmd);
        }

        public static void AddNote(string host, int orderId, string note) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_AddNote", orderId, note, null);
            db.ExecuteNonQuery(cmd);
        }

        public static void Restock(int orderId) {
            Order o = Order.Get(orderId);
            foreach (OrderItem item in o.OrderItems) {
                OrderItem.Restock(item.OrderItemId);
            }
        }

        private static void SetOrderDBRelations(DataSet ds) {
            int index = 0;
            ds.Tables[index++].TableName = "Order";
            ds.Tables[index++].TableName = "OrderStatus";
            ds.Tables[index++].TableName = "OrderTotal";

            ds.Relations.Add(Constant.DBRelation.ORDER_ORDERSTATUS, ds.Tables["OrderStatus"].Columns["orderStatusId"],
                ds.Tables["Order"].Columns["orderStatusId"]);
            ds.Relations.Add(Constant.DBRelation.ORDER_ORDERTOTAL, ds.Tables["Order"].Columns["orderId"],
                ds.Tables["OrderTotal"].Columns["orderId"]);

            index = Address.SetTableRelations(ds, index);
            ds.Relations.Add(Constant.DBRelation.ORDER_SHIPPINGADDRESS, ds.Tables["Address"].Columns["addressId"],
                ds.Tables["Order"].Columns["shippingAddressId"]);
            ds.Relations.Add(Constant.DBRelation.ORDER_BILLINGADDRESS, ds.Tables["Address"].Columns["addressId"],
                ds.Tables["Order"].Columns["billingAddressId"]);
            
            ds.Tables[index++].TableName = "CheckoutPaymentType";
            ds.Relations.Add(Constant.DBRelation.ORDER_CHECKOUTPAYMENTTYPE, ds.Tables["CheckoutPaymentType"].Columns["checkoutPaymentTypeId"],
                ds.Tables["Order"].Columns["checkoutPaymentTypeId"]);
            

            if (ds.Tables.Count > index) {
                // more tables

                ds.Tables[index++].TableName = "OrderItem";
                ds.Tables[index++].TableName = "ShipTracking";
                ds.Tables[index++].TableName = "ShipCarrier";
                ds.Tables[index++].TableName = "OrderNote";
                ds.Tables[index++].TableName = "PaymentTransaction";
                ds.Relations.Add(Constant.DBRelation.ORDER_ORDERITEM, ds.Tables["Order"].Columns["orderId"],
                    ds.Tables["OrderItem"].Columns["orderId"]);
                ds.Relations.Add(Constant.DBRelation.ORDERITEM_SHIPTRACKING, ds.Tables["ShipTracking"].Columns["shipTrackingId"],
                    ds.Tables["OrderItem"].Columns["shipTrackingId"]);
                ds.Relations.Add(Constant.DBRelation.SHIPTRACKING_SHIPCARRIER, ds.Tables["ShipCarrier"].Columns["carrierId"],
                    ds.Tables["ShipTracking"].Columns["carrierId"]);
                ds.Relations.Add(Constant.DBRelation.ORDER_ORDERNOTE, ds.Tables["Order"].Columns["orderId"],
                    ds.Tables["OrderNote"].Columns["orderId"]);
                ds.Relations.Add(Constant.DBRelation.ORDER_PAYMENTTRANSACTION, ds.Tables["Order"].Columns["orderId"],
                    ds.Tables["PaymentTransaction"].Columns["orderId"]);

                // Credit card tables
                index = CreditCard.SetCreditCardQueryDBRelations(ds, index);
                ds.Relations.Add(Constant.DBRelation.ORDER_CREDITCARD, ds.Tables["Order"].Columns["orderId"],
                    ds.Tables["CreditCard"].Columns["orderId"]);
            }
        }

        public static string Add(string userName, decimal discountRate, string shippingMethod, Address shippingAddress, 
            CheckoutPaymentTypeEnum checkoutPaymentType, CreditCard creditCard, List<OrderTotal> orderTotals, int? orderStatusId) {
            string email = Membership.GetUser(userName).Email;
            string userIp = HttpContext.Current.Request.UserHostAddress;
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_Add", userName, email, userIp, discountRate, shippingMethod,
                ConvertAddressValues(shippingAddress), (int)checkoutPaymentType, checkoutPaymentType.ToString(), //Use checkoutPaymentType as OrderPipeline
                checkoutPaymentType == CheckoutPaymentTypeEnum.CreditCard ? ConvertCreditCardValues(creditCard) : null,
                ConvertOrderTotalValues(orderTotals), orderStatusId);
            object output = db.ExecuteScalar(cmd);
            return output.ToString();
        }

        public static string Add(string userName, string email, string shippingMethod, Address shippingAddress, Address billingAddress,
            CheckoutPaymentTypeEnum checkoutPaymentType, CreditCard creditCard, List<OrderTotal> orderTotals, List<CartItem> items, 
            string customField, int orderStatusId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_AddByExternalData", userName, email, shippingMethod,
                ConvertAddressValues(shippingAddress), ConvertAddressValues(billingAddress), (int)checkoutPaymentType, checkoutPaymentType.ToString(), //Use checkoutPaymentType as OrderPipeline
                checkoutPaymentType == CheckoutPaymentTypeEnum.CreditCard ? ConvertCreditCardValues(creditCard) : null,
                ConvertOrderTotalValues(orderTotals), ConvertCartItemValues(items), customField, orderStatusId);
            object output = db.ExecuteScalar(cmd);
            return output.ToString();
        }

        public static string AddRecurring(string host, int orderItemId, decimal price, string customField, string email, string shippingMethod, 
            Address shippingAddress, Address billingAddress, CheckoutPaymentTypeEnum checkoutPaymentType, CreditCard creditCard, 
            List<OrderTotal> orderTotals, int orderStatusId) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_AddRecurring", orderItemId, price, customField, email, shippingMethod,
                ConvertAddressValues(shippingAddress), ConvertAddressValues(billingAddress), (int)checkoutPaymentType, Constant.OrderPipeline.RecurringOrderPipeline,
                checkoutPaymentType == CheckoutPaymentTypeEnum.CreditCard ? ConvertCreditCardValues(creditCard) : null,
                ConvertOrderTotalValues(orderTotals), orderStatusId);
            object output = db.ExecuteScalar(cmd);
            return output.ToString();
        }
        
        public static string AddByMigraton(string userName, string email, string userIp, string orderNumber, DateTime orderPlaced, string shippingMethod, Address shippingAddress, 
            CheckoutPaymentTypeEnum checkoutPaymentType, List<OrderTotal> orderTotals) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_AddByMigration", userName, email, userIp, orderNumber, orderPlaced, shippingMethod,
                ConvertAddressValues(shippingAddress), (int)checkoutPaymentType, checkoutPaymentType.ToString(), //Use checkoutPaymentType as OrderPipeline
                ConvertOrderTotalValues(orderTotals));
            object output = db.ExecuteScalar(cmd);
            return output.ToString();
        }

        public static void UpdateAddress(int orderId, bool isBillingAddress, Address address) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Order_UpdateAddress", orderId, isBillingAddress, address.FirstName, address.LastName, address.CompanyName,
                address.Address1, address.Address2, address.City, address.State, address.Zipcode, address.CountryId, address.Phone, address.PhoneExt);
            db.ExecuteNonQuery(cmd);
        }

        #endregion

        #region Helper Methods
        private static string ConvertAddressValues(Address address) {
            StringBuilder sb = new StringBuilder();

            sb.Append(Utils.EscapeSeparators(address.FirstName) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(address.LastName) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(Utils.ToDBNullString(address.CompanyName)) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(address.Address1) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(Utils.ToDBNullString(address.Address2)) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(address.City) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(address.State) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(address.Zipcode) + Constant.DB.FieldSeparator.ToString());
            sb.Append(address.CountryId.ToString() + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(address.Phone) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(Utils.ToDBNullString(address.PhoneExt)));
            return sb.ToString();
        }

        private static string ConvertCreditCardValues(CreditCard creditCard) {
            StringBuilder sb = new StringBuilder();

            sb.Append(Utils.EscapeSeparators(creditCard.EncryptedCardNumber) + Constant.DB.FieldSeparator.ToString());
            sb.Append(Utils.EscapeSeparators(creditCard.CardholderName) + Constant.DB.FieldSeparator.ToString());
            sb.Append(creditCard.ExpireYear + Constant.DB.FieldSeparator.ToString());
            sb.Append(creditCard.ExpireMonth + Constant.DB.FieldSeparator.ToString());
            sb.Append(creditCard.CVV2 + Constant.DB.FieldSeparator.ToString());
            sb.Append(creditCard.Phone + Constant.DB.FieldSeparator.ToString());
            sb.Append(creditCard.CreditCardType.CreditCardTypeId.ToString());
            return sb.ToString();
        }

        private static string ConvertOrderTotalValues(List<OrderTotal> orderTotals) {
            StringBuilder sb = new StringBuilder();
            bool first = true;
            foreach (OrderTotal orderTotal in orderTotals) {
                if (first) {
                    first = false;
                } else {
                    sb.Append(Constant.DB.ListSeparator.ToString());
                }
                sb.Append(Utils.EscapeSeparators(orderTotal.Name) + Constant.DB.FieldSeparator.ToString());
                sb.Append(Utils.EscapeSeparators(Utils.ToDBNullString(orderTotal.Note)) + Constant.DB.FieldSeparator.ToString());
                sb.Append(orderTotal.Amount.ToString());
            }
            return sb.ToString();
        }

        private static string ConvertCartItemValues(List<CartItem> items) {
            StringBuilder sb = new StringBuilder();
            bool first = true;
            foreach (CartItem item in items) {
                if (first) {
                    first = false;
                } else {
                    sb.Append(Constant.DB.ListSeparator.ToString());
                }
                sb.Append(Utils.EscapeSeparators(item.SKU) + Constant.DB.FieldSeparator.ToString());
                sb.Append(Utils.EscapeSeparators(Utils.ToDBNullString(item.AttributeValueIds)) + Constant.DB.FieldSeparator.ToString());
                sb.Append(item.Quantity.ToString() + Constant.DB.FieldSeparator.ToString());
                sb.Append(Utils.EscapeSeparators(Utils.ToDBNullString(item.CustomField)));
            }
            return sb.ToString();
        }        
        #endregion
    }
}
