//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Data.Common;
using System.Text;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text.RegularExpressions;
using System.Web;
using Microsoft.Practices.EnterpriseLibrary.Data;
using XShan.DotShoppingCart.Common;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Common.Object {
    public enum ProductListType {
        Featured = 0,
        New = 1,
        Sale = 2
    }

    public class Product : ILoadFromDataRow {
        #region Member Variables
        private int productId, sortOrder;
        private string sku, productName, shortDescription, manufacturerSKU;
        private int? defaultImageId, quantity;
        private decimal? retailPrice, salePrice, cost, weight, height, length, width;
        private decimal price;
        private bool? showIfOutofStock;
        private DateTime timeCreated;
        private Manufacturer manufacturer;
        private ProductStatus status;
        private ProductType productType;
        private ShipType shipType;
        private ShipEstimate shipEstimate;
        private TaxType taxType;
        private Measure weightMeasure = null, dimensionMeasure = null;
        private ProductImage[] images;
        private List<ProductAttribute> productAttributes;
        private List<ProductDescriptor> productDescriptors;
        private Category[] categories = null;
        private bool useAttributeImageInCart;
        private List<ProductReview> reviews = null;
        #endregion

        #region Properties
        public int ProductId {
            get { return productId; }
        }
        public string SKU {
            get { return sku; }
        }
        public string ProductName {
            get { return productName; }
        }
        public string ShortDescription {
            get { return shortDescription; }
        }
        public string ManufacturerSKU {
            get { return manufacturerSKU; }
        }
        public int? DefaultImageId {
            get { return defaultImageId; }
        }
        public int? Quantity {
            get { return quantity; }
        }
        public decimal? RetailPrice {
            get { return retailPrice; }
        }
        public decimal Price {
            get { return price; }
        }
        public decimal? SalePrice {
            get { return salePrice; }
        }
        public decimal? Cost {
            get { return cost; }
        }
        public decimal? Weight {
            get { return weight; }
        }
        public decimal? Height {
            get { return height; }
        }
        public decimal? Length {
            get { return length; }
        }
        public decimal? Width {
            get { return width; }
        }
        public int SortOrder {
            get { return sortOrder; }
        }
        public bool? ShowIfOutofStock {
            get { return showIfOutofStock; }
        }
        public DateTime TimeCreated {
            get { return timeCreated; }
        }
        public bool IsNew {
            get { return timeCreated + TimeSpan.FromDays((double)Constant.UI.NewItemTimespan) > DateTime.Now; }
        }
        public Manufacturer Manufacturer {
            get { return manufacturer; }
        }
        public ProductStatus Status {
            get { return status; }
        }
        public ProductType ProductType {
            get { return productType; }
        }
        public ShipType ShipType {
            get { return shipType; }
        }
        public ShipEstimate ShipEstimate {
            get { return shipEstimate; }
        }
        public TaxType TaxType {
            get { return taxType; }
        }
        public Measure WeightMeasure {
            get { return weightMeasure; }
        }
        public Measure DimensionMeasure {
            get { return dimensionMeasure; }
        }
        public ProductImage[] Images {
            get { return images; }
        }
        public List<ProductAttribute> ProductAttributes {
            get { return productAttributes; }
        }
        public List<ProductDescriptor> ProductDescriptors {
            get { return productDescriptors; }
        }
        public bool UseAttributeImageInCart {
            get { return useAttributeImageInCart; }
        }
        public List<ProductReview> Reviews {
            get {
                if (null == reviews) {
                    reviews = ProductReview.GetByProductId(productId);
                }
                return reviews;
            }
        }
        public decimal OurPrice {
            get {
                if (null != salePrice)
                    return (decimal)salePrice;
                return price;
            }
        }
        public decimal SaveAmount {
            get {
                if (null != salePrice)
                    return price - (decimal)salePrice;
                if (null != retailPrice)
                    return (decimal)retailPrice - price;
                return 0;
            }
        }
        public double SavePercentage {
            get {
                if (null != salePrice)
                    return (double)SaveAmount / (double)price;
                if (null != retailPrice)
                    return (double)SaveAmount / (double)retailPrice;
                return 0;
            }
        }
        public decimal FinalPrice {
            get {
                return WebUtils.IsDiscountGroup() ? WebUtils.GetDiscountPrice(price) : OurPrice;
            }
        }
        public bool HasRequiredAttribute {
            get {
                if (null != productAttributes) {
                    foreach (ProductAttribute pa in productAttributes) {
                        if (pa.Attribute.IsRequired)
                            return true;
                    }
                }
                return false;
            }
        }

        // Properties needed for two way binding Bind method
        public int StatusId {
            get { return Status.StatusId; }
        }
        public int ShipTypeId {
            get { return ShipType.ShipTypeId; }
        }
        public int TaxTypeId {
            get { return TaxType.TaxTypeId; }
        }
        public int ProductTypeId {
            get { return ProductType.ProductTypeId; }
        }
        #endregion

        #region Constructors
        public Product() {
        }

        public bool LoadFromDataRow(DataRow row) {
            productId = (int)row["productId"];
            sku = (string)row["sku"];
            productName = (string)row["productName"];
            shortDescription = row["shortDescription"] as string;
            manufacturerSKU = row["manufacturerSKU"] as string;
            defaultImageId = DBHelper.ToNullableInt(row["defaultImageId"]);
            quantity = DBHelper.ToNullableInt(row["quantity"]);
            retailPrice = DBHelper.ToNullableDecimal(row["retailPrice"]);
            price = (decimal)row["price"];
            salePrice = DBHelper.ToNullableDecimal(row["salePrice"]);
            cost = DBHelper.ToNullableDecimal(row["cost"]);
            weight = DBHelper.ToNullableDecimal(row["weight"]);
            height = DBHelper.ToNullableDecimal(row["height"]);
            length = DBHelper.ToNullableDecimal(row["length"]);
            width = DBHelper.ToNullableDecimal(row["width"]);
            sortOrder = (int)row["sortOrder"];
            timeCreated = (DateTime)row["timeCreated"];
            showIfOutofStock = DBHelper.ToNullableBoolean(row["showIfOutofStock"]);
            useAttributeImageInCart = (bool)row["useAttributeImageInCart"];

            // Manufacturer
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_MANUFACTURER)) {
                // expect only one record
                manufacturer = new RecordList<Manufacturer>(row.GetParentRows(Constant.DBRelation.PRODUCT_MANUFACTURER))[0];
            } else {
                throw new ApplicationException("Manufacturer data is expected when loading Product.");
            }

            // ProductStatus
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_STATUS)) {
                // expect only one record
                status = new RecordList<ProductStatus>(row.GetParentRows(Constant.DBRelation.PRODUCT_STATUS))[0];
            } else {
                throw new ApplicationException("ProductStatus data is expected when loading Product.");
            }

            // ProductType
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_PRODUCTTYPE)) {
                // expect only one record
                productType = new RecordList<ProductType>(row.GetParentRows(Constant.DBRelation.PRODUCT_PRODUCTTYPE))[0];
            } else {
                throw new ApplicationException("ProductType data is expected when loading Product.");
            }

            // ShipType
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_SHIPTYPE)) {
                // expect only one record
                shipType = new RecordList<ShipType>(row.GetParentRows(Constant.DBRelation.PRODUCT_SHIPTYPE))[0];
            } else {
                throw new ApplicationException("ShipType data is expected when loading Product.");
            }

            // ShipEstimate
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_SHIPESTIMATE)) {
                // expect only one record
                shipEstimate = new RecordList<ShipEstimate>(row.GetParentRows(Constant.DBRelation.PRODUCT_SHIPESTIMATE))[0];
            } else {
                throw new ApplicationException("ShipEstimate data is expected when loading Product.");
            }

            // TaxType
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_TAXTYPE)) {
                // expect only one record
                taxType = new RecordList<TaxType>(row.GetParentRows(Constant.DBRelation.PRODUCT_TAXTYPE))[0];
            } else {
                throw new ApplicationException("TaxType data is expected when loading Product.");
            }

            // WeightMeasure
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_MEASURE_WEIGHT)) {
                // expect only one record
                RecordList<Measure_Weight> measures = new RecordList<Measure_Weight>(row.GetParentRows(Constant.DBRelation.PRODUCT_MEASURE_WEIGHT));
                if ((null != measures) && (measures.Count > 0)) {
                    weightMeasure = measures[0];
                }
            } else {
                throw new ApplicationException("WeightMeasure data is expected when loading Product.");
            }

            // DimensionMeasure
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_MEASURE_DIMENSION)) {
                // expect only one record
                RecordList<Measure_Dimension> measures = new RecordList<Measure_Dimension>(row.GetParentRows(Constant.DBRelation.PRODUCT_MEASURE_DIMENSION));
                if ((null != measures) && (measures.Count > 0)) {
                    dimensionMeasure = measures[0];
                }
            } else {
                throw new ApplicationException("DimensionMeasure data is expected when loading Product.");
            }

            // Optional
            // ProductImage
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_IMAGE)) {
                List<ProductImage> images = new List<ProductImage>();
                foreach (DataRow imageRow in row.GetChildRows(Constant.DBRelation.PRODUCT_IMAGE)) {
                    images.Add(new ProductImage((int)imageRow["imageId"], imageRow["imageTitle"] as string));
                }
                this.images = images.ToArray();
                // set default image
                foreach (ProductImage pi in images) {
                    if (pi.ImageId == defaultImageId) {
                        pi.IsDefault = true;
                    }
                }
            } 

            // ProductAttribute
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_ATTRIBUTE)) {
                productAttributes = new RecordList<ProductAttribute>(row.GetChildRows(Constant.DBRelation.PRODUCT_ATTRIBUTE));
            } 

            // ProductDescriptor
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.PRODUCT_DESCRIPTOR)) {
                productDescriptors = new RecordList<ProductDescriptor>(row.GetChildRows(Constant.DBRelation.PRODUCT_DESCRIPTOR));
            }

            return true;
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Get list of Categories that the product belongs to.
        /// Note the category object is shallow object which doesn't have subdirectories and all the optional properties
        /// </summary>
        /// <returns></returns>
        public Category[] GetCategories() {
            if (null == categories) {
                categories = GetCategories(sku);
            }
            return categories;
        }
        
        public void AddTokens(NameValueCollection tokens) {
            tokens.Add("Product.ShortDescription", shortDescription);
            tokens.Add("Product.SKU", sku);
            tokens.Add("Product.Url", SiteNavigation.GetRewriterUrl("Product", sku));
            tokens.Add("Product.OurPrice", OurPrice.ToString("C"));

            tokens.Add("Product.ImageUrl", SiteNavigation.ResolveAbsoluteUrl(false, FileController.GetImageUrl(defaultImageId, 
                UIConfigurationSection.GetSection().ProductSummaryImageWidth, 
                UIConfigurationSection.GetSection().ProductSummaryImageHeight)));
        }

        #endregion

        #region Static Public Methods
        public static Product Load(int productId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_GetDetailByProductId", productId);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetProductTableRelations(ds);

            return new RecordList<Product>(ds.Tables["Product"])[0];
        }

        public static Product Load(string sku) {
            return Load(SiteNavigation.GetHost(), sku);
        }

        public static Product Load(string host, string sku) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_GetDetailByProductSKU", sku);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetProductTableRelations(ds);
            RecordList<Product> products = new RecordList<Product>(ds.Tables["Product"]);
            if ((null == products) || (0 == products.Count))
                return null;
            return products[0];
        }

        public static List<Product> Load(List<int> productIds) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_GetProducts", Utils.ConvertIdsToSting(productIds));
            DataSet ds = db.ExecuteDataSet(cmd);

            SetProductTableRelations(ds);

            return new RecordList<Product>(ds.Tables["Product"]);
        }

        private static string GetProductCountCacheKey(int categoryId, string keywords) {
            return string.Format("ProductSearch_{0}_{1}_{2}", SiteNavigation.GetHost(), categoryId, keywords);
        }

        public static List<Product> Search(string keywords, bool showOutofStock) {
            return Search(0, keywords, 50, 0, null, showOutofStock);
        }

        public static List<Product> Search(int categoryId, string keywords, int productPerPage, int rowIndex, string sortBy) {
            return Search(categoryId, keywords, productPerPage, rowIndex, sortBy, null);
        }

        public static List<Product> Search(int categoryId, string keywords, int productPerPage, int rowIndex, string sortBy, bool? showOutofStock) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_Search");
            if (0 < categoryId)
                db.AddInParameter(cmd, "@categoryId", DbType.Int32, categoryId);
            db.AddInParameter(cmd, "@keywords", DbType.String, Utils.ConvertToFullTextSearchString(keywords));
            db.AddInParameter(cmd, "@productsPerPage", DbType.Int32, productPerPage);
            db.AddInParameter(cmd, "@rowIndex", DbType.Int32, rowIndex);

            bool asc = true;
            if (!string.IsNullOrEmpty(sortBy)) {
                if (sortBy.EndsWith(" DESC")) {
                    sortBy = sortBy.Substring(0, sortBy.Length - 5);
                    asc = false;
                }

                db.AddInParameter(cmd, "@orderBy", DbType.String, sortBy);
            }
            db.AddInParameter(cmd, "@asc", DbType.Boolean, asc);
            db.AddInParameter(cmd, "@showOutofStock", DbType.Boolean, showOutofStock);
            db.AddOutParameter(cmd, "@totalProducts", DbType.Int32, sizeof(Int32));

            DataSet ds = db.ExecuteDataSet(cmd);
            int totalNumber = (int)db.GetParameterValue(cmd, "@totalProducts");

            SetProductTableRelations(ds);

            List<Product> products = new RecordList<Product>(ds.Tables["Product"]);

            if (null != HttpContext.Current)
                HttpContext.Current.Items[GetProductCountCacheKey(categoryId, keywords)] = totalNumber;
            return products;
        }

        public static int GetAllProductCount() {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_AllProductCount");
            object output = db.ExecuteScalar(cmd);
            return int.Parse(output.ToString());
        }

        public static int GetProductCount(int categoryId, string keywords) {
            int count = 0;
            if ((null != HttpContext.Current) &&
                (null != HttpContext.Current.Items[GetProductCountCacheKey(categoryId, keywords)]))
                count = (int)HttpContext.Current.Items[GetProductCountCacheKey(categoryId, keywords)];
            return count;
        }

        public static List<Product> GetProducts(string categoryPath, int productPerPage, int rowIndex,
            string manufacturerFilter, decimal? priceFrom, decimal? priceTo, bool showOutofStock, out int totalNumber) {
            return GetProducts(categoryPath, productPerPage, rowIndex, ProductSortBy.FeaturedItem, 
                manufacturerFilter, priceFrom, priceTo, showOutofStock, out totalNumber);
        }

        public static List<Product> GetProducts(string categoryPath, int productPerPage, int rowIndex, ProductSortBy sortBy,
            string manufacturerFilter, decimal? priceFrom, decimal? priceTo, bool showOutofStock, out int totalNumber) {
            return GetProducts(SiteNavigation.GetHost(), categoryPath, productPerPage, rowIndex, sortBy,
                manufacturerFilter, priceFrom, priceTo, showOutofStock, out totalNumber);
        }

        public static List<Product> GetProducts(string host, string categoryPath, int productPerPage, int rowIndex, ProductSortBy sortBy,
            string manufacturerFilter, decimal? priceFrom, decimal? priceTo, bool showOutofStock, out int totalNumber) {
            totalNumber = 0;
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = null;  
            switch (sortBy) {
                case ProductSortBy.FeaturedItem:
                    cmd = db.GetStoredProcCommand("DSC_Product_GetByCategoryPath_OrderByFeatured");
                    break;
                case ProductSortBy.AlphabeticalAsc:
                case ProductSortBy.AlphabeticalDesc:
                    cmd = db.GetStoredProcCommand("DSC_Product_GetByCategoryPath_OrderByProductName");
                    if (ProductSortBy.AlphabeticalDesc == sortBy)
                        db.AddInParameter(cmd, "@ASC", DbType.Boolean, 0);
                    break;
                case ProductSortBy.PriceAesc:
                case ProductSortBy.PriceDesc:
                    cmd = db.GetStoredProcCommand("DSC_Product_GetByCategoryPath_OrderByPrice");
                    if (ProductSortBy.PriceDesc == sortBy)
                        db.AddInParameter(cmd, "@ASC", DbType.Boolean, 0);
                    break;
            }
            db.AddInParameter(cmd, "@CategoryPath", DbType.String, categoryPath);
            db.AddInParameter(cmd, "@showOutofStock", DbType.Boolean, showOutofStock);
            AddCommonParameters(db, cmd, productPerPage, rowIndex, manufacturerFilter, priceFrom, priceTo);
            DataSet ds = db.ExecuteDataSet(cmd);
            totalNumber = (int)db.GetParameterValue(cmd, "@TotalProducts");

            SetProductTableRelations(ds);

            return new RecordList<Product>(ds.Tables["Product"]);
        }

        public static List<Product> GetProducts(ProductListType listType, int productPerPage, int rowIndex, ProductSortBy sortBy,
            string manufacturerFilter, decimal? priceFrom, decimal? priceTo, bool showOutofStock, out int totalNumber) {
            totalNumber = 0;
            Database db = DBController.CreateDatabase();
            DbCommand cmd = null;
            switch (sortBy) {
                case ProductSortBy.FeaturedItem:
                    cmd = db.GetStoredProcCommand("DSC_Product_GetByListType_OrderByFeatured");
                    break;
                case ProductSortBy.AlphabeticalAsc:
                case ProductSortBy.AlphabeticalDesc:
                    cmd = db.GetStoredProcCommand("DSC_Product_GetByListType_OrderByProductName");
                    if (ProductSortBy.AlphabeticalDesc == sortBy)
                        db.AddInParameter(cmd, "@ASC", DbType.Boolean, 0);
                    break;
                case ProductSortBy.PriceAesc:
                case ProductSortBy.PriceDesc:
                    cmd = db.GetStoredProcCommand("DSC_Product_GetByListType_OrderByPrice");
                    if (ProductSortBy.PriceDesc == sortBy)
                        db.AddInParameter(cmd, "@ASC", DbType.Boolean, 0);
                    break;
            }
            db.AddInParameter(cmd, "@listType", DbType.Int32, listType);
            db.AddInParameter(cmd, "@showOutofStock", DbType.Boolean, showOutofStock);
            AddCommonParameters(db, cmd, productPerPage, rowIndex, manufacturerFilter, priceFrom, priceTo);
            DataSet ds = db.ExecuteDataSet(cmd);
            totalNumber = (int)db.GetParameterValue(cmd, "@TotalProducts");

            SetProductTableRelations(ds);

            return new RecordList<Product>(ds.Tables["Product"]);
        }

        private static void AddCommonParameters(Database db, DbCommand cmd, int productPerPage, int rowIndex, string manufacturerFilter, decimal? priceFrom, decimal? priceTo) {
            db.AddInParameter(cmd, "@ProductsPerPage", DbType.Int32, productPerPage);
            db.AddInParameter(cmd, "@rowIndex", DbType.Int32, rowIndex);
            if (!string.IsNullOrEmpty(manufacturerFilter)) {
                db.AddInParameter(cmd, "@Manufacturer", DbType.String, manufacturerFilter);
            }
            if (priceFrom != null) {
                db.AddInParameter(cmd, "@PriceFrom", DbType.Currency, priceFrom);
            }
            if (priceTo != null) {
                db.AddInParameter(cmd, "@PriceTo", DbType.Currency, priceTo);
            }
            db.AddOutParameter(cmd, "@TotalProducts", DbType.Int32, sizeof(Int32));
        }


        private static void SetProductTableRelations(DataSet ds) {
            SetProductTableRelations(ds, 0);
        }

        public static void SetProductTableRelations(DataSet ds, int startTableIndex) {
            int i = startTableIndex;
            ds.Tables[i++].TableName = "Product";
            ds.Tables[i++].TableName = "Manufacturer";
            ds.Tables[i++].TableName = "Status";
            ds.Tables[i++].TableName = "ProductType";
            ds.Tables[i++].TableName = "ShipType";
            ds.Tables[i++].TableName = "ShipEstimate";
            ds.Tables[i++].TableName = "TaxType";
            ds.Tables[i++].TableName = "WeightMeasure";
            ds.Tables[i++].TableName = "DimensionMeasure";
            

            ds.Relations.Add(Constant.DBRelation.PRODUCT_MANUFACTURER, ds.Tables["Manufacturer"].Columns["manufacturerId"], ds.Tables["Product"].Columns["manufacturerId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_STATUS, ds.Tables["Status"].Columns["statusId"], ds.Tables["Product"].Columns["statusId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_PRODUCTTYPE, ds.Tables["ProductType"].Columns["productTypeId"], ds.Tables["Product"].Columns["productTypeId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_SHIPTYPE, ds.Tables["ShipType"].Columns["shipTypeId"], ds.Tables["Product"].Columns["shipTypeId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_SHIPESTIMATE, ds.Tables["ShipEstimate"].Columns["shipEstimateId"], ds.Tables["Product"].Columns["shipEstimateId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_TAXTYPE, ds.Tables["TaxType"].Columns["taxTypeId"], ds.Tables["Product"].Columns["taxTypeId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_MEASURE_WEIGHT, ds.Tables["WeightMeasure"].Columns["Id"], ds.Tables["Product"].Columns["weightMeasureId"]);
            ds.Relations.Add(Constant.DBRelation.PRODUCT_MEASURE_DIMENSION, ds.Tables["DimensionMeasure"].Columns["Id"], ds.Tables["Product"].Columns["dimensionMeasureId"]);

            if (ds.Tables.Count > i) {
                // more product tables
                ds.Tables[i++].TableName = "ProductImage";
                ds.Tables[i++].TableName = "ProductAttribute";
                ds.Tables[i++].TableName = "ProductAttributeValue";

                ds.Relations.Add(Constant.DBRelation.PRODUCT_IMAGE, ds.Tables["Product"].Columns["productId"], ds.Tables["ProductImage"].Columns["productId"]);
                ds.Relations.Add(Constant.DBRelation.PRODUCT_ATTRIBUTE, ds.Tables["Product"].Columns["productId"], ds.Tables["ProductAttribute"].Columns["productId"]);
                ds.Relations.Add(Constant.DBRelation.PRODUCT_ATTRIBUTEVALUE, ds.Tables["ProductAttribute"].Columns["productAttributeId"], ds.Tables["ProductAttributeValue"].Columns["productAttributeId"]);

                i = DSCAttribute.SetAttributeTableRelations(ds, i);

                ds.Tables[i++].TableName = "ProductDescriptor";

                ds.Relations.Add(Constant.DBRelation.PRODUCTATTRIBUTE_ATTRIBUTE, ds.Tables["Attribute"].Columns["attributeId"], ds.Tables["ProductAttribute"].Columns["attributeId"]);
                ds.Relations.Add(Constant.DBRelation.PRODUCT_DESCRIPTOR, ds.Tables["Product"].Columns["productId"], ds.Tables["ProductDescriptor"].Columns["productId"]);
            }
        }

        public static Category[] GetCategories(string sku) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Category_GetProductCategories", sku);
            DataSet ds = db.ExecuteDataSet(cmd);

            Category.SetCategoryRelations(ds, ds.Tables.Count);

            RecordList<Category> categories = new RecordList<Category>(ds.Tables[0]);
            return categories.ToArray();
        }

        public static void Update(int productId, string productName, string sku, string shortDescription, string quantity,
            decimal price, string retailPrice, string salePrice, string cost, int manufacturerId, string manufacturerSKU, int statusId, int productTypeId, int shipTypeId, int shipEstimateId,
            int taxTypeId, string weight, int? weightMeasureId, string length, string height, string width, int? dimensionMeasureId,
            string sortOrder, string timeCreated, string showIfOutofStock, ICollection<CategoryInformation> categories, ProductImage[] images, List<ProductDescriptor> descriptors,
            List<ProductAttribute> attributes) {

            string categoriesValues = ConvertFeaturedCategoriesValues(categories);
            //Change input arguments to the correct type and change empty string to null if neccessary
            int? defaultImageId;
            string imageIds, imageTitleValues;
            ConvertImageIdValues(images, out defaultImageId, out imageIds, out imageTitleValues);

            string descriptorTitleValues, descriptorDescriptionValues, isBulletedListValues;
            ConvertDescriptorValues(descriptors, out descriptorTitleValues, out descriptorDescriptionValues, out isBulletedListValues);

            string attributeValues = ConvertProductAttributeValues(attributes);

            Update(productId, productName, sku, Utils.StringChangeEmptyToNull(shortDescription), Utils.ConvertNullableFromString<int?>(quantity),
                price, Utils.ConvertNullableFromString<decimal?>(retailPrice), Utils.ConvertNullableFromString<decimal?>(salePrice),
                Utils.ConvertNullableFromString<decimal?>(cost), manufacturerId, Utils.StringChangeEmptyToNull(manufacturerSKU),
                statusId, productTypeId, shipTypeId, shipEstimateId, taxTypeId, Utils.ConvertNullableFromString<decimal?>(weight), weightMeasureId,
                Utils.ConvertNullableFromString<decimal?>(length), Utils.ConvertNullableFromString<decimal?>(height),
                Utils.ConvertNullableFromString<decimal?>(width), dimensionMeasureId,
                Utils.ChangeBadConversionToDefault(sortOrder, 0), Utils.ConvertNullableFromString<DateTime?>(timeCreated), Utils.ConvertNullableFromString<bool?>(showIfOutofStock),
                categoriesValues, defaultImageId, imageIds, imageTitleValues,
                descriptorTitleValues, descriptorDescriptionValues, isBulletedListValues, attributeValues);
        }

        public static void Update(int productId, string productName, string sku, string shortDescription, int? quantity,
            decimal price, decimal? retailPrice, decimal? salePrice, decimal? cost, int manufacturerId, string manufacturerSKU, int statusId, int productTypeId, int shipTypeId, int shipEstimateId,
            int taxTypeId, decimal? weight, int? weightMeasureId, decimal? length, decimal? height, decimal? width, int? dimensionMeasureId,
            int sortOrder, DateTime? timeCreated, bool? showIfOutofStock, string categoriesValues, int? defaultImageId, string imageIds, string imageTitleValues,
            string descriptorTitleValues, string descriptorDescriptionValues, string isBulletedListValues, string attributeValues) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_Update", productId, productName, sku, shortDescription, quantity,
                price, retailPrice, salePrice, cost, manufacturerId, manufacturerSKU, statusId, productTypeId, shipTypeId, shipEstimateId, taxTypeId, weight, weightMeasureId, length,
                height, width, dimensionMeasureId, sortOrder, timeCreated, showIfOutofStock, categoriesValues, defaultImageId, imageIds, imageTitleValues,
                descriptorTitleValues, descriptorDescriptionValues, isBulletedListValues, attributeValues);
            db.ExecuteNonQuery(cmd);
        }

        public static int Add(string productName, string sku, string shortDescription, string quantity,
            decimal price, string retailPrice, string salePrice, string cost, int manufacturerId, string manufacturerSKU, int statusId, int productTypeId, int shipTypeId, int shipEstimateId,
            int taxTypeId, string weight, int? weightMeasureId, string length, string height, string width, int? dimensionMeasureId,
            string sortOrder, string timeCreated, string showIfOutofStock, ICollection<CategoryInformation> categories, ProductImage[] images, List<ProductDescriptor> descriptors,
            List<ProductAttribute> attributes) {

            string categoriesValues = ConvertFeaturedCategoriesValues(categories);

            //Change input arguments to the correct type and change empty string to null if neccessary
            int? defaultImageId;
            string imageIds, imageTitleValues;
            ConvertImageIdValues(images, out defaultImageId, out imageIds, out imageTitleValues);

            string descriptorTitleValues, descriptorDescriptionValues, isBulletedListValues;
            ConvertDescriptorValues(descriptors, out descriptorTitleValues, out descriptorDescriptionValues, out isBulletedListValues);

            string attributeValues = ConvertProductAttributeValues(attributes);

            return Add(productName, sku, Utils.StringChangeEmptyToNull(shortDescription), Utils.ConvertNullableFromString<int?>(quantity),
                price, Utils.ConvertNullableFromString<decimal?>(retailPrice), Utils.ConvertNullableFromString<decimal?>(salePrice), Utils.ConvertNullableFromString<decimal?>(cost),
                manufacturerId, Utils.StringChangeEmptyToNull(manufacturerSKU),
                statusId, productTypeId, shipTypeId, shipEstimateId, taxTypeId, Utils.ConvertNullableFromString<decimal?>(weight), weightMeasureId,
                Utils.ConvertNullableFromString<decimal?>(length), Utils.ConvertNullableFromString<decimal?>(height),
                Utils.ConvertNullableFromString<decimal?>(width), dimensionMeasureId,
                Utils.ChangeBadConversionToDefault(sortOrder, 0), Utils.ConvertNullableFromString<DateTime?>(timeCreated), Utils.ConvertNullableFromString<bool?>(showIfOutofStock),
                categoriesValues, defaultImageId, imageIds, imageTitleValues,
                descriptorTitleValues, descriptorDescriptionValues, isBulletedListValues, attributeValues);
        }

        public static int Add(string productName, string sku, string shortDescription, int? quantity,
            decimal price, decimal? retailPrice, decimal? salePrice, decimal? cost, int manufacturerId, string manufacturerSKU, int statusId, int productTypeId, int shipTypeId, int shipEstimateId,
            int taxTypeId, decimal? weight, int? weightMeasureId, decimal? length, decimal? height, decimal? width, int? dimensionMeasureId,
            int sortOrder, DateTime? timeCreated, bool? showIfOutofStock, string categoriesValues, int? defaultImageId, string imageIds, string imageTitleValues,
            string descriptorTitleValues, string descriptorDescriptionValues, string isBulletedListValues, string attributeValues) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_Add", productName, sku, shortDescription, quantity,
                price, retailPrice, salePrice, cost, manufacturerId, manufacturerSKU, statusId, productTypeId, shipTypeId, shipEstimateId, taxTypeId, weight, weightMeasureId, length,
                height, width, dimensionMeasureId, sortOrder, timeCreated, showIfOutofStock, categoriesValues, defaultImageId, imageIds, imageTitleValues,
                descriptorTitleValues, descriptorDescriptionValues, isBulletedListValues, attributeValues);
            object output = db.ExecuteScalar(cmd);
            return int.Parse(output.ToString());
        }


        public static void Delete(int productId) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_Product_Delete", productId);
            db.ExecuteNonQuery(cmd);
        }

        public static string GetProductSummaryHtml(string host, string sku) {
            Product p = Load(host, sku);
            UIConfigurationSection uiSection = (UIConfigurationSection)SqlConfigurationController.GetSection(host, UIConfigurationSection.SectionName);
            int productBoxLength = uiSection.ProductSummaryImageWidth + 10;

            return
            "<table style='width:" + productBoxLength.ToString() + "px; height:" + productBoxLength.ToString() + "px; border-bottom:1px dashed #CCCCCC; margin: 0px; padding: 10px;'>" + 
                "<tr style='text-align:center;height:180px;'>" +
                    "<td><a href='" + SiteNavigation.GetHostRewriterUrl(host, "Product", p.SKU) + "'>" +
                        "<img style='border: 0px;' alt='" + p.ShortDescription + "' src='" + FileController.GetDbImageHttpHandlerUrl(host, p.DefaultImageId, uiSection.ProductSummaryImageWidth, uiSection.ProductSummaryImageHeight) + "' />" +
                    "</a></td>" +
                "</div>" +
                "<tr style='text-align:center; margin:4px 4px 4px 4px;'>" +
                    "<td><div>" +
                        "<a href='" + SiteNavigation.GetHostRewriterUrl(host, "Product", p.SKU) + "'>" + p.ProductName + "</a>" +
                        (p.IsNew ? "<span style='padding-left:20px; font-weight:bold; color:Red;'>New</span>" : "") +
                    "</div>" +
                    "<div>" + 
                        (null != p.SalePrice ?
                            string.Format("<span style='font-weight: bold; font-size: 10pt; font-family: verdana; text-decoration: line-through;'>{0:C}</span> " +
                                "<span style='font-weight: bold; font-size: 10pt; color: #b22222; font-family: verdana;'>{1:C} Save {2:0%} off</span>",
                                p.Price, (Decimal)p.SalePrice, p.SavePercentage) :
                            string.Format("<span style='font-weight: bold; font-size: 10pt; color: #b22222; font-family: verdana;'>{0:C}</span>", p.Price)) +
                    "</div><td>" +
                "</tr>" +
            "</table>";
        }
        #endregion

        #region Public Member Methods
        public List<ProductImage> GetProductImages(bool showOutofStock) {
            List<ProductImage> imgs = new List<ProductImage>(this.images);
            if (null != productAttributes) {
                foreach (ProductAttribute attrib in productAttributes) {
                    foreach (ProductAttributeValue value in attrib.GetAvailableValues(showOutofStock)) {
                        if (value.ImageId != null)
                            imgs.Add(new ProductImage((int)value.ImageId, value.Value));
                    }
                }
            }
            return imgs;
        }

        public int GetStockQuantity(int quantity, List<int> selectedAttributeValueIds) {
            int available = quantity;
            foreach (ProductAttribute attrib in this.productAttributes) {
                foreach (ProductAttributeValue value in attrib.Values) {
                    if (selectedAttributeValueIds.Contains(value.ProductAttributeValueId) &&
                        (null != value.Quantity) && ((int)value.Quantity < available)) 
                            available = (int)value.Quantity;
                }
            }
            if ((null != this.quantity) && ((int)this.quantity < available))
                available = (int)this.quantity;
            return available;
        }
        #endregion

        #region Private Methods
        private static void ConvertImageIdValues(ProductImage[] images, out int? defaultImageId, out string imageIds, out string imageTitleValues) {
            defaultImageId = null;
            bool firstRecord = true;
            StringBuilder imageIdsStringBuilder = new StringBuilder(), imageTitleValuesStringBuilder = new StringBuilder();
            foreach (ProductImage pi in images) {
                if (pi.IsDefault) {
                    defaultImageId = pi.ImageId;
                }
                if (!firstRecord) {
                    imageIdsStringBuilder.Append(Constant.DB.ListSeparator);
                    imageTitleValuesStringBuilder.Append(Constant.DB.ListSeparator);
                } else {
                    firstRecord = false;
                }
                imageIdsStringBuilder.Append(pi.ImageId);
                imageTitleValuesStringBuilder.Append(Utils.EscapeSeparators(Utils.ToDBNullString(pi.Title)));
            }
            imageIds = imageIdsStringBuilder.ToString();
            imageTitleValues = imageTitleValuesStringBuilder.ToString();
        }

        private static void ConvertDescriptorValues(List<ProductDescriptor> descriptors, out string descriptorTitleValues, 
            out string descriptorDescriptionValues, out string isBulletedListValues) {
            bool firstRecord = true;
            StringBuilder descriptorTitleValuesSB = new StringBuilder(), descriptorDescriptionValuesSB = new StringBuilder(), isBulletedListValuesSB = new StringBuilder();
            if (null != descriptors) {
                foreach (ProductDescriptor pd in descriptors) {
                    if (!firstRecord) {
                        descriptorTitleValuesSB.Append(Constant.DB.ListSeparator);
                        descriptorDescriptionValuesSB.Append(Constant.DB.ListSeparator);
                        isBulletedListValuesSB.Append(Constant.DB.ListSeparator);
                    } else {
                        firstRecord = false;
                    }
                    descriptorTitleValuesSB.Append(Utils.EscapeSeparators(pd.Title));
                    descriptorDescriptionValuesSB.Append(Utils.EscapeSeparators(pd.Descriptor));
                    isBulletedListValuesSB.Append(pd.IsBulletedList ? "1" : "0");
                }
            }
            descriptorTitleValues = descriptorTitleValuesSB.ToString();
            descriptorDescriptionValues = descriptorDescriptionValuesSB.ToString();
            isBulletedListValues = isBulletedListValuesSB.ToString();
        }

        private static string ConvertProductAttributeValues(List<ProductAttribute> attributes) {
            StringBuilder sb = new StringBuilder();
            bool first = true;
            if (null != attributes) {
                foreach (ProductAttribute attribute in attributes) {
                    if (first) {
                        first = false;
                    } else {
                        sb.Append(Constant.DB.ListSeparator.ToString());
                    }
                    sb.Append(attribute.Attribute.AttributeId.ToString());
                    if ((null == attribute.Values) || (0 == attribute.Values.Count))
                        throw new ApplicationException("Attribute Values must be present before product attribute can be updated or added");
                    foreach (ProductAttributeValue value in attribute.Values) {
                        sb.Append(Constant.DB.FieldSeparator.ToString());
                        sb.Append(Utils.EscapeSeparators(value.Value) + Constant.DB.FieldSeparator.ToString());
                        sb.Append(value.PriceAdjustment.ToString() + Constant.DB.FieldSeparator.ToString());
                        sb.Append(Utils.ToDBNullString(value.Quantity) + Constant.DB.FieldSeparator.ToString());
                        sb.Append(Utils.ToDBNullString(value.ImageId));
                    }
                }
            }
            return sb.ToString();
        }

        private static string ConvertFeaturedCategoriesValues(ICollection<CategoryInformation> categories) {
            StringBuilder sb = new StringBuilder();
            bool first = true;
            if (null != categories) {
                foreach (CategoryInformation c in categories) {
                    if (first) {
                        first = false;
                    } else {
                        sb.Append(Constant.DB.ListSeparator.ToString());
                    }
                    sb.Append(c.Id.ToString()).Append(Constant.DB.FieldSeparator.ToString());
                    sb.Append(c.IsFeatured ? "1" : "0").Append(Constant.DB.FieldSeparator.ToString());
                    sb.Append(c.FeatureSortOrder.ToString());
                }
            }
            return sb.ToString();
        }
        #endregion
    }
}
