//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Data.Common;
using System.Web;
using System.Web.Security;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace XShan.DotShoppingCart.Common.Object {
    public class User : ILoadFromDataRow {
        private int userId;
        private string userName;
        private DateTime created;
        private string email;
        private bool newsLetter;
        private int? referralId;
        private string otherReferralSource;
        private Address address;

        public int UserId {
            get { return userId; }
        }

        public string UserName {
            get { return userName; }
        }

        public DateTime Created {
            get { return created; }
        }

        public string Email {
            get { return email; }
        }

        public bool NewsLetter {
            get { return newsLetter; }
            set { newsLetter = value; }
        }

        public int? ReferralId {
            get { return referralId; }
            set { referralId = value; }
        }

        public string OtherReferralSource {
            get { return otherReferralSource; }
            set { otherReferralSource = value; }
        }

        public Address Address {
            get { return address; }
            set { address = value; }
        }

        public MembershipUser MembershipUser {
            get { return Membership.GetUser(userName); }
        }

        public string[] RoleList {
            get { return Roles.GetRolesForUser(userName); }
        }

        public User() {
        }

        public bool LoadFromDataRow(DataRow row) {
            userId = (int)row["userId"];
            userName = row["userName"] as string;
            created = (DateTime)row["created"];
            email = row["email"] as string;
            newsLetter = (bool)row["newsLetter"];
            referralId = null;
            if (row["referralId"] != DBNull.Value) {
                referralId = (int)row["referralId"];
            }
            otherReferralSource = row["otherReferralSource"] as string;

            //Address
            if (row.Table.DataSet.Relations.Contains(Constant.DBRelation.USER_ADDRESS)) {
                // expect only one record
                address = new RecordList<Address>(row.GetChildRows(Constant.DBRelation.USER_ADDRESS))[0];
            } else {
                throw new ApplicationException("Address data is expected when loading User.");
            }
            return true;
        }

        public static void PopulateCredentials(string host, string userName, string srcDB) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("StoreSetup_PopulateCredentials", userName, srcDB);
            db.ExecuteNonQuery(cmd);
        }

        public static int AddUser(string userName, bool newsLetter, int referralId, string otherReferralSource,
            string firstName, string lastName, string companyName, string address,
            string address2, string city, string state, string zipcode, int countryId, 
            string phone, string phoneExt, DateTime created) {

            object objReferralId = null;
            if (referralId > 0) {
                objReferralId = referralId;
            }

            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_Add", userName, newsLetter, objReferralId, 
                string.IsNullOrEmpty(otherReferralSource) ? null : otherReferralSource, firstName, lastName, 
                string.IsNullOrEmpty(companyName) ? null : companyName, address, string.IsNullOrEmpty(address2) ? null : address2,
                city, state, zipcode, countryId, phone, string.IsNullOrEmpty(phoneExt) ? null : phoneExt, created);

            return Convert.ToInt32(db.ExecuteScalar(cmd));
        }

        public static void Delete(string userName) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_Delete", userName);
            db.ExecuteNonQuery(cmd);
        }

        public static User GetUser(string userName) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_GetByUserName", userName);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetTableRelations(ds);

            RecordList<User> users = new RecordList<User>(ds.Tables["User"]);
            if (0 == users.Count)
                return null;
            return users[0];
        }

        public static User GetUser(string host, int userId) {
            Database db = DBController.CreateDatabase(host);
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_Get", userId);
            DataSet ds = db.ExecuteDataSet(cmd);

            SetTableRelations(ds);

            RecordList<User> users = new RecordList<User>(ds.Tables["User"]);
            if (0 == users.Count)
                return null;
            return users[0];
        }

        public static User GetUser(int userId) {
            return GetUser(SiteNavigation.GetHost(), userId);
        }

        private static void SetTableRelations(DataSet ds) {
            int index = 0;
            ds.Tables[index++].TableName = "User";

            Address.SetTableRelations(ds, index);
            ds.Relations.Add(Constant.DBRelation.USER_ADDRESS, ds.Tables["User"].Columns["UserId"], ds.Tables["Address"].Columns["UserId"]);
        }

        public static List<User> Search(string keywords, DateTime from, DateTime to, int rowsPerPage, int rowIndex, string sortBy) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_Search");
            db.AddInParameter(cmd, "@keywords", DbType.String, Utils.ConvertToFullTextSearchString(keywords));
            db.AddInParameter(cmd, "@from", DbType.DateTime, from);
            db.AddInParameter(cmd, "@to", DbType.DateTime, to);
            db.AddInParameter(cmd, "@rowsPerPage", DbType.Int32, rowsPerPage);
            db.AddInParameter(cmd, "@rowIndex", DbType.Int32, rowIndex);

            if (!string.IsNullOrEmpty(sortBy)) {
                bool asc = true;
                if (sortBy.EndsWith(" DESC")) {
                    sortBy = sortBy.Substring(0, sortBy.Length - 5);
                    asc = false;
                }
                db.AddInParameter(cmd, "@orderBy", DbType.String, sortBy);
                db.AddInParameter(cmd, "@asc", DbType.Boolean, asc);
            }
            db.AddOutParameter(cmd, "@totalUsers", DbType.Int32, sizeof(Int32));

            DataSet ds = db.ExecuteDataSet(cmd);
            int totalNumber = (int)db.GetParameterValue(cmd, "@totalUsers");

            SetTableRelations(ds);

            RecordList<User> users = new RecordList<User>(ds.Tables["User"]);

            if (null != HttpContext.Current)
                HttpContext.Current.Items[GetUserCountCacheKey(keywords, from, to)] = totalNumber;

            return users;
        }

        public static int GetUserCount(string keywords, DateTime from, DateTime to) {
            int count = 0;
            if ((null != HttpContext.Current) &&
                (null != HttpContext.Current.Items[GetUserCountCacheKey(keywords, from, to)]))
                count = (int)HttpContext.Current.Items[GetUserCountCacheKey(keywords, from, to)];
            return count;
        }

        private static string GetUserCountCacheKey(string keywords, DateTime from, DateTime to) {
            return string.Format("UserCount_{0}_{1}_{2}_{3}", SiteNavigation.GetHost(), keywords, from, to);
        }

        public static void UpdateUserOptional(string userName, bool newsLetter) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_Update_Optional", userName, newsLetter);
            db.ExecuteNonQuery(cmd);
        }

        public static void UpdateUserBillingAddress(string userName, string firstName, string lastName, string companyName, string address,
            string address2, string city, string state, int countryId, string zipcode, string phone, string phoneExt) {
            Database db = DBController.CreateDatabase();
            DbCommand cmd = db.GetStoredProcCommand("DSC_User_Update_BillingAddress", userName, firstName, lastName, companyName, address,
                address2, city, state, zipcode, countryId, phone, phoneExt);
            db.ExecuteNonQuery(cmd);
        }
    }
}
