//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Web.Caching;
using XShan.DotShoppingCart.Common.Configuration;

namespace XShan.DotShoppingCart.Common {
    using Object;

    public static class Utils {
        public static string GetParameter(string param, string defaultValue) {
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (null != stringValue) {
                return stringValue;
            } else {
                return defaultValue;
            }
        }

        public static string GetParameter(string param) {
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (null == stringValue) {
                throw new ApplicationException(string.Format("{0} is required parameter", param));
            }
            return stringValue;
        }

        public static bool CheckParameterExist(string param) {
            return null != HttpContext.Current.Request.QueryString[param];
        }

        public static int GetIntParameter(string param, int defaultValue) {
            int value = defaultValue;
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (!string.IsNullOrEmpty(stringValue)) {
                int.TryParse(stringValue, out value);
            }
            return value;
        }

        public static int GetIntParameter(string param) {
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (string.IsNullOrEmpty(stringValue)) {
                throw new ApplicationException(string.Format("{0} is required parameter", param));
            }
            return int.Parse(stringValue);
        }

        public static DateTime GetDateTimeParameter(string param, DateTime defaultValue) {
            DateTime value = defaultValue;
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (!string.IsNullOrEmpty(stringValue)) {
                DateTime.TryParse(HttpUtility.UrlDecode(stringValue), out value);
            }
            return value;
        }

        public static decimal? GetNullableDecimalParameter(string param) {
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (string.IsNullOrEmpty(stringValue)) {
                return null;
            }
            decimal value;
            if (decimal.TryParse(stringValue, out value))
                return value;
            return null;
        }

        public static int? GetNullableIntParameter(string param) {
            string stringValue = HttpContext.Current.Request.QueryString[param];
            if (string.IsNullOrEmpty(stringValue)) {
                return null;
            }
            int value;
            if (int.TryParse(stringValue, out value))
                return value;
            return null;
        }

        public static string StringChangeEmptyToNull(string param) {
            return string.IsNullOrEmpty(param) ? null : param;
        }

        public static T ConvertNullableFromString<T>(string param) {
            NullableConverter converter = new NullableConverter(typeof(T));
            return (T)converter.ConvertFromString(param);
        }

        public static int ChangeBadConversionToDefault(string param, int defaultValue) {
            int result = defaultValue;
            int.TryParse(param, out result);
            return result;
        }

        public static decimal ChangeBadConversionToDefault(string param, decimal defaultValue) {
            decimal result = defaultValue;
            decimal.TryParse(param, out result);
            return result;
        }

        public static int[] ParseIds(string idsString) {
            return ParseIds(idsString, Constant.DB.ListSeparator);
        }

        public static int[] ParseIds(string idsString, params char[] separator) {
            string[] ids = idsString.Split(separator);
            int[] idsConverted = new int[ids.Length];
            for (int i = 0; i < ids.Length; i++) {
                idsConverted[i] = int.Parse(ids[i]);
            }
            return idsConverted;
        }

        public static string ToDBNullString(string param) {
            if (string.IsNullOrEmpty(param))
                return Constant.DB.NullString;
            return param;
        }
        public static string ToDBNullString(object param) {
            if (null == param)
                return Constant.DB.NullString;
            return param.ToString();
        }

        public static string EscapeSeparators(string param) {
            return EscapeSeparators(param, '\\', Constant.DB.ListSeparator, Constant.DB.FieldSeparator);
        }

        public static string EscapeSeparators(string param, params char[] separators) {
            if (string.IsNullOrEmpty(param))
                return param;
            foreach (char c in separators) {
                param = param.Replace(c.ToString(), string.Format(@"\{0}", c));
            }
            return param;
        }

        public class NameValueContainer {
            private string name, value;

            public string Name {
                get { return name; }
            }
            public string Value {
                get { return value; }
            }

            public NameValueContainer(string name, string value) {
                this.name = name;
                this.value = value;
            }
        }

        public static List<NameValueContainer> GetNumbers(int from, int to) {
            List<NameValueContainer> numbers = new List<NameValueContainer>();
            for (int i = from; i <= to; i++) {
                numbers.Add(new NameValueContainer(i.ToString(), i.ToString()));
            }
            return numbers;
        }

        public static string GetUserName() {
            if (HttpContext.Current.User.Identity.IsAuthenticated) {
                return HttpContext.Current.User.Identity.Name;
            }
            return HttpContext.Current.Profile.UserName;
        }

        public static string GetSorted(List<int> values) {
            SortedList<int, bool> sorted = new SortedList<int, bool>();
            foreach (int value in values) {
                sorted.Add(value, true);
            }
            StringBuilder sb = new StringBuilder();
            foreach (int i in sorted.Keys) {
                if (sb.Length > 0) 
                    sb.Append(Constant.DB.ListSeparator);
                sb.Append(i);
            }
            return sb.ToString();
        }

        public static void EnsureSSLEnabled() {
            if (SiteNavigation.IsTestHost())
                return;

            if (!SiteNavigation.UseSSL)
                return;

            if (!HttpContext.Current.Request.IsSecureConnection) {
                string url = HttpContext.Current.Request.Url.AbsoluteUri;
                HttpContext.Current.Response.Redirect(url.Replace("http://", "https://"));
            }
        }

        public static byte[] GetData(Stream fs) {
            byte[] data = new byte[fs.Length];
            fs.Read(data, 0, (int)fs.Length);
            return data;
        }

        public static List<Type> GetProviders(string searchPatch, Type providerType) {
            List<Type> providers = new List<Type>();
            string binPath = HttpContext.Current.Server.MapPath("~/bin");
            foreach (string fileName in Directory.GetFiles(binPath, searchPatch, SearchOption.TopDirectoryOnly)) {
                string assemblyName = Path.GetFileNameWithoutExtension(fileName);
                System.Reflection.Assembly assembly = System.Reflection.Assembly.Load(assemblyName);
                foreach (Type t in assembly.GetTypes()) {
                    if (t.IsSubclassOf(providerType)) {
                        providers.Add(t);
                    }
                }
            }
            return providers;
        }

        public static List<KeyValuePair<string, int>> GetEnumNameValueList(Type enumType) {
            List<KeyValuePair<string, int>> list = new List<KeyValuePair<string, int>>();
            foreach (int value in Enum.GetValues(enumType)) {
                list.Add(new KeyValuePair<string, int>(Enum.GetName(enumType, value), value));
            }
            return list;
        }

        public static string ConvertToFullTextSearchString(string search) {
            if (string.IsNullOrEmpty(search))
                return search;
            search = search.Replace("'", "").Replace("-", "").Replace(",",""); // disallow single quote and dash for security reason although sproc has another level of defense
            List<string> keywords = new List<string>();
            int firstQuotePosition = -1;
            int wordStartPostion = 0;
            for (int i = 0; i < search.Length; i++) {
                switch (search[i]) {
                    case '\"':
                        if (firstQuotePosition >= 0) {
                            keywords.Add(search.Substring(firstQuotePosition, i - firstQuotePosition + 1));
                            firstQuotePosition = -1;
                            wordStartPostion = i + 1;
                        } else {
                            //word break with quote
                            if (wordStartPostion < i)
                                keywords.Add(search.Substring(wordStartPostion, i - wordStartPostion));
                            firstQuotePosition = i;
                        }
                        break;
                    case ' ':
                        if (-1 == firstQuotePosition) {
                            //word break with space
                            if (wordStartPostion < i)
                                keywords.Add(search.Substring(wordStartPostion, i - wordStartPostion));
                            wordStartPostion = i + 1;
                        }
                        break;
                    default:
                        break;
                }
            }
            if (firstQuotePosition >= 0)
                keywords.Add(search.Substring(firstQuotePosition) + "\"");
            else if (wordStartPostion < search.Length)
                keywords.Add(search.Substring(wordStartPostion));
            return string.Join(",", keywords.ToArray());
        }

        public static string ConvertIdsToSting(List<int> ids) {
            StringBuilder sb = new StringBuilder();
            foreach (int id in ids) {
                if (sb.Length > 0)
                    sb.Append(Constant.DB.ListSeparator);
                sb.Append(id.ToString());
            }
            return sb.ToString();
        }

        public static string GetTemplatePath() {
            // The public edition is running orderpipeline inside web context.
            if (Constant.DSC.IsPublicEdition)
                return Path.Combine(DSCConfigurationSection.GetSection().DSC.Get("InstallPath").Value, "App_Data");

            return GetInstallPath();
        }

        public static string GetInstallPath() {
            string[] args = Environment.GetCommandLineArgs();
            string path = Path.GetDirectoryName(args[0]);
            if (string.IsNullOrEmpty(path))
                return Directory.GetCurrentDirectory();
            return path;
        }

        public static string GetAppName() {
            string[] args = Environment.GetCommandLineArgs();
            return (Path.GetFileNameWithoutExtension(args[0]));
        }

        public static string GetAppDataPath(string host) {
            HttpContext ctx = HttpContext.Current;
            if (null == ctx) {
                string installPath = Constant.DSC.IsPublicEdition ?
                    DSCConfigurationSection.GetSection().DSC.Get("InstallPath").Value : GetInstallPath();
                return Path.Combine(Path.Combine(installPath, "App_Data"), host);
            } else {
                return Path.Combine(HttpContext.Current.Server.MapPath("~/App_Data"), host);
            }
        }

        public static Cache Cache {
            get {
                HttpContext ctx = HttpContext.Current;
                if (null == ctx)
                    return HttpRuntime.Cache;
                else
                    return ctx.Cache;
            }
        }

        public static NameValueCollection GetStoreTokens() {
            return GetStoreTokens(SiteNavigation.GetHost());
        }

        public static NameValueCollection GetStoreTokens(string host) {
            StoreConfigurationSection storeSection = (StoreConfigurationSection)SqlConfigurationController.GetSection(host, StoreConfigurationSection.SectionName);
            UIConfigurationSection uiSection = (UIConfigurationSection)SqlConfigurationController.GetSection(host, UIConfigurationSection.SectionName);
            return GetStoreTokens(storeSection, uiSection);
        }


        public static NameValueCollection GetStoreTokens(StoreConfigurationSection storeSection, UIConfigurationSection uiSection) {
            NameValueCollection tokens = new NameValueCollection();
            tokens.Add("Store.Name", storeSection.Name);
            tokens.Add("Store.URL", storeSection.URL);
            tokens.Add("Store.Logo", string.Format("{0}/dbfile.axd?name={1}", storeSection.URL, uiSection.LogoFilename));
            return tokens;
        }

        public static string ReplaceTokens(string template, NameValueCollection tokens) {
            foreach (string token in tokens.Keys) {
                string replacement = tokens[token];
                template = template.Replace(string.Format(@"%{0}%", token), replacement);
            }
            return template;
        }

        public static string ReplaceSKUTokens(string host, string content) {
            NameValueCollection tokens = new NameValueCollection();
            Dictionary<string, bool> names = new Dictionary<string,bool>();
            foreach (Match m in Regex.Matches(content, "%SKU:([^%]+)%", RegexOptions.Singleline)) {
                string name = m.Groups[0].Value.Replace("%", "");
                if (!names.ContainsKey(name)) {
                    tokens.Add(name, Product.GetProductSummaryHtml(host, m.Groups[1].Value));
                    names.Add(name, true);
                }
            }
            return ReplaceTokens(content, tokens);
        }

        public static bool IsAdmin() {
            return HttpContext.Current.User.IsInRole("Administrator");
        }

        public static bool IsProductionServer() {
            string site = SiteNavigation.GetHost();
            return (site != "localhost") && (site != "dual");
        }

        public static string FormatTimeSpan(TimeSpan ts) {
            return string.Format("{0}{1:00}:{2:00}:{3:00}", ts.Days == 0 ? "" : ts.Days.ToString() + ".", ts.Hours, ts.Minutes, ts.Seconds);
        }

        public static bool IsValidateDBTime(DateTime time) {
            return (Constant.DB.DBDateTimeMinValue < time) && (time < Constant.DB.DBDateTimeMaxValue);
        }

        public static string FormatPhoneForDisplay(string phone) {
            if (string.IsNullOrEmpty(phone))
                return phone;
            // remove non digits
            phone = Regex.Replace(phone, @"[^\d]", "");
            //change to the mask format (999)999-9999999
            phone = phone.Insert(0, "(");
            if (phone.Length > 4) {
                phone = phone.Insert(4, ")");
                if (phone.Length > 8) {
                    phone = phone.Insert(8, "-");
                }
            }
            return phone;
        }

        public static byte[] GetBitsFromSteam(Stream fs, int size) {
            byte[] img = new byte[size];
            fs.Read(img, 0, size);
            return img;
        }

        public static List<string> GetAllOrderPipelines() {
            List<string> orderPipelines = new List<string>();
            foreach (CheckoutPaymentTypeEnum checkoutPaymentType in Enum.GetValues(typeof(CheckoutPaymentTypeEnum)))
                orderPipelines.Add(checkoutPaymentType.ToString());
            orderPipelines.Add(Constant.OrderPipeline.RecurringOrderPipeline);

            return orderPipelines;
        }

        public static void EnsurePathExist(string path) {
            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
        }
    }
}
