//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.Caching;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Text.RegularExpressions;
using System.IO;
using System.Text;
using System.Configuration;
using System.Reflection;
using System.Xml;

using XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;

namespace XShan.DotShoppingCart.Common {
    public static class WebUtils {
        private static Dictionary<string, string> knownMimes;

        static WebUtils() {
            knownMimes = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase);
            knownMimes.Add(".jpg", "image/pjpeg");
            knownMimes.Add(".png", "image/x-png");
            knownMimes.Add(".gif", "image/gif");
            knownMimes.Add(".ico", "image/x-icon");
            knownMimes.Add(".bmp", "image/bmp");
            knownMimes.Add(".wmv", "video/x-ms-wmv");
        }

        public static string GetKnownMime(string filespec) {
            string extension = Path.GetExtension(filespec);
            if (knownMimes.ContainsKey(extension))
                return knownMimes[extension];
            return null;
        }
        
        public static void SetDropDownListSelected(DropDownList ddl, string selectedValue) {
            foreach (ListItem item in ddl.Items) {
                if (item.Selected)
                    item.Selected = false;
                if (item.Value == selectedValue) {
                    item.Selected = true;
                    break;
                }
            }
        }

        public static string GetUniqueKey(UserControl control) {
            if (null == control)
                return HttpContext.Current.Request.FilePath;
            return string.Format("{0}_{1}", HttpContext.Current.Request.FilePath, control.ClientID);
        }

        public static string GetCookie(UserControl control, string key) {
            HttpCookie cookie = HttpContext.Current.Request.Cookies[GetUniqueKey(control)];
            if (null != cookie)
                return cookie[key];
            return null;
        }

        public static void SetCookie(UserControl control, string key, string value) {
            HttpContext.Current.Response.Cookies[GetUniqueKey(control)][key] = value;
        }

        public static void ReloadPage() {
            HttpContext.Current.Response.Redirect(HttpContext.Current.Request.Url.AbsoluteUri);
        }

        public static bool IsCurrentPage(string pageUrl) {
            return HttpContext.Current.Request.Url.AbsoluteUri.StartsWith(pageUrl, StringComparison.OrdinalIgnoreCase);
        }


        public static void WriteDiscountGroupToSession(string userName) {
            //set discount group status into session
            decimal discountRate = 0;
            decimal minOrderAmount = 0;
            foreach (string roleName in Roles.GetRolesForUser(userName)) {
                DSCRole role = DSCRole.Get(roleName);
                if (role.DiscountRate > discountRate) {
                    discountRate = role.DiscountRate;
                    minOrderAmount = role.MinOrderAmount;
                }
            }
            HttpContext.Current.Session["discountRate"] = discountRate;
            HttpContext.Current.Session["minOrderAmount"] = minOrderAmount;
        }

        public static void WriteDiscountGroupToSessionByRole(DSCRole role) {
            HttpContext.Current.Session["discountRate"] = role.DiscountRate;
            HttpContext.Current.Session["minOrderAmount"] = role.MinOrderAmount;
        }

        public static void ClearDiscountGroupFromSession() {
            HttpContext.Current.Session["discountRate"] = null;
            HttpContext.Current.Session["minOrderAmount"] = null;
        }

        public static bool IsDiscountGroup() {
            return GetDiscountRate() > 0;
        }

        public static decimal GetDiscountPrice(decimal price) {
            return price * (100 - WebUtils.GetDiscountRate()) / 100;
        }

        public static decimal GetDiscountRate() {
            if (null == HttpContext.Current.Session["discountRate"])
                HttpContext.Current.Session["discountRate"] = (decimal)0;
            return (decimal)HttpContext.Current.Session["discountRate"];
        }

        public static decimal GetMinOrderAmount() {
            if (null == HttpContext.Current.Session["minOrderAmount"])
                HttpContext.Current.Session["minOrderAmount"] = (decimal)0;
            return (decimal)HttpContext.Current.Session["minOrderAmount"];
        }

        public static void HandleExceptionAndContinue(Exception ex) {
            //handle exception here
            ex.Data.Add("Url", HttpContext.Current.Request.Url.AbsoluteUri);
            ex.Data.Add("RawUrl", HttpContext.Current.Request.RawUrl);
            ex.Data.Add("UseAgent", HttpContext.Current.Request.UserAgent);
            ex.Data.Add("UrlReferrer", HttpContext.Current.Request.UrlReferrer);
            ex.Data.Add("IPAddress", HttpContext.Current.Request.UserHostAddress);
            ExceptionPolicy.HandleException(ex, "Unhandled Policy");
        }

        public static bool IsPageEditorEnable() {
            if (null == HttpContext.Current.Session[Constant.UI.SessionKeyEnablePageEditor])
                return false;
            return (bool)HttpContext.Current.Session[Constant.UI.SessionKeyEnablePageEditor];
        }

        public static void SetPageEditor(bool enable) {
            HttpContext.Current.Session[Constant.UI.SessionKeyEnablePageEditor] = enable;
        }

        public static void AddDSCCommerJS(Page page) {
            HtmlGenericControl include = new HtmlGenericControl("script");
            include.Attributes.Add("type", "text/javascript");
            include.Attributes.Add("src", SiteNavigation.CommonJSFile);
            page.Header.Controls.Add(include);
        }

        public static void CacheItemRemovedCallback(String key, object value, CacheItemRemovedReason removedReason) {
            LogEntry log = new LogEntry();
            log.Message = string.Format("Cache key ({0}) was removed due to {1}", key, removedReason.ToString());
            log.Categories.Add("CacheRemove");
            Logger.Write(log);
        }

        public static void ValidateCustomPagePermission() {
            if (SiteNavigation.GetHost() == "localhost")
                return;

            string customHost = ConfigurationManager.AppSettings["customHost"];
            if (string.IsNullOrEmpty(customHost) ||
                (!SiteNavigation.GetHost().Equals(customHost, StringComparison.OrdinalIgnoreCase)))
                throw new HttpException(404, "Unable to find the requested content page");
        }

        public static void SetCacheable(HttpCachePolicy cache) {
            cache.SetExpires(DateTime.Now.AddMinutes(Constant.Cache.expireInMinutes));
            cache.SetCacheability(HttpCacheability.Public);
            cache.SetValidUntilExpires(true); 
        }

        public static string GetImagePath() {
            string imagePath = HttpContext.Current.Request.PhysicalApplicationPath + Path.Combine(Path.Combine(DbImage.ImageDirectory, SiteNavigation.GetHost()), "Dynamic");

            if (!Directory.Exists(imagePath))
                Directory.CreateDirectory(imagePath);
            return imagePath;
        }

        public static string GetThemesTemplateRoot() {
            return HttpContext.Current.Request.PhysicalApplicationPath + Path.Combine("App_Data", "Themes_Template");
        }

        public static string GetThemesTemplatePath(string theme) {
            return Path.Combine(GetThemesTemplateRoot(), theme);
        }

        public static string RenderControl(string path, string propertyName, object propertyValue) {
            Page pageHolder = new Page();
            UserControl viewControl = (UserControl)pageHolder.LoadControl(path);

            if (null != propertyValue) {
                Type viewControlType = viewControl.GetType();
                PropertyInfo property = viewControlType.GetProperty(propertyName);

                if (null != property) {
                    property.SetValue(viewControl, propertyValue, null);
                } else {
                    throw new ApplicationException(string.Format("Control {0} does not have {1} property", path, propertyValue));
                }
            }

            pageHolder.Controls.Add(viewControl);

            StringWriter output = new StringWriter();
            HttpContext.Current.Server.Execute(pageHolder, output, false);

            return output.ToString();
        }

        private static string RenderControl(Control control) {
            StringBuilder htmlBuilder = new StringBuilder();
            using (StringWriter stringWriter = new StringWriter(htmlBuilder)) {
                using (HtmlTextWriter writer = new HtmlTextWriter(stringWriter)) {
                    control.RenderControl(writer);
                }
            }
            return htmlBuilder.ToString();
        }

        public static XmlDocument LoadThemeConfigXml(string theme) {
            string themeConfigfile = Path.Combine(GetThemesTemplatePath(theme), "config.xml");
            XmlDocument themeConfig = new XmlDocument();
            themeConfig.Load(themeConfigfile);
            return themeConfig;
        }
    
        public static string GetThemeThumbnailImageUrl(string theme, string filename) {
            return SiteNavigation.ResolveUrl(string.Format("Images/site_themes/{0}/{1}", theme, filename));
        }
    }
}
