//------------------------------------------------------------------------------
// The contents of this file are subject to the DotShoppingCart Public License Version 1.0 ("License"); you may not use this file except in compliance with the License.
// You may obtain a copy of the License at  http://www.dotshoppingcart.com/Page/license.aspx. 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
// See the License for the specific language governing rights and limitations under the License.
// 
// The Original Code is DotShoppingCart.
// The Initial Developer of the Original Code is XShan Corp.
// All Rights Reserved.
// 
// Contributor(s): _______. 
//------------------------------------------------------------------------------

using System;
using System.Data;
using System.Configuration;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;

using GCheckout;
using GCheckout.AutoGen;
using GCheckout.Util;
using DSC = XShan.DotShoppingCart.Common.Object;
using XShan.DotShoppingCart.Common;

public partial class GoogleCheckoutPostHandler : System.Web.UI.Page {
    private void ParseFirstLastNames(string fullname, out string firstname, out string lastname) {
        string[] parts = fullname.Split(' ');
        if (parts.Length > 1) {
            lastname = parts[parts.Length - 1];
            firstname = string.Join(" ", parts, 0, parts.Length - 1);
        } else {
            lastname = fullname;
            firstname = string.Empty;
        }
    }

    private DSC.Address ConvertToDSCAddress(Address address) {
        string firstname, lastname;
        ParseFirstLastNames(address.contactname, out firstname, out lastname);
        DSC.Address a = new DSC.Address();
        a.FirstName = firstname;
        a.LastName = lastname;
        a.CompanyName = address.companyname;
        a.Address1 = address.address1;
        a.Address2 = address.address2;
        a.City = address.city;
        a.State = address.region;
        a.Zipcode = address.postalcode;
        a.Phone = address.phone;
        a.Country = DSC.Country.GetFromCodeISO3166(address.countrycode);
        return a;
    }

    protected void Page_Load(object sender, EventArgs e) {
        string requestXml;

        int nid = Utils.GetIntParameter("nid", 0);
        if (nid > 0) { //process notification from db
            requestXml = DSC.GoogleCheckoutNotification.Get(nid).Message;
        } else {
            // Extract the XML from the request.
            Stream RequestStream = Request.InputStream;
            StreamReader RequestStreamReader = new StreamReader(RequestStream);
            requestXml = RequestStreamReader.ReadToEnd();
            RequestStream.Close();
            if (string.IsNullOrEmpty(requestXml))
                return;
            DSC.GoogleCheckoutNotification.Add(requestXml);
        }
        ProcessNotification(requestXml);
    }

    private void ProcessNotification(string requestXml) {
        DSC.Order order;
        DSC.PaymentTransaction trans;
        switch (EncodeHelper.GetTopElement(requestXml)) {
            case "new-order-notification":
                NewOrderNotification newOrder = (NewOrderNotification)EncodeHelper.Deserialize(requestXml, typeof(NewOrderNotification));

                string userName = newOrder.shoppingcart.merchantprivatedata.Any[0].InnerText;
                DSC.Address shippingAddress = ConvertToDSCAddress(newOrder.buyershippingaddress);
                DSC.Address billingAddress = ConvertToDSCAddress(newOrder.buyerbillingaddress);

                decimal subTotal = 0;
                string couponCode = string.Empty, giftWrapMessage = string.Empty;
                Dictionary<string, decimal> gcs = new Dictionary<string,decimal>(StringComparer.OrdinalIgnoreCase);
                
                List<DSC.CartItem> cartItems = new List<DSC.CartItem>();
                foreach (Item item in newOrder.shoppingcart.items) {
                    string sku = item.merchantprivateitemdata.Any[0].InnerText;
                    string attributeIds = item.merchantprivateitemdata.Any[1].InnerText;
                    int quantity = item.quantity;
                    cartItems.Add(new DSC.CartItem(sku, attributeIds, quantity));
                    decimal price = item.unitprice.Value;
                    subTotal += price * quantity;
                }

                MerchantCalculatedShippingAdjustment shippingAdjustment = (MerchantCalculatedShippingAdjustment)newOrder.orderadjustment.shipping.Item;
                List<DSC.OrderTotal> orderTotals = new List<DSC.OrderTotal>();
                orderTotals.Add(new DSC.OrderTotal("Subtotal", subTotal));
                orderTotals.Add(new DSC.OrderTotal("Shipping", shippingAdjustment.shippingcost.Value, shippingAdjustment.shippingname));
                orderTotals.Add(new DSC.OrderTotal("Tax", newOrder.orderadjustment.totaltax.Value));
                orderTotals.Add(new DSC.OrderTotal("Order Total", newOrder.ordertotal.Value));

                string orderNumber = DSC.Order.Add(userName, newOrder.buyershippingaddress.email, shippingAdjustment.shippingname, shippingAddress, billingAddress,
                    DSC.CheckoutPaymentTypeEnum.GoogleCheckout, null, orderTotals, cartItems, newOrder.googleordernumber, Constant.DB.OrderStatusGoogleCheckoutAuthorization); 
                // 10 is Awaiting Google Checkout Authorization. This is the initial state of google checkout order.
                break;
            case "order-state-change-notification":
                OrderStateChangeNotification stateChange = (OrderStateChangeNotification)EncodeHelper.Deserialize(requestXml, typeof(OrderStateChangeNotification));
                // The order has changed either financial or fulfillment state in Google's system.
                order = DSC.Order.GetByCustomField(stateChange.googleordernumber);
                string NewFinanceState = stateChange.newfinancialorderstate.ToString();
                switch (stateChange.newfinancialorderstate) {
                    case FinancialOrderState.CHARGEABLE:
                        //use risk information to create authorizaiton transaction
                        break;
                    case FinancialOrderState.CHARGED:
                        //use charge amount to create capture transaction
                        break;
                    case FinancialOrderState.PAYMENT_DECLINED:
                    case FinancialOrderState.CANCELLED:
                    case FinancialOrderState.CANCELLED_BY_GOOGLE:
                        DSC.Order.SetStatus(order.OrderId, Constant.DB.OrderStatusUnderReview, 
                            string.Format("Received {0} from Google (reason: {1})",
                                stateChange.newfinancialorderstate.ToString(), stateChange.reason));
                        break;
                }
                break;
            case "risk-information-notification":
                RiskInformationNotification risk = (RiskInformationNotification)EncodeHelper.Deserialize(requestXml, typeof(RiskInformationNotification));
                // This notification tells us that Google has authorized the order and it has passed the fraud check.
                // Use the data below to determine if you want to accept the order, then start processing it.
                order = DSC.Order.GetByCustomField(risk.googleordernumber);
                bool SellerProtection = risk.riskinformation.eligibleforprotection;
                
                trans = new DSC.PaymentTransaction();
                trans.PaymentGatewayType = DSC.PaymentGatewayType.Google_Checkout;
                trans.PaymentTransactionType = DSC.PaymentTransactionType.Authorize;
                trans.Amount = order.OrderTotalAmount;
                trans.TransactionDate = risk.timestamp.ToLocalTime();
                trans.AVS = risk.riskinformation.avsresponse;
                trans.CVV = risk.riskinformation.cvnresponse;
                trans.Success = true;
                trans.AuthorizationCode = string.Format("Last Four CC {0}. SellerProtection: {1}", risk.riskinformation.partialccnumber, risk.riskinformation.eligibleforprotection);
                trans.Add(order.OrderNumber);
                order.SetToNextStatusInPipeline(string.Format("Received risk information from Google. Serial#: {0}, User IP address: {1}", 
                    risk.serialnumber, risk.riskinformation.ipaddress));
                break;
            case "charge-amount-notification":
                ChargeAmountNotification chargeAmount = (ChargeAmountNotification)EncodeHelper.Deserialize(requestXml, typeof(ChargeAmountNotification));
                // Google has successfully charged the customer's credit card.
                order = DSC.Order.GetByCustomField(chargeAmount.googleordernumber);
                trans = new DSC.PaymentTransaction();
                trans.PaymentGatewayType = DSC.PaymentGatewayType.Google_Checkout;
                trans.PaymentTransactionType = DSC.PaymentTransactionType.Capture;
                trans.Amount = chargeAmount.latestchargeamount.Value;
                trans.TransactionDate = chargeAmount.timestamp.ToLocalTime();
                trans.Success = true;
                trans.AuthorizationCode = chargeAmount.serialnumber;
                trans.Add(order.OrderNumber);
                order.SetToNextStatusInPipeline(string.Format("Received successful charge from Google. Total Charge: {0}.",
                    chargeAmount.totalchargeamount.Value));
                break;
            /*
            case "refund-amount-notification":
                RefundAmountNotification N5 = (RefundAmountNotification)EncodeHelper.Deserialize(requestXml, typeof(RefundAmountNotification));
                // Google has successfully refunded the customer's credit card.
                string OrderNumber5 = N5.googleordernumber;
                decimal RefundedAmount = N5.latestrefundamount.Value;
                break;
            case "chargeback-amount-notification":
                ChargebackAmountNotification N6 = (ChargebackAmountNotification)EncodeHelper.Deserialize(requestXml, typeof(ChargebackAmountNotification));
                // A customer initiated a chargeback with his credit card company to get her money back.
                string OrderNumber6 = N6.googleordernumber;
                decimal ChargebackAmount = N6.latestchargebackamount.Value;
                break;
             * */
            default:
                break;
        }
    }
}
